@@ -1,9 +1,8 @@
 /*
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2004,2006 Oracle.  All rights reserved.
+ * Copyright (c) 2004,2009 Oracle.  All rights reserved.
  *
- * $Id: helloWorld.cpp,v 1.2 2006/10/30 17:45:43 bostic Exp $
  *
  *******
  *
@@ -22,6 +21,7 @@
  *
  */
 
+#include <iostream>
 #include <dbxml/DbXml.hpp>
 
 using namespace DbXml;
@@ -38,6 +38,7 @@ main(int argc, char **argv)
 	try {
 		// All BDB XML programs require an XmlManager instance
 		XmlManager mgr;
+		{
 		XmlContainer cont = mgr.createContainer(containerName);
 
 		// All Container modification operations need XmlUpdateContext
@@ -56,6 +57,7 @@ main(int argc, char **argv)
 
 		// In C++, resources are released as objects go out
 		// of scope.
+		}
 
 		
 	} catch (XmlException &xe) {
