@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2004,2006 Oracle.  All rights reserved.
+// Copyright (c) 2004,2009 Oracle.  All rights reserved.
 //
-// $Id: queryWithContext.java,v 1.15.2.1 2007/01/18 20:12:34 gmf Exp $
 //
 
 package dbxml.gettingStarted;
@@ -48,7 +47,6 @@ class queryWithContext
 	XmlValue value = results.next();
 	while (value != null) {
             System.out.println(value.asString());
-	    value.delete();
             value = results.next();
 	}
 	System.out.println(results.size() + " results returned for query '"
@@ -138,7 +136,6 @@ class queryWithContext
             doContextQuery(theMgr,
 			   "/desserts:item/product[.=$aDessert]", context);
 
-	    context.delete();
        } catch (Exception e) {
            System.err.println("Error performing query against " + theContainer );
            System.err.println("   Message: " + e.getMessage() );
