@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# $Id: examples.py,v 1.24 2006/10/20 21:50:04 gmf Exp $
 # As of Python 2.3, there is a built-in module for Berkeley DB,
 # called bsddb.  The next uncommented line assumes that module is being used.
 # Note the use of XmlException.  There are, as of BDB XML 2.3, a number
@@ -8,7 +7,6 @@
 # 
 from bsddb3.db import *
 from dbxml import *
-from _dbxml import *
 
 book_content = r"<book><title>Knowledge Discovery in Databases.</title></book>"
 book_content_ns = r"<book xmlns:books='http://foo.bar.com/books.dtd'><books:title>Knowledge Discovery in Databases.</books:title></book>"
@@ -44,9 +42,9 @@ def example01():
 	s = document.getContent()
 	print document.getName(), "=", s
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
     
 def example02():
     """Create an XmlManager/XmlContainer, add a document, query the container
@@ -70,9 +68,9 @@ def example02():
 	    document = value.asDocument()
 	    print document.getName(), "=", value.asString()
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
 
 def example03():
     """Create an XmlContainer, add a document that includes a
@@ -99,9 +97,9 @@ def example03():
 	    document = value.asDocument()
 	    print document.getName(), "=", value.asString()
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
 
 def example04():
     """Create an XmlContainer, define an equality string index
@@ -134,9 +132,9 @@ def example04():
 	    document = value.asDocument()
 	    print document.getName(), "=", value.asString()
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
 		
 def example05():
     """
@@ -174,9 +172,9 @@ def example05():
 	    document = value.asDocument()
 	    print document.getName(), "=", value.asString()
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
 
 def example06():
     """Create an XML Container, rename that container, delete the
@@ -196,9 +194,9 @@ def example06():
 	    del container
 	    mgr.removeContainer("test2.dbxml")
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
 
 def example07():
     """Create an use a transactional XML Container:
@@ -222,12 +220,15 @@ def example07():
 	document = container.getDocument(book_name)
 	s = document.getContent()
 	print document.getName(), "=", s
-	container.close()
+	del uc
+	del document
+	del container
+	#container.close()
 	mgr.removeContainer("test.dbxml")
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
     environment.close(0)
 
 def example08():
@@ -250,12 +251,14 @@ def example08():
 	document = container.getDocument(book_name)
 	s = document.getContent()
 	print document.getName(), "=", s
-	container.close()
+	del uc
+	del document
+	del container
 	mgr.removeContainer("test.dbxml")
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
     environment.close(0)
 
 def example09():
@@ -286,11 +289,12 @@ def example09():
 	xtxn.commit()
 	del document  # holds ref on container
 	del container # prevents removal
+	del uc
 	mgr.removeContainer("test.dbxml")
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
     environment.close(0)
 
 def example10():
@@ -314,14 +318,14 @@ def example10():
 	container1.putDocument(xtxn, book_name, book_content, uc)
 	container2.putDocument(xtxn, book_name, book_content, uc)
 	txn.commit(0)
-	
+	del uc	
 	del container1
 	del container2
 	del mgr
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
     environment.close(0)
 
 def example11():
@@ -370,9 +374,9 @@ def example11():
         #qe = mgr.prepare("collection('test.dbxml')/title[.<xs:date('1957-11-03')]", qc)
         #print qe.getQueryPlan()
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
     
 
 def example12():
@@ -391,9 +395,9 @@ def example12():
 	doc = container.getDocument(book_name)
 	print "document: ", doc.getContent()
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
 
 
 # Test example of an overloade XmlResolver instance in Python
@@ -432,7 +436,7 @@ def example13():
     >>> example13()
     XmlResolver constructor
     In resolveEntity, systemId: mydtd.dtd
-    In resolveDocument, uri: myscheme:/myscheme:xxx
+    In resolveDocument, uri: myscheme:/docname
     value of result:  <a>yyz</a>
     """
     
@@ -451,14 +455,14 @@ def example13():
 	# query to test the resolver
 	qc = mgr.createQueryContext()
 	qc.setBaseURI("myscheme:/")
-	results = mgr.query("doc('myscheme:xxx')/root/a", qc)
+	results = mgr.query("doc('docname')/root/a", qc)
 	for value in results:
 	    document = value.asDocument()
 	    print "value of result: ", value.asString()
     except XmlException, inst:
-	print "XmlException (", inst.ExceptionCode,"): ", inst.What
-	if inst.ExceptionCode == DATABASE_ERROR:
-	    print "Database error code:",inst.DBError
+	print "XmlException (", inst.exceptionCode,"): ", inst.what
+	if inst.exceptionCode == DATABASE_ERROR:
+	    print "Database error code:",inst.dbError
 
 NUMBER_OF_EXAMPLES = 13
 
