@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: XmlException.hpp,v 1.38 2006/10/30 17:45:48 bostic Exp $
 //
 
 #ifndef __XMLEXCEPTION_HPP
@@ -19,7 +18,6 @@
  * the enumerations here must be mirrored there.
  */
 
-class DbException;
 class XQException;
 class LocationInfo;
 
@@ -38,10 +36,10 @@ public:
 		NULL_POINTER,     ///< null pointer exception
 		INDEXER_PARSER_ERROR,  ///< XML Indexer could not parse a document.
 		DATABASE_ERROR,  ///< Berkeley DB reported a database problem.
-		XPATH_PARSER_ERROR,  ///< The XPath parser was unable to parse the XPath expression.
-		DOM_PARSER_ERROR,  ///< The DOM parser was unable to parse an XML document.
-		XPATH_EVALUATION_ERROR,  ///< The XPath evaluator was unable to execute the XPath expression.
-		NO_VARIABLE_BINDING,  ///< An XPath expression referred to variable without a value.
+		QUERY_PARSER_ERROR,  ///< The query parser was unable to parse the expression.
+		UNUSED1_ERROR,  ///< Unused
+		QUERY_EVALUATION_ERROR,  ///< The query evaluator was unable to execute the expression.
+		UNUSED2_ERROR,  ///< Unused
 		LAZY_EVALUATION,  ///< XmlResults is lazily evaluated.
 		DOCUMENT_NOT_FOUND,  ///< The specified document could not be found
 		CONTAINER_EXISTS,  ///< The container already exists.
@@ -56,9 +54,9 @@ public:
 		OPERATION_TIMEOUT, ///< An operation timed out
 		OPERATION_INTERRUPTED ///< An operation was explicitly interrupted
 	};
+	
 	XmlException(ExceptionCode err, const std::string &description, const char *file = 0, int line = 0);
 	XmlException(ExceptionCode err, const char *description, const char *file = 0, int line = 0);
-	explicit XmlException(const DbException &e, const char *file = 0, int line = 0);
 	XmlException(int dberr, const char *file = 0, int line = 0);
 	XmlException(const XmlException &);
 	virtual ~XmlException() throw();
