@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: XmlQueryExpression.hpp,v 1.36 2006/10/30 17:45:48 bostic Exp $
 //
 
 #ifndef __XPATHEXPRESSION_HPP
@@ -11,7 +10,7 @@
 
 #include "DbXmlFwd.hpp"
 #include "XmlPortability.hpp"
-#include <db_cxx.h>
+#include <db.h>
 #include <string>
 
 namespace DbXml
@@ -32,19 +31,20 @@ public:
 	/** @name Information Methods */
 	const std::string &getQuery() const;
 	std::string getQueryPlan() const;
-
+	bool isUpdateExpression() const;
+	
 	/** @name Execute Methods */
 
 	/**
 	 * Valid flags:
 	 * DB_READ_UNCOMMITTED, DB_RMW, DB_READ_COMMITTED, DBXML_LAZY_DOCS,
-	 * DB_TXN_SNAPSHOT
+	 * DB_TXN_SNAPSHOT, DBXML_DOCUMENT_PROJECTION, DBXML_NO_AUTO_COMMIT
 	 */
 	XmlResults execute(XmlQueryContext &context, u_int32_t flags = 0) const;
 	/**
 	 * Valid flags:
 	 * DB_READ_UNCOMMITTED, DB_RMW, DB_READ_COMMITTED, DBXML_LAZY_DOCS,
-	 * DB_TXN_SNAPSHOT
+	 * DB_TXN_SNAPSHOT, DBXML_DOCUMENT_PROJECTION, DBXML_NO_AUTO_COMMIT
 	 */
 	XmlResults execute(const XmlValue &contextItem, XmlQueryContext &context, u_int32_t flags = 0) const;
 
@@ -53,13 +53,13 @@ public:
 	/**
 	 * Valid flags:
 	 * DB_READ_UNCOMMITTED, DB_RMW, DB_READ_COMMITTED, DBXML_LAZY_DOCS,
-	 * DB_TXN_SNAPSHOT
+	 * DB_TXN_SNAPSHOT, DBXML_DOCUMENT_PROJECTION, DBXML_NO_AUTO_COMMIT
 	 */
 	XmlResults execute(XmlTransaction &txn, XmlQueryContext &context, u_int32_t flags = 0) const;
 	/**
 	 * Valid flags:
 	 * DB_READ_UNCOMMITTED, DB_RMW, DB_READ_COMMITTED, DBXML_LAZY_DOCS,
-	 * DB_TXN_SNAPSHOT
+	 * DB_TXN_SNAPSHOT, DBXML_DOCUMENT_PROJECTION, DBXML_NO_AUTO_COMMIT
 	 */
 	XmlResults execute(XmlTransaction &txn, const XmlValue &contextItem, XmlQueryContext &context, u_int32_t flags = 0) const;
 
