@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: XmlResults.hpp,v 1.40 2006/10/30 17:45:48 bostic Exp $
 //
 
 #ifndef __XMLRESULTS_HPP
@@ -64,6 +63,9 @@ public:
 
 	//@}
 
+	XmlEventWriter& asEventWriter();
+	XmlResults copyResults();
+	void concatResults(XmlResults &from);
 private:
 	Results *results_;
 };
