@@ -1,15 +1,14 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: Timer.hpp,v 1.3 2006/10/30 17:45:50 bostic Exp $
 //
 #ifndef __TIMER_H
 #define	__TIMER_H
 
 #include <string>
-#include <db_cxx.h>
+#include <db.h>
 
 namespace DbXml
 {
@@ -22,7 +21,7 @@ public:
 	virtual ~Timer();
 	void start();
 	void stop();
-	void duration(long *seconds, long *microseconds) const;
+	void duration(time_t *seconds, time_t *microseconds) const;
 	double durationInSeconds() const;
 	double durationInUsecs() const;
 	void reset();
@@ -36,8 +35,8 @@ private:
 	const char *what_;
 	const char *whats_;
 	long count_;
-	u_int32_t start_secs, start_usecs;
-	long duration_secs, duration_usecs;
+	time_t start_secs, start_usecs;
+	time_t duration_secs, duration_usecs;
 };
 
 }
