@@ -1,9 +1,8 @@
 /*
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000,2006 Oracle.  All rights reserved.
+ * Copyright (c) 2000,2009 Oracle.  All rights reserved.
  *
- * $Id: common_utils.h,v 1.4 2006/10/30 17:45:50 bostic Exp $
  */
 
 #ifndef COMMON_UTILS_H
@@ -15,6 +14,7 @@ extern "C" {
 	
 void dbxml_util_siginit();
 int dbxml_util_interrupted();
+void dbxml_util_reset_interrupted();
 void dbxml_util_sigresend();
 void dbxml_set_signal_listener(void (*listener)(int));
 
@@ -35,6 +35,7 @@ namespace DbXml {
 		SigBlock();
 		~SigBlock();
 		bool isInterrupted() const;
+    void reset();
 		void setListener(SignalListener *listener);
 	};
 }
