@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: BaseInputStream.cpp,v 1.10 2006/10/30 17:45:51 bostic Exp $
 //
 
 #include "BaseInputStream.hpp"
@@ -29,13 +28,13 @@ BaseInputStream::~BaseInputStream()
 
 unsigned int BaseInputStream::curPos() const
 {
-	return getBinInputStream()->curPos();
+	return (unsigned int)getBinInputStream()->curPos();
 }
 
 unsigned int BaseInputStream::readBytes(char *toFill, 
 					const unsigned int maxToRead)
 {
-	return getBinInputStream()->readBytes((XMLByte*)toFill, maxToRead);
+	return (unsigned int) getBinInputStream()->readBytes((XMLByte*)toFill, maxToRead);
 }
 
 XERCES_CPP_NAMESPACE_QUALIFIER BinInputStream *BaseInputStream::getBinInputStream() const
