@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: ConfigurationDatabase.cpp,v 1.35.2.1 2007/01/09 22:32:49 gmf Exp $
 //
 
 #include "DbXmlInternal.hpp"
@@ -17,6 +16,8 @@
 #include "Buffer.hpp"
 #include "Container.hpp"
 #include "TypeConversions.hpp"
+#include "ScopedDbt.hpp"
+#include "Transaction.hpp"
 
 using namespace DbXml;
 using namespace std;
@@ -38,89 +39,83 @@ static const char *sequence_name = "sequence";
 // The cache value is set to 5 (see code below).  Cached sequence numbers
 // are never returned, so it's possible to skip unused numbers from the cache.
 //
-// TBD: eventually, the simplest solution to limits is to make Document IDs
-// 64-bit numbers (db_seq_t).
-//
 
-static DbSequence * initDbSequence(Transaction *txn, Db &db, u_int32_t incr,
-				   u_int32_t flags)
+static DB_SEQUENCE * initDbSequence(Transaction *txn, DB *db, u_int32_t incr,
+				    u_int32_t flags)
 {
-	Dbt seqKey((void*)"dbxmlID", 7);
+	DbXml::DbXmlDbt seqKey((void*)"dbxmlID", 7);
 	static db_seq_t initValue = 2;
 	static db_seq_t maxValue = (db_seq_t)(unsigned int)0xffffffff;
-	DbTxn *dbtxn = 0;
+	DB_TXN *dbtxn = 0;
 	size_t cacheSize = (txn ? 0 : incr);
+	DB_SEQUENCE *seq;
 
 	if (txn)
-		dbtxn = txn->getDbTxn();
+		dbtxn = txn->toDB_TXN(txn);
 
 	// always set DB_CREATE and DB_THREAD
-	flags &= (DB_TXN_NOSYNC);
+	flags &= ~(DB_TXN_NOSYNC);
 	flags |= DB_CREATE|DB_THREAD;
-	DbSequence *seq =  new DbSequence(&db, 0);
-	// no-op if already initialized
-	seq->initial_value(initValue);
-	seq->set_range(initValue, maxValue);
-	seq->set_cachesize(cacheSize);
-	int err = seq->open(dbtxn, &seqKey, flags);
+	int err = db_sequence_create(&seq, db, 0);
 	if (err)
-		throw XmlException(XmlException::INTERNAL_ERROR,
-				   "Could not create DbSequence object");
+		throw XmlException(err);
+
+	// no-op if already initialized
+	seq->initial_value(seq, initValue);
+	seq->set_range(seq, initValue, maxValue);
+	seq->set_cachesize(seq, (int32_t)cacheSize);
+	err = seq->open(seq, dbtxn, &seqKey, flags);
+	if (err) {
+		seq->close(seq, 0);
+		throw XmlException(err);
+	}
 	return seq;
 }
 
 ConfigurationDatabase::ConfigurationDatabase(
-	DbEnv *env, Transaction *txn, const std::string &name,
-	XmlContainer::ContainerType &type,
-	u_int32_t pageSize, u_int32_t seqIncr, u_int32_t flags, int mode,
+	DB_ENV *env, Transaction *txn, const std::string &name,
+	ContainerConfig &config,
 	bool &indexNodes, bool doVersionCheck)
 	: environment_(env),
 	  name_(name),
 	  database_(env, name, "secondary_", configuration_name,
-		    pageSize, flags & DB_XA_CREATE),
+		    DEFAULT_CONFIG),
 	  seqDatabase_(env, name, "secondary_", sequence_name,
-		       pageSize, flags & DB_XA_CREATE),
+		       DEFAULT_CONFIG),
 	  seq_(0),
-	  seqIncr_(seqIncr),
-	  creatingTxn_(txn)
+	  seqIncr_(config.getSequenceIncrement())
 {
-	flags &= ~DB_XA_CREATE;
 	int err = 0;
 	try {
-		bool rdonly = (flags & DB_RDONLY) != 0;
+		bool rdonly = config.getReadOnly();
 
 		// Open the Db objects
-		err = database_.open(txn, DB_BTREE, flags, mode);
+		err = database_.open(txn, DB_BTREE, config);
 		// Check version before attempting to open sequence DB,
 		// which will fail for pre-2.2 containers
 		if (err == 0 && doVersionCheck)
 			err = checkVersion(txn, rdonly);
-		if (err == 0)
-			err = seqDatabase_.open(txn, DB_BTREE, flags, mode);
-
-		if (err == 0)
-			seq_ = initDbSequence(txn, seqDatabase_.getDb(),
-					      seqIncr_, flags);
-
-		// Add the default index specification, if there
-		// isn't one already
-		if(err == 0 && !rdonly)
-			err = checkIndexSpecification(txn);
-
+		if (err == 0 && !rdonly) {
+			err = seqDatabase_.open(txn, DB_BTREE, config);
+			if (err == 0) {
+				seq_ = initDbSequence(
+					txn, seqDatabase_.getDb(),
+					seqIncr_, config.getSeqFlags());
+				// Add the default index specification
+				err = checkIndexSpecification(txn);
+			}
+		}
 		// Add, or set the container type.  If this is
 		// a create, it won't be there. "type" is by ref
 		if(err == 0)
-			type = checkContainerType(txn, database_, type, rdonly);
+			config.setContainerType(checkContainerType(txn, database_, config.getContainerType(), rdonly));
 
 		// Add indexNodes, if it's not there already, otherwise
 		// retrieve it.
 		if(err == 0)
 			indexNodes = checkIndexNodes(txn, indexNodes, rdonly);
-	} catch (DbException &e) {
-		err = e.get_errno();
-		// fall through
-	}
-	catch (XmlException &) {
+
+	} catch (XmlException &) {
 		if (txn)
 			txn->abort();
 		throw;
@@ -145,44 +140,23 @@ ConfigurationDatabase::ConfigurationDatabase(
 
 ConfigurationDatabase::~ConfigurationDatabase()
 {
-	if (seq_) {
-		seq_->close(0);
-		delete seq_;
-	}
+	if (seq_)
+		seq_->close(seq_, 0);
 }
 
 int ConfigurationDatabase::generateID(Transaction *txn, DocID &newId)
 {
-	int err = 0;
 	db_seq_t newValue = 0;
 	u_int32_t flags = 0;
-	DbTxn *dbtxn = 0;
-	if (txn) {
-		if (txn == creatingTxn_) {
-			dbtxn = txn->getDbTxn();
-		} else {
-			flags |= DB_AUTO_COMMIT|DB_TXN_NOSYNC;
-		}
-	}
-	try {
-		err = seq_->get(dbtxn, 1, &newValue, flags);
-	} catch (DbException &e) {
-		throw XmlException(e);
-	}
-	newId = (docId_t)newValue;
+	DBXML_ASSERT(seq_);
+	if (txn) flags |= DB_AUTO_COMMIT|DB_TXN_NOSYNC;
+	// Do not pass txn to DB_SEQUENCE -- let autocommit work
+	int err =seq_->get(seq_, 0, 1, &newValue, flags);
+	if (err == 0)
+		newId = (docId_t)newValue;
 	return err;
 }
 
-void ConfigurationDatabase::resetSequenceCache()
-{
-	creatingTxn_ = 0;
-	try {
-		seq_->set_cachesize(seqIncr_);
-	} catch (DbException &e) {
-		throw XmlException(e);
-	}
-}
-
 int ConfigurationDatabase::getConfigurationItem(
 	Transaction *txn, const char *key, size_t keyLength, Buffer &b,
 	bool lock) const
@@ -194,15 +168,10 @@ int ConfigurationDatabase::getConfigurationItem(
 	//
 	DbtIn k((void*)key, keyLength);
 	DbtOut v;
-#if 1
 	u_int32_t flags = (txn ? (lock ? DB_RMW : DB_READ_COMMITTED) : 0);
 	int err = database_.get(txn, &k, &v, flags);
-#else
-	Cursor cursor(database_, txn, CURSOR_READ);
-	int err = cursor.get(&k, &v, DB_SET | (txn && lock ? DB_RMW : 0));
-#endif
 	if (err == 0) {
-		b.write(v.get_data(), v.get_size());
+		b.write(v.data, v.size);
 	}
 	return err;
 }
@@ -228,7 +197,7 @@ XmlContainer::ContainerType ConfigurationDatabase::checkContainerType(
 	int err = db.get(txn, &keydbt, &datadbt, 0);
 	if (!err) {
 		// existing DB.
-		const char *ttype = (const char*)datadbt.get_data();
+		const char *ttype = (const char*)datadbt.data;
 		if (::strcmp(ttype, ctypeNode) == 0)
 			type = XmlContainer::NodeContainer;
 		else if (::strcmp(ttype, ctypeDoc) == 0)
@@ -273,7 +242,7 @@ bool ConfigurationDatabase::checkIndexNodes(Transaction *txn, bool indexNodes,
 	int err = database_.get(txn, &keydbt, &datadbt, 0);
 	if (!err) {
 		// existing DB.
-		uint8_t val = *(const uint8_t*)datadbt.get_data();
+		uint8_t val = *(const uint8_t*)datadbt.data;
 		indexNodes = (val == 1);
 	} else if (err == DB_NOTFOUND) {
 		if (isReadonly) {
@@ -281,7 +250,7 @@ bool ConfigurationDatabase::checkIndexNodes(Transaction *txn, bool indexNodes,
 				   "Cannot write configuration on read-only Container");
 		}
 		datadbt.set(0, 1); // reserve space
-		*(uint8_t*)datadbt.get_data() = indexNodes ? 1 : 0;
+		*(uint8_t*)datadbt.data = indexNodes ? 1 : 0;
 		err = database_.put(txn, &keydbt, &datadbt, 0);
 
 		if (err)
@@ -336,7 +305,7 @@ int ConfigurationDatabase::getVersion(Transaction *txn, unsigned int &version) c
 	DbtOut datadbt;
 	int err = database_.get(txn, &keydbt, &datadbt, 0);
 	version =
-		(err!=0 ? 0 : DbXml::stringToInt((char *)datadbt.get_data()));
+		(err!=0 ? 0 : DbXml::stringToInt((char *)datadbt.data));
 	return err;
 }
 
@@ -358,11 +327,45 @@ int ConfigurationDatabase::checkIndexSpecification(Transaction *txn)
 	if(err == DB_NOTFOUND) {
 		// Write the default index specification
 		err = IndexSpecification().write(this, txn);
+
+		// Write the index version number
+		if(err == 0) err = updateIndexVersion(txn);
+
+		// Set container to auto-index mode to true
+		// for new containers
+		IndexSpecification::setAutoIndex(this, txn, true);
 	}
 
 	return err;
 }
 
+static const char *version_key = "index_version";
+static size_t version_keyLength = strlen(version_key) + 1;
+
+int ConfigurationDatabase::getIndexVersion(Transaction *txn, unsigned int &version) const
+{
+	Buffer b;
+	int err = getConfigurationItem(txn, version_key, version_keyLength, b, /*lock*/false);
+
+	if(err == DB_NOTFOUND) {
+		version = DEFAULT_INDEX_VERSION;
+		err = 0;
+	} else {
+		NsFormat::unmarshalInt((xmlbyte_t*)b.getBuffer(), &version);
+	}
+
+	return err;
+}
+
+int ConfigurationDatabase::updateIndexVersion(Transaction *txn)
+{
+	Buffer b;
+	size_t offset;
+	b.reserve(offset, NsFormat::countInt(CURRENT_INDEX_VERSION));
+	NsFormat::marshalInt((xmlbyte_t*)b.getBuffer(offset), CURRENT_INDEX_VERSION);
+	return putConfigurationItem(txn, version_key, b);
+}
+
 int ConfigurationDatabase::getIndexSpecification(
 	Transaction *txn, IndexSpecification &index, bool lock) const
 {
@@ -380,7 +383,7 @@ void ConfigurationDatabase::setIndexNodes(Transaction *txn, bool indexNodes)
 	static const char *indexNodesKey = "indexNodes";
 	DbtIn keydbt((void *) indexNodesKey, strlen(indexNodesKey) + 1); // include null in key
 	DbtOut datadbt(0, 1);
-	*(uint8_t*)datadbt.get_data() = indexNodes ? 1 : 0;
+	*(uint8_t*)datadbt.data = indexNodes ? 1 : 0;
 	int err = database_.put(txn, &keydbt, &datadbt, 0);
 	if (err)
 		throw XmlException(
@@ -388,14 +391,36 @@ void ConfigurationDatabase::setIndexNodes(Transaction *txn, bool indexNodes)
 			"Unexpected error from DB writing configuration");
 }
 
+static const char *compression_key = "compression_name";
+static size_t compression_keyLength = strlen(compression_key) + 1;
+
+std::string ConfigurationDatabase::getCompressionName(Transaction *txn) const
+{
+	Buffer b;
+	int err = getConfigurationItem(txn, compression_key, compression_keyLength, b, /*lock*/false);
+
+	if(!err) 
+		return (const char *)b.getBuffer();
+
+	return "";
+}
+
+int ConfigurationDatabase::putCompressionName(Transaction *txn, const char *name)
+{
+	Buffer b(name, ::strlen(name) + 1);
+	int err = putConfigurationItem(txn, compression_key, b);
+
+	return err;
+}
+
 //static
 XmlContainer::ContainerType
-ConfigurationDatabase::readContainerType(DbEnv *env,
+ConfigurationDatabase::readContainerType(DB_ENV *env,
 					 const std::string &name)
 {
 	DbWrapper database(env, name, "secondary_",
-			   configuration_name, 0, 0);
-	int err = database.open(0, DB_BTREE, 0, 0);
+			   configuration_name, DEFAULT_CONFIG);
+	int err = database.open(0, DB_BTREE, DEFAULT_CONFIG);
 	if (err != 0)
 		throw XmlException(XmlException::DATABASE_ERROR,
 				   "Unexpected error opening Configuration DB");
@@ -405,10 +430,10 @@ ConfigurationDatabase::readContainerType(DbEnv *env,
 }
 
 int ConfigurationDatabase::dump(
-	DbEnv *env, const std::string &name, std::ostream *out)
+	DB_ENV *env, const std::string &name, std::ostream *out)
 {
-	DbWrapper  database(env, name, "secondary_", configuration_name, 0, 0);
-	DbWrapper seqDatabase(env, name, "secondary_", sequence_name, 0, 0);
+	DbWrapper  database(env, name, "secondary_", configuration_name,DEFAULT_CONFIG);
+	DbWrapper seqDatabase(env, name, "secondary_", sequence_name, DEFAULT_CONFIG);
 
 	int err = Container::writeHeader(database.getDatabaseName(), out);
 	if(err == 0) err = database.dump(out);
@@ -423,11 +448,11 @@ int ConfigurationDatabase::dump(
 }
 
 int ConfigurationDatabase::load(
-	DbEnv *env, const std::string &name,
+	DB_ENV *env, const std::string &name,
 	std::istream *in, unsigned long *lineno)
 {
-	DbWrapper  database(env, name, "secondary_", configuration_name, 0, 0);
-	DbWrapper seqDatabase(env, name, "secondary_", sequence_name, 0, 0);
+	DbWrapper  database(env, name, "secondary_", configuration_name, DEFAULT_CONFIG);
+	DbWrapper seqDatabase(env, name, "secondary_", sequence_name, DEFAULT_CONFIG);
 
 	int err = Container::verifyHeader(database.getDatabaseName(), in);
 	if(err != 0) {
@@ -455,26 +480,29 @@ int ConfigurationDatabase::load(
 }
 
 int ConfigurationDatabase::verify(
-	DbEnv *env, const std::string &name, std::ostream *out,
+	DB_ENV *env, const std::string &name, std::ostream *out,
 	u_int32_t flags)
 {
-	DbWrapper  database(env, name, "secondary_", configuration_name, 0, 0);
-	DbWrapper seqDatabase(env, name, "secondary_", sequence_name, 0, 0);
+	DbWrapper  database(env, name, "secondary_", configuration_name, DEFAULT_CONFIG);
+	DbWrapper seqDatabase(env, name, "secondary_", sequence_name, DEFAULT_CONFIG);
 
 	int err = 0;
+	int terr = 0;
 	if(flags & DB_SALVAGE)
-		err = Container::writeHeader(database.getDatabaseName(), out);
-	if(err == 0)
-		err = database.verify(out, flags);
-
+		terr = Container::writeHeader(database.getDatabaseName(), out);
+	err = terr;
+	terr = database.verify(out, flags);
+	if (terr)
+		err = terr;
 	// sequence DB
-	if (err == 0) {
-		if(flags & DB_SALVAGE)
-			err = Container::writeHeader(seqDatabase.getDatabaseName(),
-						     out);
-		if(err == 0)
-			err = seqDatabase.verify(out, flags);
-	}
+	if(flags & DB_SALVAGE)
+		terr = Container::writeHeader(seqDatabase.getDatabaseName(),
+					      out);
+	if (terr)
+		err = terr;
+	terr = seqDatabase.verify(out, flags);
+	if (terr)
+		err = terr;
 
 	return err;
 }
@@ -495,7 +523,8 @@ int ConfigurationDatabase::putVersion(Transaction *txn,
 }
 
 // static
-void ConfigurationDatabase::upgrade(const std::string &name, Manager &mgr,
+void ConfigurationDatabase::upgrade(const std::string &name,
+				    const std::string &tname, Manager &mgr,
 				    int old_version, int current_version)
 {
 	DBXML_ASSERT(old_version < current_version);
@@ -506,19 +535,21 @@ void ConfigurationDatabase::upgrade(const std::string &name, Manager &mgr,
 		//   o move sequence record from configuration db
 		// o upgrade version to current_version
 		int err;
-		DbWrapper database(mgr.getDbEnv(), name, "secondary_",
-			configuration_name, 0, 0);
-		err = database.open(0, DB_BTREE, 0, 0);
+		DbWrapper database(mgr.getDB_ENV(), name, "secondary_",
+			configuration_name, DEFAULT_CONFIG);
+		err = database.open(0, DB_BTREE, DEFAULT_CONFIG);
 		if (old_version < VERSION_22) {
 			// need to upgrade to 2.2 format, with a separate
 			// sequence database
-			DbWrapper seqDatabase(mgr.getDbEnv(), name, "secondary_",
-				sequence_name, database.getPageSize(), 0);
+			ContainerConfig seqConfig;
+			seqConfig.setPageSize(database.getPageSize());
+			DbWrapper seqDatabase(mgr.getDB_ENV(), name, "secondary_",
+				sequence_name, seqConfig);
 			if (err == 0)
 				err = seqDatabase.open(0, DB_BTREE,
-						       DB_CREATE|DB_EXCL, 0);
+						       CREATE_CONFIG);
 			if (err == 0) {
-				Dbt seqKey((void*)"dbxmlID", 7);
+				DbXmlDbt seqKey((void*)"dbxmlID", 7);
 				DbtOut datadbt;
 				err = database.get(0, &seqKey, &datadbt, 0);
 				if (err == 0) {
@@ -528,8 +559,13 @@ void ConfigurationDatabase::upgrade(const std::string &name, Manager &mgr,
 						err = database.del(0, &seqKey, 0);
 				}
 			}
-		}
 
+		}
+		if (err == 0) 
+			err = DbWrapper::copySecondary(
+				mgr.getDB_ENV(),
+				name, tname, "secondary_",
+				sequence_name);
 		// Upgrade the index specification
 		if(err == 0) {
 			IndexSpecification is;
@@ -537,8 +573,16 @@ void ConfigurationDatabase::upgrade(const std::string &name, Manager &mgr,
 		}
 		if (err == 0)
 			err = putVersion(0, database, current_version);
-		if (err != 0)
-			throw XmlException(XmlException::DATABASE_ERROR,
-				   "Unexpected error upgrading Configuration DB");
+
+		if (err == 0)
+			err = DbWrapper::copySecondary(
+				mgr.getDB_ENV(),
+				name, tname, "secondary_",
+				configuration_name);
+		if (err != 0) {
+			char buf[100];
+			sprintf(buf, "Unexpected error upgrading Configuration DB: %d", err);
+			throw XmlException(XmlException::DATABASE_ERROR, buf);
+		}
 	}
 }
