@@ -1,16 +1,15 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: ConfigurationDatabase.hpp,v 1.21 2006/10/30 17:45:51 bostic Exp $
 //
 
 #ifndef __CONFIGURATIONDATABASE_HPP
 #define	__CONFIGURATIONDATABASE_HPP
 
 #include <string>
-#include <db_cxx.h>
+#include <db.h>
 #include "SharedPtr.hpp"
 #include "DbWrapper.hpp"
 #include "dbxml/XmlContainer.hpp"
@@ -22,7 +21,13 @@ class Buffer;
 class IndexSpecification;
 class Transaction;
 class DocID;
-	
+
+enum IndexVersion {
+	DEFAULT_INDEX_VERSION   = 1, ///< No index version (before index versions)
+	SUBSTRING_3CHAR_VERSION = 2, ///< Substring indexes include keys for lookups with < 3 chars
+	CURRENT_INDEX_VERSION = SUBSTRING_3CHAR_VERSION
+};
+
 /**
  * Wraps the configuration database, which is part of a container
  */
@@ -31,16 +36,13 @@ class ConfigurationDatabase
 public:
 	typedef SharedPtr<ConfigurationDatabase> Ptr;
 
-	ConfigurationDatabase(DbEnv *env, Transaction *txn,
+	ConfigurationDatabase(DB_ENV *env, Transaction *txn,
 			      const std::string &name,
-			      XmlContainer::ContainerType &type, // in/out
-			      u_int32_t pageSize,
-			      u_int32_t seqIncr, u_int32_t flags, int mode,
+			      ContainerConfig &config,
 			      bool &indexNodes,
 			      bool doVersionCheck);
 	~ConfigurationDatabase();
 	int generateID(Transaction *txn, DocID &newId);
-	void resetSequenceCache();
 	int getConfigurationItem(Transaction *txn, const char *key,
 				 size_t keyLength, Buffer &b, bool lock) const;
 	int putConfigurationItem(Transaction *txn, const char *key,
@@ -57,6 +59,12 @@ public:
 
 	void setIndexNodes(Transaction *txn, bool indexNodes);
 
+	std::string getCompressionName(Transaction *txn) const;
+	int putCompressionName(Transaction *txn, const char *name);
+
+	int getIndexVersion(Transaction *txn, unsigned int &version) const;
+	int updateIndexVersion(Transaction *txn);
+
 	void sync() {
 		database_.sync();
 	}
@@ -67,20 +75,21 @@ public:
 	}
 
 	static XmlContainer::ContainerType readContainerType(
-		DbEnv *env, const std::string &name);
+		DB_ENV *env, const std::string &name);
 
-	static int dump(DbEnv *env, const std::string &name,
+	static int dump(DB_ENV *env, const std::string &name,
 			std::ostream *out);
-	static int load(DbEnv *env, const std::string &name,
+	static int load(DB_ENV *env, const std::string &name,
 			std::istream *in, unsigned long *lineno);
-	static int verify(DbEnv *env, const std::string &name,
+	static int verify(DB_ENV *env, const std::string &name,
 			  std::ostream *out, u_int32_t flags);
 	// upgrade
 	static int putVersion(Transaction *txn, DbWrapper &db,
 			      unsigned int version);
-	static void upgrade(const std::string &name, Manager &mgr,
+	static void upgrade(const std::string &name,
+			    const std::string &tname, Manager &mgr,
 			    int old_version, int current_version);
-	
+
 private:
 	// no need for copy and assignment
 	ConfigurationDatabase(const ConfigurationDatabase&);
@@ -94,13 +103,12 @@ private:
 		Transaction *txn, DbWrapper &db,
 		XmlContainer::ContainerType type, bool isReadonly);
 	
-	DbEnv *environment_;
+	DB_ENV *environment_;
 	std::string name_;
 	mutable DbWrapper database_;
 	mutable DbWrapper seqDatabase_;
-	DbSequence *seq_;
+	DB_SEQUENCE *seq_;
 	u_int32_t seqIncr_;
-	Transaction *creatingTxn_;
 };
 
 }
