@@ -1,11 +1,10 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: DictionaryDatabase.cpp,v 1.25.2.1 2007/01/09 22:32:49 gmf Exp $
 //
-
+#include "DbXmlInternal.hpp"
 #include <sstream>
 
 #include <dbxml/XmlException.hpp>
@@ -15,6 +14,7 @@
 #include "ScopedDbt.hpp"
 #include "Container.hpp"
 #include "OperationContext.hpp"
+#include "MutexLock.hpp"
 #include "db_utils.h"
 
 using namespace DbXml;
@@ -73,54 +73,63 @@ static const u_int32_t lastPreloadId = 20;
 static const u_int32_t dictCacheBytes = 30 * 1024; // 30k
 static const char *dictionary_name = "dictionary";
 
-DictionaryDatabase::DictionaryDatabase(DbEnv *env, Transaction *txn,
+DictionaryDatabase::DictionaryDatabase(DB_ENV *env, Transaction *txn,
 				       const std::string &name,
-				       u_int32_t pageSize,
-				       u_int32_t flags, int mode)
-	: environment_(env),
+				       const ContainerConfig &config,
+				       bool useMutex)
+	: stringCache_(true),
+	  environment_(env),
 	  name_(name),
 	  primary_(new PrimaryDatabase(env, name,
 				       dictionary_name,
-				       pageSize, flags & DB_XA_CREATE)),
+				       DEFAULT_CONFIG)),
 	  secondary_(new SecondaryDatabase(env, name,
 					   dictionary_name,
-					   pageSize, flags & DB_XA_CREATE)),
+					   DEFAULT_CONFIG)),
 	  nidName_(0),
 	  nidRoot_(0),
-	  usePreloads_(false)
+	  usePreloads_(false),
+	  isTransacted_(txn ? true : false),
+	  mutex_(useMutex ? MutexLock::createMutex() : 0)
 {
-	flags &= ~DB_XA_CREATE;
+	cache_.setDictionaryDatabase(this);
+	if (!isTransacted_ && env) {
+		u_int32_t envFlags;
+		env->get_flags(env, &envFlags);
+		if (envFlags & DB_CDB_ALLDB)
+			isTransacted_ = true;
+	}
 	int err = 0;
 	try {
-		// set cache sizes low if no DbEnv -- this is the in-memory
+		// set cache sizes low if no DB_ENV -- this is the in-memory
 		// dictionary for transient docs
 		if (!env) {
-			primary_->getDb().set_cachesize(0, dictCacheBytes, 1);
-			secondary_->getDb().set_cachesize(0, dictCacheBytes, 1);
+			primary_->getDb()->set_cachesize(primary_->getDb(), 0, dictCacheBytes, 1);
+			secondary_->getDb()->set_cachesize(secondary_->getDb(), 0, dictCacheBytes, 1);
 		}
 
 		// Open the Db objects
-		err = primary_->open(txn, flags, mode);
+		err = primary_->open(txn, config);
 		if (err == 0)
-			err = secondary_->open(txn, /*duplicates*/true, flags, mode);
+			err = secondary_->open(txn, /*duplicates*/true, config);
 
 		if (err == 0) {
 			// Lookup/Define the dbxml namespace names (but don't define
 			// if this is a read-only container)
-			bool rdonly = (flags & DB_RDONLY) != 0;
+			bool rdonly = config.getReadOnly();
 			preloadDictionary(txn, rdonly);
 		}
 
-	} catch (DbException &e) {
-		err = e.get_errno();
-		// fall through
-	}
-	catch (XmlException &) {
+	} catch (XmlException &) {
+		secondary_->cleanup();
+		primary_->cleanup();
 		if (txn)
 			txn->abort();
 		throw;
 	}
 	if (err != 0) {
+		secondary_->cleanup();
+		primary_->cleanup();
 		if (txn)
 			txn->abort();
 		string msg = name;
@@ -139,7 +148,8 @@ DictionaryDatabase::DictionaryDatabase(DbEnv *env, Transaction *txn,
 
 DictionaryDatabase::~DictionaryDatabase()
 {
-	// nothing to do
+	if (mutex_)
+		MutexLock::destroyMutex(mutex_);
 }
 
 //
@@ -181,7 +191,7 @@ void DictionaryDatabase::preloadDictionary(Transaction *txn, bool rdonly)
 				const char *unused = "x";
 				DbtIn primaryData((void*)unused, 2);
 				err = primary_->appendPrimary(oc, dummy,
-							      &primaryData,
+							      primaryData,
 							      /*no flags*/0);
 				i++;
 			}
@@ -202,8 +212,9 @@ void DictionaryDatabase::preloadDictionary(Transaction *txn, bool rdonly)
 	}
 }
 
+// mutex is locked if present and necessary
 int DictionaryDatabase::defineStringName(OperationContext &context,
-					 const char *name, u_int32_t namelen,
+					 const char *name, size_t namelen,
 					 NameID &id)
 {
 	// Primary { id -> name\0 }
@@ -213,24 +224,39 @@ int DictionaryDatabase::defineStringName(OperationContext &context,
 	DbtIn primaryData((void*)name, namelen + 1); // add null
 	DbtIn secondaryKey((void*)name, namelen); // don't add null
 	err = primary_->appendPrimary(context, id,
-				      &primaryData, /*no flags*/0);
+				      primaryData, /*no flags*/0);
 	if (err == 0) {
+		/* Cache after insert.  This prevents the cache from returning bad answers
+		 * on a transaction abort, and speeds up access to newly inserted element
+		 * and attribute names that are needed for indexing after an insert.  
+		 */
+		cache_.insert(id.raw(), primaryData);
 		id.setDbtFromThis(context.key());
-		err = secondary_->put(context.txn(), &secondaryKey,
+		Transaction *txn = getTxn(context);
+		err = secondary_->put(txn, &secondaryKey,
 				      &context.key(),
 				      /*no flags*/0);
-		if (err == 0 &&
-			Log::isLogEnabled(Log::C_DICTIONARY, Log::L_INFO)) {
-			ostringstream oss;
-			oss << "Define new name " << id << " -> " << name;
-			Log::log(environment_, Log::C_DICTIONARY, Log::L_INFO,
-				 name_.c_str(),
-				 oss.str().c_str());
+		if (err == 0) {
+			/* Add to the transaction's string cache, that way if an abort occurs
+			 * the values can be removed from the cache. 
+			 */
+			if (txn) { 
+				DictionaryStringCache *dsc = txn->getStringCache(this, true);
+				dsc->insert(&secondaryKey, id.raw());
+			}
+			if (Log::isLogEnabled(Log::C_DICTIONARY, Log::L_INFO)) {
+				ostringstream oss;
+				oss << "Define new name " << id << " -> " << name;
+				Log::log(environment_, Log::C_DICTIONARY, Log::L_INFO,
+					name_.c_str(),
+					oss.str().c_str());
+			}
 		}
 	}
 	return err;
 }
 
+// mutex is locked if present and necessary
 int DictionaryDatabase::defineName(OperationContext &context,
 				   const Name &name, NameID &id)
 {
@@ -238,23 +264,38 @@ int DictionaryDatabase::defineName(OperationContext &context,
 	// Secondary { name:uri -> id }
 	id.reset();
 	name.setDbtFromThis_PrimaryValue(context.data());
-	int err = primary_->appendPrimary(context, id, &context.data(), /*no flags*/0);
+	int err = primary_->appendPrimary(context, id, context.data(), /*no flags*/0);
 	if (err == 0) {
+		/*Cache after insert.  This prevents the cache from returning bad answers
+		 * on a transaction abort, and speeds up access to newly inserted element
+		 * and attribute names that are needed for indexing after an insert.  
+		 */
+		cache_.insert(id.raw(), context.data());
 		id.setDbtFromThis(context.key());
 		name.setDbtFromThis_SecondaryKey(context.data());
-		err = secondary_->put(context.txn(), &context.data(), &context.key(), /*no flags*/0);
-		if (err == 0 &&
-			Log::isLogEnabled(Log::C_DICTIONARY, Log::L_INFO)) {
-			ostringstream oss;
-			oss << "Define new name " << id << " -> " << name;
-			Log::log(environment_, Log::C_DICTIONARY, Log::L_INFO,
-				 name_.c_str(),
-				 oss.str().c_str());
+		Transaction *txn = getTxn(context);
+		err = secondary_->put(txn, &context.data(), &context.key(), /*no flags*/0);
+		if (err == 0) {
+			/* Add to the transaction's string cache, that way if an abort occurs
+			 * the values can be removed from the cache. 
+			 */
+			if (txn) { 
+				DictionaryStringCache *dsc = txn->getStringCache(this, true);
+				dsc->insert(&context.data(), id.raw());
+			}
+			if(Log::isLogEnabled(Log::C_DICTIONARY, Log::L_INFO)) {
+				ostringstream oss;
+				oss << "Define new name " << id << " -> " << name;
+				Log::log(environment_, Log::C_DICTIONARY, Log::L_INFO,
+					 name_.c_str(),
+					oss.str().c_str());
+			}
 		}
-	}
+	}	
 	return err;
 }
 
+// mutex is locked if present and necessary
 int DictionaryDatabase::defineQName(OperationContext &context,
 				    const char *uriname, NameID &id)
 {
@@ -262,27 +303,71 @@ int DictionaryDatabase::defineQName(OperationContext &context,
 	return defineName(context, name, id);
 }
 
-// puts result in context.data()
+// puts result in dbt, which may be context.data(), or may be user-provided
 int DictionaryDatabase::lookupFromID(
 	OperationContext &context, DbtOut &dbt,
 	const NameID &id) const
 {
 	int err = 0;
-	u_int32_t flags = (context.txn()) ? DB_READ_COMMITTED : 0;
+	if (cache_.lookup(context, id, dbt, false))
+		return 0;
+	u_int32_t flags = (getTxn(context)) ? DB_READ_COMMITTED : 0;
 	// primary key is the integer value of the id (database is
 	// recno)
 	id.setDbtFromThisAsId(context.key());
-	err = primary_->get(context.txn(), &context.key(),
+	MutexLock ml(mutex_);
+	err = primary_->get(getTxn(context), &context.key(),
 			    &dbt, flags);
+	if (err == 0)
+		cache_.insert(id.raw(), dbt);
 	return err;
 }
 
+void DictionaryDatabase::display(OperationContext &context, ostream &out) const
+{
+	{
+	Cursor myCursor(const_cast<PrimaryDatabase&>(*primary_.get()), getTxn(context), CURSOR_READ, 0);
+	if(myCursor.error() != 0) throw XmlException(myCursor.error());
+
+	int err = 0;
+	NameID id;
+	while((err = myCursor.get(context.key(), context.data(), DB_NEXT)) == 0) {
+		id.setThisFromDbtAsId(context.key());
+		Buffer val(context.data().data, context.data().size, true);
+		out << id << " -> " << val.asString(true) << endl;
+	}
+	}
+	{
+	Cursor myCursor(const_cast<SecondaryDatabase&>(*secondary_.get()), getTxn(context), CURSOR_READ, 0);
+	if(myCursor.error() != 0) throw XmlException(myCursor.error());
+
+	int err = 0;
+	NameID id;
+	while((err = myCursor.get(context.key(), context.data(), DB_NEXT)) == 0) {
+		Buffer val(context.key().data, context.key().size, true);
+		id.setThisFromDbt(context.data());
+		out << val.asString(true) << " -> " << id << endl;
+	}
+	}
+}
+
+// lookup through cache
+
+const char *DictionaryDatabase::lookupName(OperationContext &context,
+					   const NameID &id) const
+{
+	nameId_t raw = id.raw();
+	if ((raw < DICTIONARY_RESERVE_SIZE) && usePreloads_) {
+		return preloadNames[raw-1]; // preload space is 1-based, not 0
+	} else
+		return cache_.lookup(context, raw, true);
+}
 
 int DictionaryDatabase::lookupNameFromID(OperationContext &context,
 					 const NameID &id, Name &name) const
 {
 	int err = 0;
-	uint32_t raw = id.raw() - 1; // id space is 1-based, not 0
+	nameId_t raw = id.raw() - 1; // id space is 1-based, not 0
 	if (id == nidName_) {
 		name = Name::dbxml_colon_name;
 		return 0;
@@ -309,13 +394,13 @@ int DictionaryDatabase::lookupStringNameFromID(
 	const NameID &id, const char **name) const
 {
 	int err = 0;
-	uint32_t raw = id.raw() - 1; // id space is 1-based, not 0
+	nameId_t raw = id.raw() - 1; // id space is 1-based, not 0
 	if ((raw < DICTIONARY_RESERVE_SIZE) && usePreloads_) {
 		*name = preloadNames[raw];
 	} else {
 		err = lookupFromID(context, context.data(), id);
 		if (err == 0)
-			*name = (const char *)context.data().get_data();
+			*name = (const char *)context.data().data;
 	}
 
 	return err;
@@ -326,7 +411,7 @@ int DictionaryDatabase::lookupStringNameFromID(
 	const NameID &id, DbtOut &dbt) const
 {
 	int err = 0;
-	uint32_t raw = id.raw() - 1; // id space is 1-based, not 0
+	nameId_t raw = id.raw() - 1; // id space is 1-based, not 0
 	if ((raw < DICTIONARY_RESERVE_SIZE) && usePreloads_) {
 		const char *name = preloadNames[raw];
 		dbt.set((const void *)name, ::strlen(name) + 1);
@@ -336,6 +421,32 @@ int DictionaryDatabase::lookupStringNameFromID(
 	return err;
 }
 
+bool DictionaryDatabase::stringCacheLookup(OperationContext &context,
+					   DbXmlDbt &dbt, NameID &id) const
+{
+	nameId_t *nid = 0;
+	Transaction *txn = getTxn(context);
+	if (txn) {
+		DictionaryStringCache *dsc =
+			txn->getStringCache(const_cast<DictionaryDatabase*>(this), false);
+		if (dsc) {
+			nid = dsc->lookup(&dbt);
+			if (nid) {
+				DbXmlDbt dbt((void *)nid, sizeof(nameId_t));
+				id.setThisFromDbtAsId(dbt);
+				return true;
+			}
+		}
+	}
+	nid = stringCache_.lookup(&dbt);
+	if (nid) {
+		DbXmlDbt tdbt((void *)nid, sizeof(nameId_t));
+		id.setThisFromDbtAsId(tdbt);
+		return true;
+	}
+	return false;
+}
+
 int DictionaryDatabase::lookupIDFromName(OperationContext &context,
 					 const Name &name,
 					 NameID &id, bool define)
@@ -347,11 +458,15 @@ int DictionaryDatabase::lookupIDFromName(OperationContext &context,
 		id = nidRoot_;
 	}
 	if (id == 0) {
-		u_int32_t flags = (context.txn()) ? DB_READ_COMMITTED : 0;
+		MutexLock ml(mutex_);
+		u_int32_t flags = (getTxn(context) && !define) ? DB_READ_COMMITTED : 0;
 		name.setDbtFromThis_SecondaryKey(context.key());
-		err = secondary_->get(context.txn(), &context.key(), &context.data(), flags);
+		if (stringCacheLookup(context, context.key(), id))
+			return 0;
+		err = secondary_->get(getTxn(context), &context.key(), &context.data(), flags);
 		if (err == 0) {
 			id.setThisFromDbt(context.data());
+			stringCache_.insert(&context.key(), id.raw());
 		} else if (err == DB_NOTFOUND && define) {
 			err = defineName(context, name, id); // define from name:uri
 		} else {
@@ -361,49 +476,60 @@ int DictionaryDatabase::lookupIDFromName(OperationContext &context,
 	return err;
 }
 
+// qname is of format name:uri
 int DictionaryDatabase::lookupIDFromQName(OperationContext &context,
-					  const char *uriname,
+					  DbXmlDbt &qname,
 					  NameID &id, bool define)
 {
-	int err = lookupIDFromStringName(context, uriname,
-					 ::strlen(uriname), id);
-	if (err == DB_NOTFOUND && define)
-		err = defineQName(context, uriname, id); // define from name:uri
+	MutexLock ml(mutex_);
+	int err = lookupIDFromStringNameInternal(context, qname,
+						 id, define);
+	if (err == DB_NOTFOUND && define)  // define from name:uri
+		err = defineQName(context, (const char *)qname.data, id);
 	return err;
 }
 
 int DictionaryDatabase::lookupIDFromStringName(OperationContext &context,
 					       const char *name,
-					       u_int32_t namelen,
+					       size_t namelen,
 					       NameID &id, bool define)
 {
-	int err = lookupIDFromStringName(context, name, namelen, id);
+	MutexLock ml(mutex_);
+	DbtIn dbt((void *)name, namelen);
+	int err = lookupIDFromStringNameInternal(context, dbt, id, define);
 	if (err == DB_NOTFOUND && define)
 		err = defineStringName(context, name, namelen, id); // define from name
 	return err;
 }
 
-int DictionaryDatabase::lookupIDFromStringName(OperationContext &context,
-					       const char *name,
-					       u_int32_t namelen,
-					       NameID &id) const
+int DictionaryDatabase::lookupIDFromStringNameInternal(
+	OperationContext &context,
+	DbXmlDbt &dbt,
+	NameID &id,
+	bool define) const
 {
-	context.key().set((void*)name, namelen);
-	u_int32_t flags = (context.txn()) ? DB_READ_COMMITTED : 0;
-	int err = secondary_->get(context.txn(), &context.key(),
+	if (!dbt.size) {
+		id.reset();
+		return 0;
+	}
+	u_int32_t flags = (getTxn(context) && !define) ? DB_READ_COMMITTED : 0;
+	if (stringCacheLookup(context, dbt, id))
+			return 0;
+	int err = secondary_->get(getTxn(context), &dbt,
 				  &context.data(), flags);
 	if (err == 0) {
 		id.setThisFromDbt(context.data());
+		stringCache_.insert(&dbt, id.raw());
 	} else
 		id.reset();
 	return err;
 }
 
-int DictionaryDatabase::dump(DbEnv *env, const std::string &name,
+int DictionaryDatabase::dump(DB_ENV *env, const std::string &name,
 			     std::ostream *out)
 {
-	PrimaryDatabase::Ptr primary(new PrimaryDatabase(env, name, dictionary_name, 0, 0));
-	SecondaryDatabase::Ptr secondary(new SecondaryDatabase(env, name, dictionary_name, 0, 0));
+	PrimaryDatabase::Ptr primary(new PrimaryDatabase(env, name, dictionary_name, DEFAULT_CONFIG));
+	SecondaryDatabase::Ptr secondary(new SecondaryDatabase(env, name, dictionary_name, DEFAULT_CONFIG));
 
 	int err = Container::writeHeader(primary->getDatabaseName(), out);
 	if(err == 0) err = primary->dump(out);
@@ -414,11 +540,11 @@ int DictionaryDatabase::dump(DbEnv *env, const std::string &name,
 	return err;
 }
 
-int DictionaryDatabase::load(DbEnv *env, const std::string &name,
+int DictionaryDatabase::load(DB_ENV *env, const std::string &name,
 			     std::istream *in, unsigned long *lineno)
 {
-	PrimaryDatabase::Ptr primary(new PrimaryDatabase(env, name, dictionary_name, 0, 0));
-	SecondaryDatabase::Ptr secondary(new SecondaryDatabase(env, name, dictionary_name, 0, 0));
+	PrimaryDatabase::Ptr primary(new PrimaryDatabase(env, name, dictionary_name, DEFAULT_CONFIG));
+	SecondaryDatabase::Ptr secondary(new SecondaryDatabase(env, name, dictionary_name, DEFAULT_CONFIG));
 
 	// Load primary
 	int err = Container::verifyHeader(primary->getDatabaseName(), in);
@@ -447,19 +573,29 @@ int DictionaryDatabase::load(DbEnv *env, const std::string &name,
 	return err;
 }
 
-int DictionaryDatabase::verify(DbEnv *env, const std::string &name,
+int DictionaryDatabase::verify(DB_ENV *env, const std::string &name,
 			       std::ostream *out, u_int32_t flags)
 {
-	PrimaryDatabase::Ptr primary(new PrimaryDatabase(env, name, dictionary_name, 0, 0));
-	SecondaryDatabase::Ptr secondary(new SecondaryDatabase(env, name, dictionary_name, 0, 0));
+	PrimaryDatabase::Ptr primary(new PrimaryDatabase(env, name, dictionary_name, DEFAULT_CONFIG));
+	SecondaryDatabase::Ptr secondary(new SecondaryDatabase(env, name, dictionary_name, DEFAULT_CONFIG));
 
 	int err = 0;
-	if(flags & DB_SALVAGE) err = Container::writeHeader(primary->getDatabaseName(), out);
-	if(err == 0) err = primary->verify(out, flags);
-
-	if(err == 0 && (flags & DB_SALVAGE)) err = Container::writeHeader(secondary->getDatabaseName(), out);
-	if(err == 0) err = secondary->verify(out, flags);
+	int terr = 0;
+	// once created the databases need to be verified to destroy the handles
+	if(flags & DB_SALVAGE)
+		terr = Container::writeHeader(primary->getDatabaseName(), out);
+	err = terr;
+	terr = primary->verify(out, flags);
+	if (terr)
+		err = terr;
 
+	if(flags & DB_SALVAGE)
+		terr = Container::writeHeader(secondary->getDatabaseName(), out);
+	if (terr)
+		err = terr;
+	terr = secondary->verify(out, flags);
+	if (terr)
+		err = terr;
 	return err;
 }
 
@@ -493,8 +629,10 @@ static bool needsSwap(u_int32_t val)
 	return false;
 	
 }
+
 //static
-void DictionaryDatabase::upgrade(const std::string &name, Manager &mgr,
+void DictionaryDatabase::upgrade(const std::string &name,
+				 const std::string &tname, Manager &mgr,
 				 int old_version, int current_version)
 {
 	DBXML_ASSERT(old_version < current_version);
@@ -502,38 +640,38 @@ void DictionaryDatabase::upgrade(const std::string &name, Manager &mgr,
 	// could probably do 2-stage upgrade in one iteration, but
 	// for now, it's simpler to do 2 separate stages
 	if (old_version == VERSION_20) {
-		SecondaryDatabase secondary(mgr.getDbEnv(),
-					    name, dictionary_name, 0, 0);
-		err = secondary.open(0, /*duplicates*/true, 0, 0);
+		SecondaryDatabase secondary(mgr.getDB_ENV(),
+					    name, dictionary_name, DEFAULT_CONFIG);
+		err = secondary.open(0, /*duplicates*/true, DEFAULT_CONFIG);
 		// byte-swap dictionary ids
 		// iterate through all records in secondary
-		Dbt key;
-		Dbt data;
+		DbXmlDbt key;
+		DbXmlDbt data;
 		Cursor curs(secondary, (Transaction*)0, DbXml::CURSOR_WRITE);
 		int ret = 0;
-		while ((ret = curs.get(&key, &data, DB_NEXT)) == 0) {
-			u_int32_t *p = reinterpret_cast<u_int32_t*>(data.get_data());
+		while ((ret = curs.get(key, data, DB_NEXT)) == 0) {
+			u_int32_t *p = reinterpret_cast<u_int32_t*>(data.data);
 			if (!needsSwap(*p))
 				break;
 			curs.del(0);
 			M_32_SWAP(*p);
-			int putRet = curs.put(&key, &data, DB_NODUPDATA);
+			int putRet = curs.put(key, data, DB_NODUPDATA);
 			if (putRet != 0)
 				err = 1;
 		}
 	}
 	if (err == 0 && old_version < VERSION_22) {
 		// need to do upgrade to 2.2
-		SecondaryDatabase secondary(mgr.getDbEnv(),
-					    name, dictionary_name, 0, 0);
-		err = secondary.open(0, /*duplicates*/true, 0, 0);
-		Dbt key;
-		Dbt data;
+		SecondaryDatabase secondary(mgr.getDB_ENV(),
+					    name, dictionary_name, DEFAULT_CONFIG);
+		err = secondary.open(0, /*duplicates*/true, DEFAULT_CONFIG);
+		DbXmlDbt key;
+		DbXmlDbt data;
 		Cursor curs(secondary, (Transaction*)0, DbXml::CURSOR_WRITE);
 		int ret = 0;
-		while ((ret = curs.get(&key, &data, DB_NEXT)) == 0) {
+		while ((ret = curs.get(key, data, DB_NEXT)) == 0) {
 			u_int32_t tid = *(reinterpret_cast<u_int32_t *>
-					  (data.get_data()));
+					  (data.data));
 			if (Manager::isBigendian())
 				M_32_SWAP(tid);
 			// create and marshal ID, then put
@@ -541,13 +679,40 @@ void DictionaryDatabase::upgrade(const std::string &name, Manager &mgr,
 			NameID id(tid);
 			id.setDbtFromThis(dbt);
 			curs.del(0);
-			int putRet = curs.put(&key, &dbt, DB_NODUPDATA);
+			int putRet = curs.put(key, dbt, DB_NODUPDATA);
 			if (putRet != 0)
 				err = 1;
 		}
 	}
-	if (err != 0)
-		throw XmlException(
-			XmlException::DATABASE_ERROR,
-			"Unexpected error upgrading Dictionary DB");
+	// copy to new container
+	if (err == 0)
+		err = DbWrapper::copySecondary(
+			mgr.getDB_ENV(),
+			name, tname, "secondary_",
+			dictionary_name, true);
+	if (err == 0) {
+		// copy dictionary primary
+		PrimaryDatabase primary(mgr.getDB_ENV(), name,
+					dictionary_name, DEFAULT_CONFIG);
+		primary.open(0, DEFAULT_CONFIG);
+		PrimaryDatabase newPrimary(mgr.getDB_ENV(), tname,
+					   dictionary_name, DEFAULT_CONFIG);
+		newPrimary.open(0, CREATE_CONFIG);
+
+		Cursor src(primary, (Transaction*)0, DbXml::CURSOR_READ);
+		DbXmlDbt key;
+		DbXmlDbt data;
+		int err = 0, ret = 0;
+		while ((err == 0) &&
+		       (ret = src.get(key, data, DB_NEXT)) == 0) {
+			newPrimary.put(0, &key, &data, DB_APPEND);
+		}
+	}
+
+	if (err != 0) {
+		char buf[100];
+		sprintf(buf, "Unexpected error upgrading Dictionary DB: %d",
+			err);
+		throw XmlException(XmlException::DATABASE_ERROR, buf);
+	}
 }
