@@ -1,9 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: Document.hpp,v 1.101 2006/10/30 17:45:52 bostic Exp $
+// $Id: Document.hpp,v 86f215ae8bc4 2008/08/20 17:42:30 john $
 //
 
 #ifndef __DOCUMENT_HPP
@@ -19,11 +19,12 @@
 #include "SharedPtr.hpp"
 #include "Value.hpp"
 #include "MetaDatum.hpp"
-#include "TransactedContainer.hpp"
+#include "Container.hpp"
 #include <dbxml/XmlInputStream.hpp>
 #include "OperationContext.hpp"
+#include "CacheDatabaseMinder.hpp"
+#include "dataItem/DbXmlConfiguration.hpp"
 
-#include <xercesc/dom/DOM.hpp>
 #include <xercesc/parsers/XercesDOMParser.hpp>
 
 namespace DbXml
@@ -34,17 +35,21 @@ class DbtOut;
 class OperationContext;
 class QueryContext;
 class NsDocument;
-class NsXDOMFactory;
+class NsDomElement;
 class NsPushEventSource;
 class EventReader;
 class EventWriter;
 class NsEventTranslator;
 class ReferenceMinder;
 class XmlEventReader;
+class DocDatabase;
+class DictionaryDatabase;
+class DbWrapper;
+class NsNode;
 	
 typedef std::vector<MetaDatum*> MetaData;
 
-class MetaDataIterator : public ReferenceCounted
+class DBXML_EXPORT MetaDataIterator : public ReferenceCounted
 {
 public:
 	MetaDataIterator(Document *document);
@@ -53,6 +58,8 @@ public:
 	bool next(std::string &uri, std::string &name, XmlValue &value);
 	void reset();
 	
+	bool previousIsModified() const;
+	bool previousIsRemoved() const;
 private:
 	// no need for copy and assignment
 	MetaDataIterator(const MetaDataIterator&);
@@ -62,7 +69,7 @@ private:
 	MetaData::const_iterator i_;
 };
 	
-class Document : public ReferenceCounted
+class DBXML_EXPORT Document : public ReferenceCounted
 {
 public:
  	enum Content {
@@ -79,26 +86,29 @@ public:
 		WF_ONLY = 3
 	};
 public:
-	Document(XmlManager &mgr);
+	Document(Manager &mgr);
 	virtual ~Document();
 
 	Document *createCopy() const;
 
-	void reset();
-	
-	void setMetaDataFromContainer(TransactedContainer *container,
+	void setMetaDataFromContainer(Container *container,
 				      const DocID &id, Transaction *txn,
 				      u_int32_t flags);
 	void setEagerMetaData();
 	void setEagerContentAndMetaData();
 
-	void setContainer(TransactedContainer *container);
-	TransactedContainer *getContainer() const;
+	void setContainer(Container *);
+	void setContainerID(int cid) { cid_ = cid; }
+	int getContainerID() const { return cid_; }
+	std::string getContainerName() const;
+	bool isWholedoc()const;
 
+	DictionaryDatabase *getDictionaryDB() const;
+	
 	void setValidation(int validation) {
 		validation_ = validation;
 	}
-	
+
 	/**
 	 * Set the ID of a document. The ID is assigned when the document
 	 * is first written to a container.
@@ -113,9 +123,9 @@ public:
 		return id_;
 	}
 
-	const XMLCh *getDocumentURI() const {
-		return documentURI_;
-	}
+	Manager &getManager() const { return (Manager&)mgr_; }
+
+	const XMLCh *getDocumentURI() const;
 	void setDocumentURI(const XMLCh *uri);
 
 	void addReferenceMinder(ReferenceMinder *r);
@@ -136,10 +146,10 @@ public:
 
 	// Meta-Data, as Dbt
 	void setMetaData(const Name &name, XmlValue::Type type,
-			 const Dbt &value, bool modified);
+			 const XmlData &value, bool modified);
 	void setMetaData(const Name &name, XmlValue::Type type,
 			 DbtOut **value, bool modified); // Note: Consumes value
-	bool getMetaData(const Name &name, Dbt &value);
+	bool getMetaData(const Name &name, XmlData &value);
 
 	// Remove Meta-Data item
 	void removeMetaData(const Name &name);
@@ -150,6 +160,10 @@ public:
 	MetaData::const_iterator metaDataBegin() const;
 	MetaData::const_iterator metaDataEnd() const;
 
+	// Consider removal; for now, these are needed/used
+	void resetContentAsNsDom(bool resetDocDb) const;
+	void resetContentAsDbt() const;
+	
 	// Content
 	bool isContentModified() const;
 	void setContentModified(bool modified) const
@@ -160,42 +174,52 @@ public:
 	{
 		return definitiveContent_;
 	}
-	void domModified() const;
 
-	bool needsCopyForUpdate() const;
-	
 	// Content as Dbt
 	void setContentAsDbt(DbtOut **value, bool setOnly = false); // Consumes value
-	const Dbt *getContentAsDbt() const;
-	void resetContentAsDbt() const;
+	const DbXmlDbt *getContentAsDbt() const;
 
 	// Content as InputStream
 	void setContentAsInputStream(XmlInputStream **adopted_str);
 	XmlInputStream *getContentAsInputStream() const; // donated to caller
-	void resetContentAsInputStream() const;
 
-	// Content as DOM
-	void changeContentToDOM() const;
-	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *getContentAsDOM() const;
-	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *getElement(const NsNid *nid) const;
-	void setContentAsDOM(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc);
-	void resetContentAsDOM() const;
+	// Content as NsDom
+	void changeContentToNsDom(const ISNVector *isns) const;
+	NsDomElement *getContentAsNsDom(const ISNVector *isns,
+					Transaction *txn) const;
+	NsDomElement *getElement(const NsNid nid, const ISNVector *isns) const;
+	NsDomElement *getElement(NsNode *node) const;
+
+	// assign a given doc id and cache db toa document
+	void setContentAsNsDom(const DocID &did, CacheDatabase *db);
 
 	// Content as XmlEventReader
 	void setContentAsEventReader(XmlEventReader &reader);
 	XmlEventReader &getContentAsEventReader() const;
-	void resetContentAsEventReader() const;
-
+	
 	// Content as XmlEventWriter
 	void getContentAsEventWriter(XmlEventWriter &writer);
 
-	NsDocument *getNsDocument() const
-	{
+
+	bool isUninitialized() const; // mostly for debug
+
+	bool isEmpty() const {
+		return (definitiveContent_ == NONE && lazy_ == NEITHER);
+	}
+	
+	void resetDocDb() const;
+
+
+	DbWrapper *getDocDb();
+
+	CacheDatabase *getCacheDatabase() { return (CacheDatabase *)docdb_; }
+
+	NsDocument *getNsDocument() const {
 		return nsDocument_;
 	}
 
 	// Content as events
-	EventReader *getElementAsReader(XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *element) const;
+	EventReader *getElementAsReader(NsDomElement *element) const;
 	NsPushEventSource *getContentAsEventSource(
 		Transaction *txn,
 		bool needsValidation, // true on initial input (wholedoc only)
@@ -203,14 +227,33 @@ public:
 		bool useId = false // force use of ID, not definitiveContent_
 		) const;
 
-	bool isNodeDocument() const
-	{
-		return container_ && container_->getContainerType() ==
-			XmlContainer::NodeContainer;
+	void initNsObjects(DbWrapper *db, Transaction *txn) const;
+
+	CacheDatabaseMinder &getDbMinder() {
+		return dbMinder_;
+	}
+
+	// unconditionally set txn
+	void setTransaction(Transaction *txn) const {
+		oc_.set(txn);
+	}
+	
+	void setDbMinder(CacheDatabaseMinder &minder) {
+		dbMinder_ = minder;
+	}
+	void setDbMinderIfNull(CacheDatabaseMinder &minder) {
+		if (dbMinder_.isNull()) dbMinder_ = minder;
+	}
+	OperationContext &getOperationContext() const {
+		return oc_;
 	}
-	void initNsObjects() const;
 
 	bool operator==(const Document &other) const;
+
+	/* Used by RawNodeValue, where an empty document is created
+		that has a document and container id, but is created
+		as empty. LJF*/
+	void setAsNotMaterialized();
 private:
 	// no need for copy and assignment
 	Document(const Document&);
@@ -222,22 +265,24 @@ private:
 	// @{
 
 	void id2dbt() const;
-	void id2dom(bool includeEntityInfo = false) const;
+	void id2dom(const ISNVector *isns) const;
 	void id2stream() const;
 	void id2reader() const;
-	void dbt2stream() const;
-	void dbt2dom(Transaction *txn, bool includeEntityInfo = false) const;
+	void dbt2stream(bool adopt=false) const;
+	void dbt2dom() const;
 	void dbt2reader() const;
 	void dom2dbt() const;
 	void dom2stream() const;
 	void dom2reader() const;
-	void stream2dom(Transaction *txn, bool includeEntityInfo = false) const;
+	void stream2dom(const ISNVector *isns) const;
 	void stream2dbt() const;
 	void stream2reader() const;
 
 	void reader2dbt() const;
 	void reader2stream() const;
 
+	bool hasNsDom() const;
+	
 	NsPushEventSource *id2events(Transaction *txn, bool needsValidation,
 		bool nodeEvents) const;
 	NsPushEventSource *dbt2events(Transaction *txn, bool needsValidation,
@@ -245,35 +290,39 @@ private:
 	NsPushEventSource *dom2events(Transaction *txn, bool needsValidation,
 		bool nodeEvents) const;
 	NsPushEventSource *stream2events(Transaction *txn, bool needsValidation,
-		bool nodeEvents) const;
+		bool nodeEvents, const ISNVector *isns) const;
 
 	// @}
 
-	void createBlankDOM() const;
-	void createNsObjects() const;
+	void createNsObjects(bool resetDom = true) const;
 
 private:
 	/** @name Document ID attributes */
 
 	mutable XmlManager mgr_;
-	TransactedContainer *container_;
 	DocID id_;
+	int cid_; // container id
 	XMLCh *documentURI_;
 	int validation_;
+	mutable CacheDatabaseHandle docdb_;
+	mutable CacheDatabaseMinder dbMinder_;
 
 	typedef std::set<ReferenceMinder*> RefMinders;
-	RefMinders refMinders_; ///< The ReferenceMinder objects that are pointing to us
+	RefMinders refMinders_; // ReferenceMinder objects that point to us
 
 	/** @name Metadata attributes */
 
 	MetaData metaData_;
 
 	/** @name Lazy document attributes */
-
+	// The sense of this is reverse in that the flag says
+	// what is "lazy" as in not yet materialized.  Also,
+	// the NEITHER state is the initial state of the Document object,
+	// where it has no definitiveContent_
 	enum LazyMode {
-		NEITHER,
-		CONTENT,
-		BOTH
+		NEITHER, // means that content and m-d are materialized
+		CONTENT, // means that content has not been materialized
+		BOTH     // means that neither has been materialized
 	};
 
 	LazyMode lazy_;
@@ -284,6 +333,7 @@ private:
 
 	mutable Content definitiveContent_;
 	mutable bool contentModified_;
+	mutable bool consumed_;
 
 	/** @name content as dbt attributes */
 
@@ -297,10 +347,12 @@ private:
 
 	mutable XmlEventReader *reader_;
 
-	/** @name content as DOM attributes */
+	/** @name content as NsDom attributes */
 
 	mutable NsDocument *nsDocument_;
-	mutable NsXDOMFactory *nsDomFactory_;
+
+	// allow DocumentDatabase to access private methods
+	friend class DocumentDatabase;
 };
 
 }
