@@ -1,13 +1,11 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
-//
-// $Id: DocumentDatabase.cpp,v 1.47.2.1.2.1 2008/04/08 15:06:10 gmf Exp $
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
 
+#include "DbXmlInternal.hpp"
 #include <sstream>
-
 #include <dbxml/XmlException.hpp>
 #include "Container.hpp"
 #include "DocumentDatabase.hpp"
@@ -20,56 +18,53 @@
 #include "NameID.hpp"
 #include "db_utils.h"
 #include "nodeStore/NsEvent.hpp"
+#include "nodeStore/NsDocument.hpp"
 
 using namespace DbXml;
 using namespace std;
 static const char *document_name = "document";
 
-DocumentDatabase::DocumentDatabase(DbEnv *env, Transaction *txn,
+DocumentDatabase::DocumentDatabase(DB_ENV *env, Transaction *txn,
 				   const std::string &name,
-				   XmlContainer::ContainerType type,
-				   u_int32_t pageSize, u_int32_t flags,
-				   int mode)
+				   const ContainerConfig &config,
+				   XmlCompression *compression)
 	: environment_(env),
 	  name_(name),
-	  type_(type),
+	  type_(config.getContainerType()),
 	  content_(env, name, "content_", document_name,
-		   pageSize, flags & DB_XA_CREATE),
-	  secondary_(env, name, document_name, pageSize,
-		     flags & DB_XA_CREATE)
+		   DEFAULT_CONFIG),
+	  secondary_(env, name, document_name, DEFAULT_CONFIG),
+	  compressor_(compression)
 {
-	flags &= ~DB_XA_CREATE;
-	open(txn, flags, mode);
+	open(txn, config);
 }
 
-DocumentDatabase::DocumentDatabase(DbEnv *env, const std::string &name,
-				   XmlContainer::ContainerType type)
+DocumentDatabase::DocumentDatabase(DB_ENV *env, const std::string &name,
+				   XmlContainer::ContainerType type,
+				   XmlCompression *compression)
 	: environment_(env),
 	  name_(name),
 	  type_(type),
-	  content_(env, name, "content_", document_name, 0, 0),
-	  secondary_(env, name, document_name, 0, 0)
+	  content_(env, name, "content_", document_name, DEFAULT_CONFIG),
+	  secondary_(env, name, document_name, DEFAULT_CONFIG),
+	  compressor_(compression)
 {
-	open(0, 0, 0);
+	open(0, DEFAULT_CONFIG);
 }
 
 void
-DocumentDatabase::open(Transaction *txn, u_int32_t flags, int mode)
+DocumentDatabase::open(Transaction *txn, const ContainerConfig &config)
 {
 	int err = 0;
 	try {
 		// Open the Db objects
 		if (type_ == XmlContainer::WholedocContainer) {
-			err = content_.open(txn, DB_BTREE, flags, mode);
+			err = content_.open(txn, DB_BTREE, config);
 		}
 		if (err == 0)
 			err = secondary_.open(txn, /*duplicates*/false,
-					      flags, mode);
-	} catch (DbException &e) {
-		err = e.get_errno();
-		// fall through
-	}
-	catch (XmlException &xe) {
+					      config);
+	} catch (XmlException &xe) {
 		if (txn)
 			txn->abort();
 		throw;
@@ -106,31 +101,45 @@ int DocumentDatabase::getContent(OperationContext &context,
 		document->getID().setDbtFromThis(context.key());
 		// NOTE: db_.get() does not throw on DB_NOTFOUND, but
 		// try/catch anyway...
-		err = content_.get(context.txn(),
-			&context.key(), data, flags);
+		err = getContent(context.txn(), context.key(), *data, flags);
 	} catch (...) {
 		delete data;
 		throw; // re-throw
 	}
-
-	if(err == 0 && (data->get_size() != 0)) {
+ 
+	if(err == 0 && (data->size != 0)) 
 		document->setContentAsDbt(&data); // Note: consumes data
-	} else {
+	else 
 		delete data;
-	}
 	if (err == DB_NOTFOUND)
 		err = 0; // allow no-content documents
 	return err;
 }
 
-// object is donated to caller
-int DocumentDatabase::createDocumentCursor(
-	Transaction *txn, ScopedPtr<DocumentCursor> &cursor, u_int32_t flags) const
+int DocumentDatabase::getContent(Transaction *txn, DbtOut &key, DbtOut &data,
+			       u_int32_t flags) const
 {
-	DocumentDatabaseCursor *newCursor =
-		new DocumentDatabaseCursor(txn, content_, flags);
-	cursor.reset(newCursor);
-	return 0;
+	int err = 0;
+	// NOTE: db_.get() does not throw on DB_NOTFOUND, but
+	// try/catch anyway...
+	err = content_.get(txn, &key, &data, flags);
+ 
+	if(err == 0 && (data.size != 0)) {
+		if (compressor_) {
+			XmlData source(data.data, data.size);
+			XmlData dest;
+			XmlTransaction xtxn(txn);
+			if(!compressor_->decompress(xtxn, source, dest)){
+				throw XmlException(XmlException::INTERNAL_ERROR,
+						   "Error while tring to decompress your XML document.");
+			}
+			/* Replace the buffer in data with the one in dest.  Because
+			of this, DbtOut and Buffer must both use malloc and free*/
+			if (data.data != dest.get_data()) 
+				data.setNoCopy((*dest.getBuffer())->donateBuffer(), dest.get_size());
+		}
+	} 
+	return err;
 }
 
 u_int32_t DocumentDatabase::getPageSize() const
@@ -155,21 +164,39 @@ int DocumentDatabase::deleteID(OperationContext &context, const DocID& oldId)
 int DocumentDatabase::addContent(Document &document, UpdateContext &context)
 {
 	OperationContext &oc = context.getOperationContext();
-	
-	Dbt *dbt = (Dbt*) document.getContentAsDbt();
+
+	DbXmlDbt *dbt = (DbXmlDbt*) document.getContentAsDbt();
 	// Allow no-content documents
 	int err = 0;
-	if (dbt && (dbt->get_size() != 0)) {
+	if (dbt && (dbt->size != 0)) {
 		document.getID().setDbtFromThis(oc.key());
-		err = content_.put(oc.txn(), &oc.key(),
-			dbt, 0);
+		return addContent(oc.txn(), oc.key(), dbt, 0);
 	}
 	return err;
 }
 
+int DocumentDatabase::addContent(Transaction *txn, DbtOut &key, DbXmlDbt *data, u_int32_t flags)
+{
+	XmlData dest;
+	DbXmlDbt compDbt;
+	if(compressor_){
+		XmlData source(data->data, data->size);
+		XmlTransaction xtxn(txn);
+		if(!compressor_->compress(xtxn, source, dest)){
+			throw XmlException(XmlException::INTERNAL_ERROR,
+					   "Error while tring to compress your XML document.");
+		}
+		compDbt.data = dest.get_data();
+		compDbt.size = (u_int32_t)(dest.get_size());
+		data = &compDbt;
+	}
+	return content_.put(txn, &key, data, flags);
+}
+
 int DocumentDatabase::updateContentAndIndex(Document &new_document,
 					    UpdateContext &context,
-					    KeyStash &stash)
+					    KeyStash &stash, bool validate,
+					    bool updateStats)
 {
 	OperationContext &oc = context.getOperationContext();
 	DocID id = new_document.getID();
@@ -178,49 +205,44 @@ int DocumentDatabase::updateContentAndIndex(Document &new_document,
 	int err = 0;
 	bool resetId = false;
 
-	// NOTE: consider adding DB_RMW if transacted.
+	// Add DB_RMW if transacted to reduce deadlocks
+	u_int32_t flags =  ((oc.txn() && !content_.isCDBEnv()) ? DB_RMW : 0);
 	
 	// Check to see if the old document exists, first
 	// If ID is non-zero, let's trust it.  If not, get by name.
 	// Retrieve the old document
 	XmlDocument old_document;
-
 	if (id == 0) {
 		// will throw if name is bad or doc doesn't exist
 		err = indexer.getContainer()->getDocument(
-			oc, new_document.getName(), old_document, DBXML_LAZY_DOCS);
+			oc, new_document.getName(), old_document, flags);
 		if (err == 0) {
 			id = ((Document&)old_document).getID();
 			new_document.getIDToSet() = id;
 			resetId = true;
-			// clear modified flag if set on name
-			const_cast<Document*>(&new_document)->
-				clearModified(Name(metaDataName_uri_name));
 		}
 	} else {
 		err = indexer.getContainer()->getDocument(
-			oc, id, old_document, DBXML_LAZY_DOCS);
+			oc, id, old_document, flags);
 	}
 	if(err != 0) return err;
 	
-	// If this document's content is a stream, convert it to a dbt,
-	// because streams can only be used once... - jpcs
-	if(new_document.getDefinitiveContent() == Document::INPUTSTREAM) {
-		new_document.getContentAsDbt();
-	}
-
 	// Index the new document
 	indexer.indexMetaData(index, new_document, stash, true);
 	if(new_document.isContentModified()) {
+		// Will need the Dbt later, so get it now.  This also
+		// allows the content to be used more than once in case
+		// definitiveContent_ is INPUTSTREAM
+		(void) new_document.getContentAsDbt();
+		// use private method that explicitly assumes Dbt content
 		ScopedPtr<NsPushEventSource>
-			source(new_document.
-			       getContentAsEventSource(
-				       oc.txn(), /*needsValidation*/true,
-				       indexer.getContainer()->getIndexNodes()));
+			source(new_document.dbt2events(
+				       oc.txn(), /*needsValidation*/validate,
+				       indexer.getContainer()->nodesIndexed()));
 		if (source.get()) {
 			indexer.initIndexContent(index, id,
-						 source.get(), stash,
-						 true);
+				source.get(), stash, updateStats,
+				true, /*isDelete*/false);
 			source.get()->start();
 		}
 	}
@@ -248,11 +270,11 @@ int DocumentDatabase::updateContentAndIndex(Document &new_document,
 			       getContentAsEventSource(
 				       oc.txn(),
 				       /*needsValidation*/false,
-				       indexer.getContainer()->getIndexNodes()));
+				       indexer.getContainer()->nodesIndexed()));
 		if (source.get()) {
 			indexer.initIndexContent(delete_index, id,
-						 source.get(), stash,
-						 false);
+				source.get(), stash, updateStats,
+				false, /*isDelete*/true);
 			source.get()->start();
 		}
 	}
@@ -261,8 +283,8 @@ int DocumentDatabase::updateContentAndIndex(Document &new_document,
 	if(new_document.isContentModified()) {
 		OperationContext &oc = context.getOperationContext();
 		id.setDbtFromThis(oc.key());
-		err = content_.put(oc.txn(), &oc.key(),
-			(Dbt*)new_document.getContentAsDbt(), 0);
+		DbXmlDbt *dbt = (DbXmlDbt*)new_document.getContentAsDbt();
+		err = addContent(oc.txn(), oc.key(),dbt, 0);
 	}
 
 	if(err == 0) new_document.setContentModified(false);
@@ -273,7 +295,8 @@ int DocumentDatabase::updateContentAndIndex(Document &new_document,
 
 int DocumentDatabase::removeContentAndIndex(const Document &document,
 					    UpdateContext &context,
-					    KeyStash &stash)
+					    KeyStash &stash,
+					    bool updateStats)
 {
 	Indexer &indexer = context.getIndexer();
 	OperationContext &oc = context.getOperationContext();
@@ -286,11 +309,11 @@ int DocumentDatabase::removeContentAndIndex(const Document &document,
 	ScopedPtr<NsPushEventSource>
 		source(document.getContentAsEventSource(
 			       oc.txn(), /*needsValidation*/false,
-			       indexer.getContainer()->getIndexNodes()));
+			       indexer.getContainer()->nodesIndexed()));
 	if (source.get()) {
 		indexer.initIndexContent(index, document.getID(),
-					 source.get(), stash,
-					 false);
+					 source.get(), stash, updateStats,
+					 false, /*isDelete*/true);
 		source.get()->start();
 	}
 
@@ -305,7 +328,7 @@ int DocumentDatabase::removeContentAndIndex(const Document &document,
 }
 
 int DocumentDatabase::reindex(const IndexSpecification &is, const Document &document,
-	UpdateContext &context, KeyStash &stash)
+	UpdateContext &context, KeyStash &stash, bool updateStats)
 {
 	Indexer &indexer = context.getIndexer();
 	OperationContext &oc = context.getOperationContext();
@@ -314,10 +337,10 @@ int DocumentDatabase::reindex(const IndexSpecification &is, const Document &docu
 	indexer.indexMetaData(is, document, stash, /*checkModified*/false);
 	ScopedPtr<NsPushEventSource> source(document.
 		getContentAsEventSource(oc.txn(), /*needsValidation*/false,
-					indexer.getContainer()->getIndexNodes()));
+					indexer.getContainer()->nodesIndexed()));
 	if (source.get()) {
-		indexer.initIndexContent(is, document.getID(),
-					 source.get(), stash, false);
+		indexer.initIndexContent(is, document.getID(), source.get(),
+			stash, updateStats, false, /*isDelete*/false);
 		source.get()->start();
 	}
 
@@ -338,9 +361,8 @@ int DocumentDatabase::getAllMetaData(OperationContext &context,
 	// database.  Content is no longer considered metadata
 	//
 	Cursor cursor(const_cast<SecondaryDatabase&>(secondary_),
-		context.txn(), CURSOR_READ, flags);
-	flags = DB_CURSOR_GET_MASK(flags);
-	orig_flags = DB_CURSOR_GET_MASK(flags);
+		context.txn(), CURSOR_READ, "DocumentMetaData", flags);
+	orig_flags = flags = DB_CURSOR_GET_MASK(const_cast<SecondaryDatabase&>(secondary_),flags);
 	flags |= DB_SET_RANGE;
 	bool done = false;
 	while (!done) {
@@ -348,7 +370,7 @@ int DocumentDatabase::getAllMetaData(OperationContext &context,
 		DbtIn none;
 		none.set_flags(DB_DBT_PARTIAL); // Don't pull back the data.
 		// if DB_RMW set, don't get locks on this get, wait for a match
-		err = cursor.get(&context.key(), &none, flags & (~DB_RMW));
+		err = cursor.get(context.key(), none, flags & (~DB_RMW));
 		if (err == 0) {
 			DocID db_did;
 			NameID db_nid;
@@ -360,7 +382,7 @@ int DocumentDatabase::getAllMetaData(OperationContext &context,
 				err = dictionary->lookupNameFromID(context, db_nid, name);
 				if(err == 0 && !document->containsMetaData(name)) {
 					DbtOut *data = new DbtOut();
-					err = cursor.get(&context.key(), data,
+					err = cursor.get(context.key(), *data,
 							 DB_CURRENT);
 					if(err == 0) document->setMetaData(
 						name, type, &data,
@@ -384,7 +406,7 @@ int DocumentDatabase::getAllMetaData(OperationContext &context,
 
 int DocumentDatabase::getMetaData(OperationContext &context,
 	DictionaryDatabase *dictionary, const Name &name,
-	const DocID &did, XmlValue::Type &type, Dbt *metadata,
+	const DocID &did, XmlValue::Type &type, DbXmlDbt *metadata,
 	u_int32_t flags) const
 {
 	NameID nid;
@@ -392,12 +414,12 @@ int DocumentDatabase::getMetaData(OperationContext &context,
 					       nid, /*define=*/false);
 	if(err == 0) {
 		Cursor cursor(const_cast<SecondaryDatabase&>(secondary_),
-			context.txn(), CURSOR_READ, flags);
-		u_int32_t origFlags = DB_CURSOR_GET_MASK(flags);
+			context.txn(), CURSOR_READ, "DocumentMetaData", flags);
+		u_int32_t origFlags = DB_CURSOR_GET_MASK(const_cast<SecondaryDatabase&>(secondary_),flags);
 		MetaDatum::setKeyDbt(did, nid, XmlValue::NONE, context.key());
 		DbtIn none;
 		none.set_flags(DB_DBT_PARTIAL); // Don't pull back the data.
-		err = cursor.get(&context.key(), &none,
+		err = cursor.get(context.key(), none,
 				 (flags | DB_SET_RANGE) & ~DB_RMW);
 		if (err == 0) {
 			DocID db_did;
@@ -405,7 +427,7 @@ int DocumentDatabase::getMetaData(OperationContext &context,
 			MetaDatum::decodeKeyDbt(context.key(), db_did,
 						db_nid, type);
 			if(db_did == did && db_nid == nid) {
-				err = cursor.get(&context.key(), metadata,
+				err = cursor.get(context.key(), *metadata,
 						 origFlags|DB_CURRENT);
 			} else {
 				return DB_NOTFOUND;
@@ -507,7 +529,7 @@ int DocumentDatabase::removeMetaData(OperationContext &oc,
 				     std::vector<NameID> *toRemove)
 {
 	Cursor cursor(const_cast<SecondaryDatabase&>(secondary_), oc.txn(),
-		      CURSOR_WRITE);
+		      CURSOR_WRITE, "DocumentMetaData_remove");
 
 	DbtIn none;
 	none.set_flags(DB_DBT_PARTIAL); // Don't pull back the data.
@@ -515,7 +537,7 @@ int DocumentDatabase::removeMetaData(OperationContext &oc,
 	id.setDbtFromThis(oc.key());
 
 	DocID db_id;
-	int err = cursor.get(&oc.key(), &none, DB_SET_RANGE);
+	int err = cursor.get(oc.key(), none, DB_SET_RANGE);
 	try {
 		while(err == 0) {
 			if (toRemove) {
@@ -531,13 +553,12 @@ int DocumentDatabase::removeMetaData(OperationContext &oc,
 			}
 			if (id != db_id) // done with document?
 				break;
-			err = cursor.get(&oc.key(), &none, DB_NEXT);
+			err = cursor.get(oc.key(), none, DB_NEXT);
 		}
 	} catch (...) {
 		cursor.close();
 		throw;
 	}
-
 	if(err == DB_NOTFOUND) {
 		err = 0;
 	}
@@ -545,12 +566,12 @@ int DocumentDatabase::removeMetaData(OperationContext &oc,
 	return err;
 }
 
-int DocumentDatabase::dump(DbEnv *env, const std::string &name,
+int DocumentDatabase::dump(DB_ENV *env, const std::string &name,
 			   XmlContainer::ContainerType type,
 			   std::ostream *out)
 {
-	DbWrapper content(env, name, "content_", document_name, 0, 0);
-	SecondaryDatabase secondary(env, name, document_name, 0, 0);
+	DbWrapper content(env, name, "content_", document_name, DEFAULT_CONFIG);
+	SecondaryDatabase secondary(env, name, document_name, DEFAULT_CONFIG);
 	int err = 0;
 	if (type == XmlContainer::WholedocContainer) {
 		err = Container::writeHeader(content.getDatabaseName(), out);
@@ -565,12 +586,12 @@ int DocumentDatabase::dump(DbEnv *env, const std::string &name,
 	return err;
 }
 
-int DocumentDatabase::load(DbEnv *env, const std::string &name,
+int DocumentDatabase::load(DB_ENV *env, const std::string &name,
 			   XmlContainer::ContainerType type,
 			   std::istream *in, unsigned long *lineno)
 {
-	DbWrapper content(env, name, "content_", document_name, 0, 0);
-	SecondaryDatabase secondary(env, name, document_name, 0, 0);
+	DbWrapper content(env, name, "content_", document_name, DEFAULT_CONFIG);
+	SecondaryDatabase secondary(env, name, document_name, DEFAULT_CONFIG);
 
 	int err = 0;
 	if (type == XmlContainer::WholedocContainer) {
@@ -600,52 +621,108 @@ int DocumentDatabase::load(DbEnv *env, const std::string &name,
 	return err;
 }
 
-int DocumentDatabase::verify(DbEnv *env, const std::string &name,
+int DocumentDatabase::verify(DB_ENV *env, const std::string &name,
 			     XmlContainer::ContainerType type,
 			     std::ostream *out, u_int32_t flags)
 {
-	DbWrapper content(env, name, "content_", document_name, 0, 0);
-	SecondaryDatabase secondary(env, name, document_name, 0, 0);
+	DbWrapper content(env, name, "content_", document_name, DEFAULT_CONFIG);
+	SecondaryDatabase secondary(env, name, document_name, DEFAULT_CONFIG);
 
 	int err = 0;
+	int terr = 0;
 	if (type == XmlContainer::WholedocContainer) {
 		if(flags & DB_SALVAGE)
-			err = Container::writeHeader(content.getDatabaseName(),
-						     out);
-		if(err == 0)
-			err = content.verify(out, flags);
+			terr = Container::writeHeader(content.getDatabaseName(),
+						      out);
+		err = terr;
+		terr = content.verify(out, flags);
+		if (terr)
+			err = terr;
 	}
-	if(err == 0 && (flags & DB_SALVAGE))
-		err = Container::writeHeader(secondary.getDatabaseName(), out);
-	if(err == 0)
-		err = secondary.verify(out, flags);
+	if(flags & DB_SALVAGE)
+		terr = Container::writeHeader(secondary.getDatabaseName(), out);
+	if(terr == 0)
+		err = terr;
+	terr = secondary.verify(out, flags);
+	if(terr == 0)
+		err = terr;
 	return err;
 }
 
-DocumentDatabaseCursor::DocumentDatabaseCursor(Transaction *txn,
-					       DbWrapper &db, u_int32_t flags) :
-	cursor_(db, txn, CURSOR_READ, flags),
-	cursorFlags_(DB_CURSOR_GET_MASK(flags)),
-	done_(false) {
-	data_.set_flags(DB_DBT_PARTIAL); // only want keys (for now)
-}
+// DocumentDatabaseCursor -- shared with NsDocumentDatabase since
+// it iterates over the (shared) metadata database (secondary_)
+class DocumentDatabaseCursor : public DocumentCursor
+{
+public:
+	DocumentDatabaseCursor(Transaction *txn, DbWrapper &db, u_int32_t flags)
+		: cursor_(db, txn, CURSOR_READ, "DocumentDatabaseCursor", flags),
+		  cursorFlags_(DB_CURSOR_GET_MASK(db,flags)),
+		  done_(false)
+	{
+		data_.set_flags(DB_DBT_PARTIAL); // only want keys (for now)
+	}
+	virtual int first(DocID &id)
+	{
+		return next(id);
+	}
+	virtual int next(DocID &id)
+	{
+		int err = 0;
+		if(done_) {
+			id = 0;
+		} else {
+			while((err = cursor_.get(key_, data_, cursorFlags_|DB_NEXT)) == 0) {
+				id.setThisFromDbt(key_);
+				if(id != last_) {
+					last_ = id;
+					break;
+				}
+			}
+			if (err == DB_NOTFOUND || err == DB_KEYEMPTY) {
+				err = 0;
+				done_ = true;
+				id = 0;
+			}
+		}
+		return err;
+	}
+	virtual int seek(DocID &id)
+	{
+		if (last_ >= id) return next(id);
 
-int DocumentDatabaseCursor::next(DocID &id) {
-	int err = 0;
-	if (done_) {
-		id = 0;
-	} else {
-		err = cursor_.get(&key_, &data_,
-				  cursorFlags_|DB_NEXT_NODUP); // no throw
-		if (err == 0)
-			id.setThisFromDbt(key_);
-		if (err == DB_NOTFOUND || err == DB_KEYEMPTY) {
+		id.setDbtFromThis(key_);
+		// DB_SET_RANGE does >= get on the key (which is the id)
+		int err = cursor_.get(key_, data_, DB_SET_RANGE);
+		if(err == DB_NOTFOUND) {
 			err = 0;
 			done_ = true;
 			id = 0;
+		} else if(err == 0) {
+			last_.setThisFromDbt(key_);
+			DBXML_ASSERT(last_ >= id);
+			id = last_; // let caller know where it landed
 		}
+		return err;
 	}
-	return err;
+private:
+	Cursor cursor_;
+	u_int32_t cursorFlags_;
+	bool done_;
+	DocID last_;
+	DbtOut key_;
+	DbtOut data_;
+};
+
+// object is donated to caller
+int DocumentDatabase::createDocumentCursor(
+	Transaction *txn, ScopedPtr<DocumentCursor> &cursor, u_int32_t flags) const
+{
+	DocumentDatabaseCursor *newCursor =
+		new DocumentDatabaseCursor(txn,
+					   const_cast<SecondaryDatabase&>(secondary_),
+					   flags);
+	cursor.reset(newCursor);
+	return 0;
 }
 
 // upgrade from 2.1 (version 4) to 2.2 (version 5)
@@ -656,29 +733,30 @@ int DocumentDatabaseCursor::next(DocID &id) {
 // 1.  open old DB and create a new database
 // 2.  for each record:
 //    o unmarshal old format; marshal in new, and put into new DB
-// 3.  remove old DB
-// 4.  rename new DB to correct name
-static int upgrade22(const std::string &name, Manager &mgr)
+static int upgrade22(const std::string &name,
+		     const std::string &tname, Manager &mgr)
 {
 	int err = 0;
-	SecondaryDatabase secondary(mgr.getDbEnv(), name,
-				    document_name, 0, 0);
-	err = secondary.open(0, /*duplicates*/false, 0, 0);
-	SecondaryDatabase newSec(mgr.getDbEnv(), name,
-				 "newdoc", secondary.getPageSize(), 0);
-	err = newSec.open(0, /*duplicates*/false, DB_CREATE|DB_EXCL, 0);
+	SecondaryDatabase secondary(mgr.getDB_ENV(), name,
+				    document_name, DEFAULT_CONFIG);
+	err = secondary.open(0, /*duplicates*/false, DEFAULT_CONFIG);
+	ContainerConfig config;
+	config.setPageSize(secondary.getPageSize());
+	SecondaryDatabase newSec(mgr.getDB_ENV(), tname,
+				 document_name, config);
+	err = newSec.open(0, /*duplicates*/false, CREATE_CONFIG);
 	// the key is: docId,nodeId,type
-	Dbt key;
-	Dbt data;
+	DbXml::DbXmlDbt key;
+	DbXml::DbXmlDbt data;
 	Cursor curs(secondary, (Transaction*)0, DbXml::CURSOR_READ);
 	int ret = 0;
 	while ((err == 0) &&
-	       (ret = curs.get(&key, &data, DB_NEXT)) == 0) {
+	       (ret = curs.get(key, data, DB_NEXT)) == 0) {
 		// decode key using pre-2.2 code
 		DocID did;
 		NameID nid;
 		XmlValue::Type type;
-		u_int32_t *p = (u_int32_t *)key.get_data();
+		u_int32_t *p = (u_int32_t *)key.data;
 		u_int32_t id1, id2;
 		id1 = *p++;
 		id2 = *p++;
@@ -698,73 +776,45 @@ static int upgrade22(const std::string &name, Manager &mgr)
 	secondary.close(0);
 	newSec.close(0);
 	if (err == 0) {
-		// remove old db, rename new one
-		std::string dbname("secondary_");
-		std::string tname("secondary_");
-		dbname += document_name;
-		tname += "newdoc";
-		err = mgr.getDbEnv()->dbremove(0, name.c_str(),
-					       dbname.c_str(), 0);
-		if (err == 0)
-			err = mgr.getDbEnv()->dbrename(0, name.c_str(),
-						       tname.c_str(),
-						       dbname.c_str(), 0);
+		DbWrapper content(mgr.getDB_ENV(), name,
+				  "content_",
+				  document_name, DEFAULT_CONFIG);
+		err = content.open(0, DB_BTREE, DEFAULT_CONFIG);
+		if (err != 0) {
+			// ignore ENOENT
+			if (err == ENOENT)
+				err = 0;
+			return err;
+		}
+		config.setPageSize(content.getPageSize());
+		DbWrapper newContent(mgr.getDB_ENV(), tname,
+				     "content_", document_name,
+				     config);
+		err = newContent.open(0, DB_BTREE,
+				      CREATE_CONFIG);
 		if (err != 0)
 			return err;
-		try {
-			DbWrapper content(mgr.getDbEnv(), name,
-					  "content_",
-					  document_name, 0, 0);
-			err = content.open(0, DB_BTREE, 0, 0);
-			if (err != 0) {
-				// ignore ENOENT
-				if (err == ENOENT)
-					err = 0;
-				return err;
-			}
-			DbWrapper newContent(mgr.getDbEnv(), name,
-					     "newcontent_", document_name,
-					     content.getPageSize(), 0);
-			err = newContent.open(0, DB_BTREE,
-					      DB_CREATE|DB_EXCL, 0);
-			if (err != 0)
-				return err;
-			// the key is: docId
-			Dbt key;
-			Dbt data;
-			Cursor curs1(content, (Transaction*)0, DbXml::CURSOR_READ);
-			int ret = 0;
-			while ((err == 0) &&
-			       (ret = curs1.get(&key, &data, DB_NEXT)) == 0) {
-				// decode/encode key, rewrite
-				u_int32_t *p = (u_int32_t *)key.get_data();
-				u_int32_t id1;
-				id1 = *p;
-				if (Manager::isBigendian())
-					M_32_SWAP(id1);
-				DocID id(id1);
-				DbtOut newKey;
-				id.setDbtFromThis(newKey);
-				err = newContent.put(0, &newKey, &data, 0);
-			}
-			curs1.close();
-			content.close(0);
-			newContent.close(0);
-			if (err == 0) {
-				std::string dbname("content_");
-				std::string tname("newcontent_");
-				dbname += document_name;
-				tname += document_name;
-				// remove old db, rename new one
-				err = mgr.getDbEnv()->dbremove(
-					0, name.c_str(), dbname.c_str(), 0);
-				if (err == 0)
-					err = mgr.getDbEnv()->dbrename(
-						0, name.c_str(), tname.c_str(),
-						dbname.c_str(), 0);
-			}
+		// the key is: docId
+		DbXml::DbXmlDbt key;
+		DbXml::DbXmlDbt data;
+		Cursor curs1(content, (Transaction*)0, DbXml::CURSOR_READ);
+		int ret = 0;
+		while ((err == 0) &&
+		       (ret = curs1.get(key, data, DB_NEXT)) == 0) {
+			// decode/encode key, rewrite
+			u_int32_t *p = (u_int32_t *)key.data;
+			u_int32_t id1;
+			id1 = *p;
+			if (Manager::isBigendian())
+				M_32_SWAP(id1);
+			DocID id(id1);
+			DbtOut newKey;
+			id.setDbtFromThis(newKey);
+			err = newContent.put(0, &newKey, &data, 0);
 		}
-		catch (DbException &de) {} // ignore
+		curs1.close();
+		content.close(0);
+		newContent.close(0);
 	}
 	return err;
 }
@@ -790,39 +840,53 @@ static bool needsSwap(u_int32_t val)
 }
 
 // static
-void DocumentDatabase::upgrade(const std::string &name, Manager &mgr,
+void DocumentDatabase::upgrade(const std::string &name,
+			       const std::string &tname, Manager &mgr,
 			       int old_version, int current_version)
 {
 	int err = 0;
 	DBXML_ASSERT(old_version < current_version);
 	// 2.0 -> 2.1 upgrade, if necessary, must be first
 	if (old_version == VERSION_20){
-		SecondaryDatabase secondary(mgr.getDbEnv(), name,
-					    document_name, 0, 0);
-		err = secondary.open(0, /*duplicates*/false, 0, 0);
+		SecondaryDatabase secondary(mgr.getDB_ENV(), name,
+					    document_name, DEFAULT_CONFIG);
+		err = secondary.open(0, /*duplicates*/false, DEFAULT_CONFIG);
 		// byte-swap name ids in document metadata keys.
 		// iterate through all records in secondary.
 		// the key is: docId,nodeId,type
-		Dbt key;
-		Dbt data;
+		DbXmlDbt key;
+		DbXmlDbt data;
 		Cursor curs(secondary, (Transaction*)0, DbXml::CURSOR_WRITE);
 		int ret = 0;
 		while ((err == 0) &&
-		       (ret = curs.get(&key, &data, DB_NEXT)) == 0) {
-			u_int32_t *p = reinterpret_cast<u_int32_t*>(key.get_data());
+		       (ret = curs.get(key, data, DB_NEXT)) == 0) {
+			u_int32_t *p = reinterpret_cast<u_int32_t*>(key.data);
 			++p;
 			// swap happens on second int, not first
 			if (!needsSwap(*p))
 				break;
 			M_32_SWAP(*p);
 			curs.del(0);
-			err = curs.put(&key, &data, DB_KEYFIRST);
+			err = curs.put(key, data, DB_KEYFIRST);
 		}
 	}
 	if (old_version < VERSION_22 && err == 0)
-		err = upgrade22(name, mgr);
-	if (err != 0)
-		throw XmlException(
-			XmlException::DATABASE_ERROR,
-			"Unexpected error upgrading Document DB");
+		err = upgrade22(name, tname, mgr);
+	else if (err == 0) {
+		err = DbWrapper::copySecondary(
+			mgr.getDB_ENV(),
+			name, tname, "secondary_",
+			document_name);
+		if (err == 0)
+			err = DbWrapper::copySecondary(
+				mgr.getDB_ENV(),
+				name, tname, "content_",
+				document_name);
+	}
+	if (err != 0) {
+		char buf[100];
+		sprintf(buf, "Unexpected error upgrading Document DB: %d",
+			err);
+		throw XmlException(XmlException::DATABASE_ERROR, buf);
+	}
 }
