@@ -1,9 +1,7 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
-//
-// $Id: DocumentDatabase.hpp,v 1.28 2006/10/31 23:39:51 jsnelson Exp $
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
 
 #ifndef __DOCUMENTDATABASE_HPP
@@ -11,7 +9,7 @@
 
 #include <string>
 #include <sstream>
-#include <db_cxx.h>
+#include <db.h>
 #include "SharedPtr.hpp"
 #include "DbWrapper.hpp"
 #include "Cursor.hpp"
@@ -40,17 +38,23 @@ class DocumentDatabase
 public:
 	typedef SharedPtr<DocumentDatabase> Ptr;
 
-	DocumentDatabase(DbEnv *env, Transaction *txn,
+	DocumentDatabase(DB_ENV *env, Transaction *txn,
 			 const std::string &name,
-			 XmlContainer::ContainerType type,
-			 u_int32_t pageSize, u_int32_t flags,
-			 int mode);
+			 const ContainerConfig &config,
+			 XmlCompression *compression);
 	// used for defaulted, read-only access to existing database
-	DocumentDatabase(DbEnv *env, const std::string &name,
-			 XmlContainer::ContainerType type);
+	DocumentDatabase(DB_ENV *env, const std::string &name,
+			 XmlContainer::ContainerType type,
+			 XmlCompression *compression);
 
 	virtual ~DocumentDatabase();
 
+	DbWrapper &getContentDatabase()
+	{
+		return content_;
+	}
+
+	virtual DbWrapper *getNodeDatabase() { return 0; }
 	virtual u_int32_t getPageSize() const; // page size of content db
 	virtual unsigned long getNumberOfPages() const;
 	virtual int deleteID(OperationContext &context, const DocID& oldId);
@@ -59,26 +63,28 @@ public:
 					 u_int32_t flags) const;
 	virtual int getContent(OperationContext &context, Document *document,
 			       u_int32_t flags) const;
+	int getContent(Transaction *txn, DbtOut &key, DbtOut &data,
+		       u_int32_t flags) const;
 	virtual int updateContentAndIndex(Document &new_document,
 					  UpdateContext &context,
-					  KeyStash &stash);
+					  KeyStash &stash, bool validate,
+					  bool updateStats);
 	/// Also deallocates the ID
 	virtual int removeContentAndIndex(const Document &document,
 					  UpdateContext &context,
-					  KeyStash &stash);
+					  KeyStash &stash, bool updateStats);
 
 	virtual int reindex(const IndexSpecification &is, const Document &document,
-		UpdateContext &context, KeyStash &stash);
-
+			    UpdateContext &context, KeyStash &stash, bool updateStats);
 
 	virtual int getAllMetaData(OperationContext &context,
 				   DictionaryDatabase *dictionary,
 				   Document *document, u_int32_t flags) const;
 	virtual int getMetaData(OperationContext &context,
-		DictionaryDatabase *dictionary,
-		const Name &name, const DocID &did,
-		XmlValue::Type &type, Dbt *metadata,
-		u_int32_t flags) const;
+				DictionaryDatabase *dictionary,
+				const Name &name, const DocID &did,
+				XmlValue::Type &type, DbXmlDbt *metadata,
+				u_int32_t flags) const;
 	virtual int addMetaData(OperationContext &oc,
 				DictionaryDatabase *dictionary,
 				Document &document);
@@ -98,6 +104,7 @@ public:
 	}
 
 	virtual int addContent(Document &document, UpdateContext &context);
+	int addContent(Transaction *txn, DbtOut &key, DbXmlDbt *data, u_int32_t flags);
 	
 	virtual void addContentException(Document &document,
 					 UpdateContext &context, bool isDbExc) {}
@@ -108,37 +115,38 @@ public:
 		runner.run(txn, secondary_);
 	}
 
-	static int dump(DbEnv *env, const std::string &name,
+	static int dump(DB_ENV *env, const std::string &name,
 			XmlContainer::ContainerType type,
 			std::ostream *out);
-	static int load(DbEnv *env, const std::string &name,
+	static int load(DB_ENV *env, const std::string &name,
 			XmlContainer::ContainerType type,
 			std::istream *in, unsigned long *lineno);
-	static int verify(DbEnv *env, const std::string &name,
+	static int verify(DB_ENV *env, const std::string &name,
 			  XmlContainer::ContainerType type,
 			  std::ostream *out, u_int32_t flags);
 	// upgrade
-	static void upgrade(const std::string &name, Manager &mgr,
+	static void upgrade(const std::string &name,
+			    const std::string &tname, Manager &mgr,
 			    int old_version, int current_version);
 
-	DbEnv *getDbEnv() { return environment_; }
+	DB_ENV *getDB_ENV() { return environment_; }
 
 	const std::string &getContainerName() const {
 		return name_;
 	}
-	const XmlContainer::ContainerType getContainerType() const {
+	XmlContainer::ContainerType getContainerType() const {
 		return type_;
 	}
 
 private:
-	void open(Transaction *txn, u_int32_t flags, int mode);
+	void open(Transaction *txn, const ContainerConfig &config);
 
 protected:
 	// no need for copy and assignment
 	DocumentDatabase(const DocumentDatabase&);
 	DocumentDatabase &operator=(const DocumentDatabase&);
 	
-	DbEnv *environment_;
+	DB_ENV *environment_;
 	std::string name_;
 	XmlContainer::ContainerType type_;
 	/**
@@ -148,23 +156,8 @@ protected:
 	 */
 	mutable DbWrapper content_;
 	SecondaryDatabase secondary_;
-};
 
-// Iterate through DocumentDatabase content databases,
-// pulling out keys (IDs)
-class DocumentDatabaseCursor : public DocumentCursor
-{
-public:
-	DocumentDatabaseCursor(Transaction *txn, DbWrapper &db, u_int32_t flags);
-	virtual ~DocumentDatabaseCursor() {}
-	virtual int first(DocID &id) { return next(id); }
-	virtual int next(DocID &id);
-private:
-	Cursor cursor_;
-	u_int32_t cursorFlags_;
-	bool done_;
-	DbtOut key_;
-	DbtOut data_;
+	XmlCompression *compressor_;
 };
 
 }
