@@ -1,17 +1,19 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: Globals.cpp,v 1.31 2006/11/16 17:03:23 gmf Exp $
 //
 
 #include "DbXmlInternal.hpp"
 #include "Globals.hpp"
 #include "Value.hpp"
+#include "Log.hpp"
 #include "IndexSpecification.hpp"
 #include "SyntaxManager.hpp"
+#include "QueryContext.hpp"
 #include "nodeStore/NsNid.hpp"
+#include <sstream>
 
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xqilla/utils/XQillaPlatformUtils.hpp>
@@ -38,13 +40,16 @@ Globals::NameToNumber *Globals::indexMap = 0;
 
 bool Globals::isBigendian_;
 int Globals::refCount_ = 0;
-void *Globals::mutex_ = 0;
+dbxml_mutex_t Globals::mutex_ = 0;
 DocumentCache *Globals::documentCache_ = 0;
 DatatypeLookup *Globals::datatypeLookup_ = 0;
 
 static SimpleMemoryManager staticMemoryManager;
 MemoryManager *Globals::defaultMemoryManager = &staticMemoryManager;
 
+static Counters staticCounters;
+Counters *Globals::counters_ = &staticCounters;
+
 static int bdb_major = DB_VERSION_MAJOR;
 static int bdb_minor = DB_VERSION_MINOR;
 static int bdb_patch = DB_VERSION_PATCH;
@@ -62,7 +67,7 @@ static void checkDbVersion()
 	}
 }
 
-void *SimpleMemoryManager::allocate(size_t size)
+void *SimpleMemoryManager::allocate(XercesSize size)
 {
 	return malloc(size);
 }
@@ -72,10 +77,16 @@ void SimpleMemoryManager::deallocate(void *p)
 	free(p);
 }
 
-void Globals::initialize(DbEnv *env)
+MemoryManager *SimpleMemoryManager::getExceptionMemoryManager()
+{
+  return XMLPlatformUtils::fgMemoryManager;
+}
+
+
+void Globals::initialize(DB_ENV *env)
 {
 	if (mutex_ == 0) {
-		mutex_ = XMLPlatformUtils::makeMutex(defaultMemoryManager);
+		mutex_ = MutexLock::createMutex();
 	}
 	MutexLock lock(mutex_);
 
@@ -90,7 +101,7 @@ void Globals::initialize(DbEnv *env)
 
 		// init Xerces and XQilla, and related structures
 		initializeXmlPlatform();
-
+		
 		// initialise the map of index names to values
 		indexMap = new NameToNumber;
 		(*indexMap)["none"] = Index::NONE;
@@ -103,9 +114,9 @@ void Globals::initialize(DbEnv *env)
 		(*indexMap)["equality"] = Index::KEY_EQUALITY;
 		(*indexMap)["presence"] = Index::KEY_PRESENCE;
 		(*indexMap)["substring"] = Index::KEY_SUBSTRING;
-		isBigendian_ = (env->get_DB_ENV()->is_bigendian()) ? true : false;
+		isBigendian_ = (env->is_bigendian()) ? true : false;
 		// initialise the document root NID
-		NsNid::initDocRootNid();
+		NsFullNid::initDocRootNid();
 	}
 }
 
@@ -153,6 +164,14 @@ void Globals::terminate()
 			// terminate XQilla and Xerces
 			XQillaPlatformUtils::terminate();
 			SyntaxManager::uninitSyntaxManager();
+			// log counters
+			if (Log::isLogEnabled(Log::C_MANAGER, Log::L_INFO)) {
+				ostringstream oss;
+				dumpCountersToStream(oss);
+				Log::log(0, //env *
+					 Log::C_MANAGER, Log::L_INFO,
+					 oss.str().c_str());
+			}
 		}
 	}
 	// Don't close the mutex, as it can't be done in a thread safe fashion
