@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: KeyGenerator.hpp,v 1.15 2006/10/30 17:45:52 bostic Exp $
 //
 
 #ifndef __KEYGENERATOR_HPP
@@ -24,6 +23,7 @@ public:
 	KeyGenerator() {}
 	virtual ~KeyGenerator() {}
 	virtual void set(const char *s, size_t l) = 0;
+	virtual void getWholeValue(const char *&p, size_t &pl) = 0;
 	virtual bool next(const char *&p, size_t &pl) = 0;
 	virtual size_t noOfKeys() = 0;
 };
@@ -34,7 +34,8 @@ public:
 	EmptyKeyGenerator() {}
 	virtual ~EmptyKeyGenerator() {}
 	virtual void set(const char *s, size_t l) {}
-	virtual bool next(const char *&p, size_t &pl)	{ return false; }
+	virtual void getWholeValue(const char *&p, size_t &pl) {}
+	virtual bool next(const char *&p, size_t &pl) { return false; }
 	virtual size_t noOfKeys() { return 0; }
 };
 
@@ -51,6 +52,11 @@ public:
 		s_= s;
 		l_= l;
 	}
+	virtual void getWholeValue(const char *&p, size_t &pl)
+	{
+		p= s_;
+		pl= l_;
+	}
 	virtual bool next(const char *&p, size_t &pl)
 	{
 		if(done_) {
@@ -73,7 +79,9 @@ private:
 class SubstringKeyGenerator : public KeyGenerator, private StringTransform
 {
 public:
-	SubstringKeyGenerator(const char *s, size_t l);
+	SubstringKeyGenerator(const char *s, size_t l, bool generateShortKeys);
+
+	virtual void getWholeValue(const char *&p, size_t &pl);
 
 	virtual void set(const char *s, size_t l);
 	virtual bool next(const char *&p, size_t &pl);
@@ -82,6 +90,7 @@ public:
 private:
 	virtual void pushChar(unsigned int ch);
 
+	bool generateShortKeys_;
 	Buffer buf_;
 	const char *p_;
 	size_t c_;
