@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2004,2006 Oracle.  All rights reserved.
+// Copyright (c) 2004,2009 Oracle.  All rights reserved.
 //
-//   $Id: MemBufInputStream.hpp,v 1.9 2006/10/30 17:45:53 bostic Exp $
 //
 
 #ifndef __MEMBUFINPUTSTREAM_HPP
@@ -14,7 +13,8 @@
 
 namespace DbXml
 {
-
+	
+class DbXmlDbt;
 class Document;
 
 /**
@@ -27,9 +27,15 @@ public:
 	MemBufInputStream(const char *srcDocBytes,
 			  const unsigned int byteCount,
 			  const char *const bufId,
-			  const bool adoptBuffer = false);
+			  bool adoptBuffer = false);
+	
+	MemBufInputStream(DbXmlDbt *dbt,
+			  const char *const bufId,
+			  bool adoptDbt = false);
 
 	virtual ~MemBufInputStream();
+private:
+	DbXmlDbt *dbt_;
 };
 
 }
