@@ -1,11 +1,11 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: MetaDatum.cpp,v 1.16 2006/10/30 17:45:53 bostic Exp $
 //
 
+#include "DbXmlInternal.hpp"
 #include <dbxml/XmlData.hpp>
 #include "MetaDatum.hpp"
 #include "Container.hpp"
@@ -64,8 +64,8 @@ void MetaDatum::setDbt(DbtOut **dbt) // Note: Consumes the Dbt
 const char *MetaDatum::getValue() const
 {
 	const char *r = 0;
-	if(dbt_ && dbt_->get_size() > 0) {
-		r = (const char *)dbt_->get_data();
+	if(dbt_ && dbt_->size > 0) {
+		r = (const char *)dbt_->data;
 	}
 	return r;
 }
@@ -85,10 +85,10 @@ void MetaDatum::decodeKeyDbt(const DbtOut &key, DocID &did, NameID &nid,
 {
 	// Key
 	//  did nid type
-	const char *p = (const char *)key.get_data();
+	const char *p = (const char *)key.data;
 	u_int32_t len = did.unmarshal(p);
 	len += nid.unmarshal(p+len);
-	if (len <= key.get_size())
+	if (len <= key.size)
 		type = (XmlValue::Type) * ((unsigned char*)(p+len));
 	else
 		type = XmlValue::NONE;
@@ -103,7 +103,7 @@ void MetaDatum::setKeyDbt(const DocID &did, const NameID &nid,
 	u_int32_t nlen = nid.marshal(nbuf);
 	size_t l = dlen + nlen + (type == XmlValue::NONE ? 0 : 1);
 	key.set(0, l);
-	Buffer b(key.get_data(), l, /*wrapper=*/true);
+	Buffer b(key.data, l, /*wrapper=*/true);
 	b.write(dbuf, dlen);
 	b.write(nbuf, nlen);
 	if (type != XmlValue::NONE) {
@@ -115,18 +115,19 @@ void MetaDatum::setKeyDbt(const DocID &did, const NameID &nid,
 void MetaDatum::setValueDbtFromThis(DbtIn &value)
 {
 
-	value.set_data(dbt_->get_data());
-	value.set_size(dbt_->get_size());
+	value.data = dbt_->data;
+	value.size = dbt_->size;
 }
 
 bool MetaDatum::canBeIndexed() const
 {
-  return  dbt_ != 0 &&
-	  type_ >= XmlValue::ANY_SIMPLE_TYPE &&
-	  !removed_;
+	return  (dbt_ != 0) &&
+		(type_ >= XmlValue::ANY_SIMPLE_TYPE) &&
+		!removed_;
 }
 
 void MetaDatum::asValue(XmlValue &value) const
 {
-	value = XmlValue(type_, *dbt_);
+	if (dbt_ != 0)
+		value = XmlValue(type_, XmlData(dbt_->data, dbt_->size));
 }
