@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: Name.cpp,v 1.29.4.1 2008/04/08 15:06:10 gmf Exp $
 //
 
 #include "DbXmlInternal.hpp"
@@ -15,6 +14,12 @@
 using namespace std;
 using namespace DbXml;
 
+//
+// these globals are used to make detection of these particular Name objects
+// efficient.  When used in construction or comparison of Name objects
+// new objects will point to these via the known_ member rather than
+// going to the expense of copying strings.
+//
 const Name Name::dbxml_colon_name(metaDataNamespace_uri, metaDataName_name);
 const Name Name::dbxml_colon_root(metaDataNamespace_uri, metaDataName_root);
 
@@ -95,6 +100,12 @@ bool Name::operator<(const Name &n) const
 
 bool Name::operator==(const Name &n) const
 {
+	// compare() is not all that efficient in
+	// the face of "known" objects, so bypass some code
+	if (known_ && (known_ == n.known_ || known_ == &n))
+		return true;
+	else if (n.known_ == this)
+		return true;
 	return (compare(n) == 0);
 }
 
@@ -111,7 +122,7 @@ const char *Name::getName() const
 void Name::setThisFromDbt(const DbtOut &dbt)
 {
 	// name \0 uri \0
-	const char *p = (const char*)dbt.get_data();
+	const char *p = (const char*)dbt.data;
 	size_t nl = strlen(p);
 	size_t ul = strlen(p + nl + 1);
 	set(p, nl, p + nl + 1, ul);
@@ -136,7 +147,7 @@ void Name::setDbtFromThis_SecondaryKey(DbtOut &dbt) const
 		size_t nl = (name_ == 0 ? 0 : strlen(name_));
 		size_t l = ul + (ul > 0 ? 1 : 0) + nl;
 		dbt.set(0, l);
-		Buffer b(dbt.get_data(), l, /*wrapper=*/true);
+		Buffer b(dbt.data, l, /*wrapper=*/true);
 		writeToBuffer(b, name_, nl, uri_, ul);
 	}
 }
@@ -239,12 +250,13 @@ void Name::set(const Name &n)
 	if (&n == &dbxml_colon_name) {
 		reset();
 		known_ = &n;
-	}
-	else if(n.known_ != 0) {
+	} else if (&n == &dbxml_colon_root) {
+		reset();
+		known_ = &n;
+	} else if (n.known_ != 0) {
 		reset();
 		known_ = n.known_;
-	}
-	else {
+	} else {
 		size_t ul = (n.uri_ == 0 ? 0 : strlen(n.uri_));
 		size_t nl = (n.name_ == 0 ? 0 : strlen(n.name_));
 		set(n.name_, nl, n.uri_, ul);
