@@ -1,18 +1,20 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: ReferenceCounted.hpp,v 1.19 2006/10/30 17:45:53 bostic Exp $
 //
 
 #ifndef __REFERENCECOUNTED_HPP
 #define	__REFERENCECOUNTED_HPP
 
+#include "MutexLock.hpp"
+#include <dbxml/XmlPortability.hpp>
+
 namespace DbXml
 {
 
-class ReferenceCounted
+class DBXML_EXPORT ReferenceCounted
 {
 public:
 	ReferenceCounted() : count_(0) {}
@@ -24,35 +26,35 @@ private:
 	int count_;
 };
 
-template<class T> class RefCountJanitor
+template<class T> class DBXML_EXPORT RefCountJanitor
 {
 public:
 	RefCountJanitor(T *p = 0) : p_(p) {
-		if(p_) p_->acquire();
+		if(p_) ((ReferenceCounted*)p_)->acquire();
 	}
 	
 	RefCountJanitor(const RefCountJanitor<T> &o) : p_(o.p_) {
-		if(p_) p_->acquire();
+		if(p_) ((ReferenceCounted*)p_)->acquire();
 	}
 
 	RefCountJanitor &operator=(const RefCountJanitor<T> &o) {
 		if(p_ != o.p_) {
-			if(p_) p_->release();
+			if(p_) ((ReferenceCounted*)p_)->release();
 			p_ = o.p_;
-			if(p_) p_->acquire();
+			if(p_) ((ReferenceCounted*)p_)->acquire();
 		}
 		return *this;
 	}
 
 	~RefCountJanitor() {
-		if(p_) p_->release();
+		if(p_) ((ReferenceCounted*)p_)->release();
 	}
 
 	void reset(T *p = 0) {
 		if(p_ != p) {
-			if(p_) p_->release();
+			if(p_) ((ReferenceCounted*)p_)->release();
 			p_ = p;
-			if(p_) p_->acquire();
+			if(p_) ((ReferenceCounted*)p_)->acquire();
 		}
 	}
 
@@ -71,7 +73,15 @@ public:
 	operator bool() const {
 		return p_ != 0;
 	}
-	
+
+	bool operator==(const RefCountJanitor<T> &o) {
+		return (p_ == o.p_);
+	}
+
+	bool operator!=(const RefCountJanitor<T> &o) {
+		return (p_ != o.p_);
+	}
+
 private:
 	T *p_;
 };
@@ -79,32 +89,22 @@ private:
 // ReferenceCountedProtected	
 // a lock-protected instance of ReferenceCounted, that
 // protects the count for free-threaded objects
-//
-class ReferenceCountedProtected
+// Make release and acquire virtual to allow objects
+// finer control
+class DBXML_EXPORT ReferenceCountedProtected
 {
 public:
 	ReferenceCountedProtected();
 	virtual ~ReferenceCountedProtected();
-	void acquire();
-	void release();
+	virtual void acquire();
+	virtual void release();
 	void lock();
 	void unlock();
-private:
+protected:
 	int count_;
-	void *mutex_;
+	dbxml_mutex_t mutex_;
 };
 
-class MutexLock
-{
-public:
-	MutexLock(void *mutex);
-	~MutexLock();
-private:
-	MutexLock(const MutexLock&);
-	MutexLock &operator=(const MutexLock&);
-	
-	void *toLock_;
-};
 
 }
 
