@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: ReferenceMinder.hpp,v 1.10 2006/11/21 18:49:43 jsnelson Exp $
 //
 
 #ifndef __REFERENCEMINDER_HPP
@@ -14,8 +13,7 @@
 
 #include "DocID.hpp"
 
-#include <xercesc/dom/DOMNode.hpp>
-#include <xercesc/dom/DOMDocument.hpp>
+#include <xercesc/util/XMLUri.hpp>
 
 #include <map>
 
@@ -23,51 +21,48 @@ namespace DbXml
 {
 
 class Document;
-class Container;
+
+struct DocMapKey
+{
+	DocMapKey(int cid, DocID i)
+		: containerID(cid), id(i) {}
+
+	bool operator<(const DocMapKey &o) const;
+
+	const int containerID;
+	DocID id;
+};
 
 class ReferenceMinder
 {
 public:
-	struct DocMapKey
-	{
-		DocMapKey(const Container *c, DocID i)
-			: container(c), id(i) {}
-
-		const Container *container;
-		DocID id;
-	};
-
 	ReferenceMinder() {}
 	~ReferenceMinder();
 
-	void addContainer(TransactedContainer *container);
-	void removeContainer(TransactedContainer *container);
+	void addContainer(Container *container);
+	void removeContainer(int cid);
 
 	void addDocument(Document *doc);
 	void removeDocument(Document *doc);
 	/// Returns 0 if none found
 	Document *findDocument(const XMLCh *uri);
 	/// Returns 0 if none found
-	Document *findDocument(const Container *container, DocID id);
+	Document *findDocument(int cid, DocID id);
 
 	void resetMinder();
 
+	std::map<int, XmlContainer> getContainers() const;
 private:
 	ReferenceMinder(const ReferenceMinder&);
 	ReferenceMinder &operator=(const ReferenceMinder&);
 
-	struct docMapCompare
-	{
-		bool operator()(const DocMapKey &s1, const DocMapKey &s2) const;
-	};
-
 	struct xmlchCompare
 	{
 		bool operator()(const XMLCh *s1, const XMLCh *s2) const;
 	};
 
-	typedef std::map<TransactedContainer*, XmlContainer> ContainerMap;
-	typedef std::map<DocMapKey, Document*, docMapCompare> ID2DocumentMap;
+	typedef std::map<int, XmlContainer> ContainerMap;
+	typedef std::map<DocMapKey, Document*> ID2DocumentMap;
 	typedef std::map<const XMLCh*, Document*, xmlchCompare> URI2DocumentMap;
 
 	ContainerMap containers_;
