@@ -1,18 +1,20 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: Results.cpp,v 1.102 2006/10/30 17:45:53 bostic Exp $
 //
 
 #include <xqilla/simple-api/XQQuery.hpp>
 #include <xqilla/context/DynamicContext.hpp>
+#include <xqilla/context/impl/XQDynamicContextImpl.hpp>
 
 #include "DbXmlInternal.hpp"
 #include "dbxml/XmlNamespace.hpp"
 #include "dbxml/XmlException.hpp"
 #include "dbxml/XmlResults.hpp"
+#include "dbxml/XmlContainerConfig.hpp"
+#include "dbxml/XmlEventReaderToWriter.hpp"
 #include "Results.hpp"
 #include "QueryContext.hpp"
 #include "OperationContext.hpp"
@@ -23,9 +25,10 @@
 #include "HighResTimer.hpp"
 #include "Document.hpp"
 #include "IndexLookup.hpp"
-#include "dataItem/DbXmlContext.hpp"
+#include "dataItem/DbXmlFactoryImpl.hpp"
+#include "ResultsEventWriter.hpp"
+#include "nodeStore/NsDom.hpp"
 
-#include <xercesc/dom/DOM.hpp>
 #if defined(XERCES_HAS_CPP_NAMESPACE)
   XERCES_CPP_NAMESPACE_USE
 #endif
@@ -42,52 +45,165 @@ using namespace std;
 // big macro to encapsulate exceptions that might
 // come from XQilla and Xerces
 #define CATCH_XQUERY_EXCEPTIONS \
-	catch (QueryInterruptedException &ie) { \
+	catch (QueryInterruptedException &) { \
 		throw XmlException(XmlException::OPERATION_INTERRUPTED, \
 				   "Query was interrupted by the application");\
 	} \
-	catch (QueryTimeoutException &ie) { \
+	catch (QueryTimeoutException &) { \
 		throw XmlException(XmlException::OPERATION_TIMEOUT, \
 				   "Query timed out"); \
 	} \
 	catch (const XQillaException &e) { \
-		throw XmlException(XmlException::XPATH_EVALUATION_ERROR, XMLChToUTF8(e.getString()).str()); \
+		throw XmlException(XmlException::QUERY_EVALUATION_ERROR, XMLChToUTF8(e.getString()).str()); \
 	} \
 	catch(const XQException &e) { \
-		throw XmlException(XmlException::XPATH_EVALUATION_ERROR, e); \
-	} \
-	catch(XERCES_CPP_NAMESPACE_QUALIFIER DOMException &e) { \
-		throw XmlException(XmlException::XPATH_EVALUATION_ERROR, XMLChToUTF8(e.msg).str()); \
+		throw XmlException(XmlException::QUERY_EVALUATION_ERROR, e); \
 	} \
 	catch(XERCES_CPP_NAMESPACE_QUALIFIER XMLException &e) { \
-		throw XmlException(XmlException::XPATH_EVALUATION_ERROR, XMLChToUTF8(e.getMessage()).str()); \
+		throw XmlException(XmlException::QUERY_EVALUATION_ERROR, XMLChToUTF8(e.getMessage()).str()); \
 	}
 
+
+ResultsEventWriter& Results::asEventWriter(void)
+{
+	if (isLazy()) {
+    		throw XmlException(XmlException::INVALID_VALUE,
+			"XmlResults::asEventWriter() can not be called on lazy result sets");
+	}
+
+	if(writer_ == NULL)
+		writer_ = new ResultsEventWriter(this);
+
+	return *writer_;
+}
+
+Results::~Results()
+{
+	if (writer_)
+		delete writer_;
+}
+
+Results* Results::copyResults(void)
+{
+	if (!isLazy())
+		reset();
+	
+	Results* copy = new ValueResults(mgr_);
+	copy->concatResults(*this);
+
+	if (!isLazy())
+		reset();
+
+	return copy;
+}
+
+XmlEventReader::XmlEventType Results::mapToEtype(short nodeType)
+{
+	if (nodeType == XmlValue::TEXT_NODE)
+		return XmlEventReader::Characters;
+	if (nodeType == XmlValue::COMMENT_NODE)
+		return XmlEventReader::Comment;
+	if (nodeType == XmlValue::CDATA_SECTION_NODE)
+		return XmlEventReader::CDATA;
+	
+	DBXML_ASSERT(false);
+	return (XmlEventReader::XmlEventType)0;
+}
+
+void Results::concatResults(Results &from)
+{
+	if (isLazy())
+		throw XmlException(XmlException::LAZY_EVALUATION,
+			"XmlResults::concatResults() can only be called on eager result sets");
+
+	XmlEventWriter &writer = this->asEventWriter();
+
+	XmlValue value;
+	while (from.hasNext()){
+		from.next(value);
+		if (value.isNode()){//copy node types
+			short nodeType = value.getNodeType();
+			switch (nodeType){
+				case XmlValue::ELEMENT_NODE:
+				case XmlValue::DOCUMENT_NODE:{
+					XmlEventReader &reader = value.asEventReader();
+					XmlEventReaderToWriter r2w(reader, writer, true, false);
+					r2w.start();
+					break;
+				}
+				case XmlValue::ATTRIBUTE_NODE:
+					writer.writeAttribute((const unsigned char *)value.getLocalName().c_str(),
+						(const unsigned char *)value.getPrefix().c_str(),
+						(const unsigned char *)value.getNamespaceURI().c_str(),
+						(const unsigned char *)value.getNodeValue().c_str(),
+						false);
+					break;
+				case XmlValue::CDATA_SECTION_NODE:
+				case XmlValue::TEXT_NODE:
+				case XmlValue::COMMENT_NODE:
+					writer.writeText(mapToEtype(nodeType), 
+						(const unsigned char *)value.getNodeValue().c_str(), 0);
+					break;
+				case XmlValue::DOCUMENT_TYPE_NODE:
+					writer.writeDTD((const unsigned char *)value.getNodeValue().c_str(),0);
+					break;
+				case XmlValue::PROCESSING_INSTRUCTION_NODE:
+					writer.writeProcessingInstruction((const unsigned char *)value.getNodeName().c_str(), 
+						(const unsigned char *)value.getNodeValue().c_str()); 
+					break;
+				default :
+					DBXML_ASSERT(false);		
+			}
+		} else {
+			XmlValue newValue(value.getType(), value.asString());
+			this->add(newValue);
+		}
+	}
+}
+
 // Results from Values
-ValueResults::ValueResults()
-	: vvi_(0)
+ValueResults::ValueResults(XmlManager &mgr, Transaction *txn)
+	: Results(mgr, txn), vvi_(0)
 {
 }
 
-ValueResults::ValueResults(const XmlValue &value)
-	: vvi_(0)
+ValueResults::ValueResults(const ValueResults &o, XmlManager &mgr,
+			   Transaction *txn)
+	: Results(mgr, txn), vv_(o.vv_),
+	  vvi_(0)
+{
+	dbMinder_ = o.dbMinder_;
+}
+
+ValueResults::ValueResults(const XmlValue &value, XmlManager &mgr,
+			   Transaction *txn)
+	: Results(mgr, txn), vvi_(0)
 {
 	if(!value.isNull())
 		vv_.push_back(value);
 }
 
-ValueResults::ValueResults(Results *resultsToAdopt)
-	: vvi_(0)
+ValueResults::ValueResults(Results *resultsToAdopt, XmlManager &mgr,
+			   Transaction *txn)
+	: Results(mgr, txn), vvi_(0)
 {
 	if(resultsToAdopt) {
 		XmlResults results(resultsToAdopt);
-		XmlValue value;
-		while(results.next(value)) {
-			if (value.getType() == XmlValue::BINARY)
-				throw XmlException(XmlException::INVALID_VALUE,
-						   "XmlQueryContext::setVariableValue value cannot be binary");
-			vv_.push_back(value);
-		}
+		init(results);
+		dbMinder_ = resultsToAdopt->getDbMinder();
+		if((*mgr_).allowAutoOpen())
+			containers_ = resultsToAdopt->getContainers();
+	}
+}
+
+void ValueResults::init(XmlResults &results)
+{
+	XmlValue value;
+	while(results.next(value)) {
+		if (value.getType() == XmlValue::BINARY)
+			throw XmlException(XmlException::INVALID_VALUE,
+					   "XmlQueryContext::setVariableValue value cannot be binary");
+		vv_.push_back(value);
 	}
 }
 
@@ -111,23 +227,28 @@ size_t ValueResults::size() const
 // NOTE: algorithm keeps vvi_ pointing to "current" entry
 int ValueResults::doNext(XmlValue &value, bool isPeek)
 {
-	if(vvi_ == 0) {
-		if (vv_.size() == 0) {
-			value = XmlValue();
-			return 0;
-		}
-		vvi_ = new XmlValueVector::iterator;
-		*vvi_ = vv_.begin();
-	} else if (!isPeek)
-		(*vvi_)++;		
+    if (vv_.size() == 0){
+        value = XmlValue();
+        return 0;
+    }
 
-	if(*vvi_ != vv_.end()) {
-		value = **vvi_;
-	} else {
-		value = XmlValue();
-	}
+    if (vvi_ == 0) {
+        vvi_ = new XmlValueVector::iterator;
+        *vvi_ = vv_.begin();
+    } 
 
-	return 0;
+    //return the current
+    if (*vvi_ != vv_.end()){ 
+        value = **vvi_;
+    } else {
+        value = XmlValue();
+    }
+
+    // move if not peek or not after last
+    if (!isPeek && (*vvi_ != vv_.end()))
+        (*vvi_)++;		
+	
+    return 0;
 }
 
 int ValueResults::next(XmlValue &value)
@@ -145,7 +266,7 @@ bool ValueResults::hasNext()
 	// Don't create the vector if it's not already done; just use size 
 	if(vvi_ == 0)
 		return (vv_.size() != 0);
-	if(*vvi_ != vv_.end() && ((*vvi_) + 1 != vv_.end()))
+	if(*vvi_ != vv_.end())
 		return true;
 	return false;
 }
@@ -173,6 +294,9 @@ void ValueResults::add(const XmlValue &value)
 {
 	if(!value.isNull())
 		vv_.push_back(value);
+	else
+		throw XmlException(XmlException::INVALID_VALUE, 
+				"A null XmlValue object cannot be added to the result set.");
 }
 
 void ValueResults::add(const XmlValueVector &values)
@@ -198,22 +322,46 @@ size_t LazyResults::size() const
 }
 
 // Results from XQ tree, evaluated lazily
-LazyDIResults::LazyDIResults(QueryContext *context, Value *contextItem, QueryExpression &expr, Transaction *txn, u_int32_t flags)
-	: context_(context),
+LazyDIResults::LazyDIResults(QueryContext *context, Value *contextItem,
+			     QueryExpression &expr, Transaction *txn, u_int32_t flags)
+	: LazyResults(context->getManager(), txn), context_(context),
 	  expr_(&expr),
 	  contextItem_(contextItem),
 	  lazyDocs_((flags & DBXML_LAZY_DOCS) != 0),
 	  qec_(context_, /*debugging*/false),
+	  conf_(*context, txn, expr.getCompileInfo(),
+		  ((DbXmlFactoryImpl*)((QueryExpression*)expr_)->
+			  getDynamicContext()->getItemFactory())->getDatatypeLookup()),
 	  xqc_(0),
 	  result_(0),
 	  nextItem_(0)
 {
-	((QueryContext &)context_).setTransaction(txn);
-	((QueryContext &)context_).setMinder(&evaluationMinder_);
-	((QueryContext &)context_).setQueryExecutionContext(&qec_);
+	// this code ensures that transiently-parsed XmlDocument objects
+	// share the same CacheDatabaseMinder implementation as the query
+	if (contextItem && (contextItem->getType() == XmlValue::NODE)) {
+		XmlDocument &xdoc = (XmlDocument&)contextItem->asDocument();
+		CacheDatabaseMinder &minder = ((Document *)xdoc)->getDbMinder();
+		if (minder.isNull() && !((Document*)xdoc)->getContainerID()) {
+			dbMinder_.findOrAllocate((Manager&)(context->getManager()),
+						 ((Document *)xdoc)->getContainerID());
+			minder = dbMinder_;
+		}else
+			dbMinder_ = minder;
+		if ((*xdoc).isUninitialized()) (*xdoc).changeContentToNsDom(0);
+	}
+	oc_.set(txn);
+	conf_.setMinder(&evaluationMinder_);
+	conf_.setQueryExecutionContext(&qec_);
+	conf_.setDbMinder(dbMinder_);
+
+	// Set up document projection
+	if((flags & DBXML_DOCUMENT_PROJECTION) != 0) {
+		conf_.setProjectionInfo(expr.getProjectionInfo());
+		flags &= ~DBXML_DOCUMENT_PROJECTION;
+	}
 
 	// Documents are always lazy, until they are returned to the user
-	((QueryContext &)context_).setFlags(flags | DBXML_LAZY_DOCS);
+	conf_.setFlags(flags | DBXML_LAZY_DOCS);
 
 	int err = reset();
 	if(err != 0)
@@ -233,27 +381,33 @@ int LazyDIResults::next(XmlValue &value)
 
 	try {
 		Item::Ptr item;
-		if (nextItem_ != NULLRCP) {
+		if (nextItem_.notNull()) {
 			item = nextItem_;
 			nextItem_ = 0;
 		} else
 			item = result_->next(xqc_);
-		if(item == NULLRCP) {
+		if(item.isNull()) {
 			value = XmlValue();
 		} else {
 			if(item->isNode()) {
-				value = Value::create(item, context_, lazyDocs_);
+				value = Value::create(item, lazyDocs_);
 			}
 			else {
 				value = Value::create(item, xqc_);
 			}
 		}
+		// iteration may result in construction of a
+		// CacheDatabaseMinderImpl object and this object must
+		// share it with the DbXmlConfiguration
+		if (dbMinder_.isNull() && !conf_.getDbMinder().isNull())
+			dbMinder_ = conf_.getDbMinder();
 	}
 	CATCH_XQUERY_EXCEPTIONS
 	timer_.stop();
 
 	if(value.isNull() && !result_.isNull()) {
 		result_ = 0;
+
 		if(Log::isLogEnabled(Log::C_QUERY, Log::L_INFO)) {
 			ostringstream s;
 			s << "Finished query execution, time taken = "
@@ -300,9 +454,9 @@ int LazyDIResults::previous(XmlValue &)
 bool LazyDIResults::hasNext()
 {
 	try {
-		if (nextItem_ == NULLRCP)
+		if (nextItem_.isNull())
 			nextItem_ = result_->next(xqc_);
-		if (nextItem_ == NULLRCP)
+		if (nextItem_.isNull())
 			return false;
 	}
 	CATCH_XQUERY_EXCEPTIONS
@@ -315,7 +469,6 @@ bool LazyDIResults::hasPrevious()
 	return false;
 }
 	
-	
 int LazyDIResults::reset()
 {
 	((Manager &)((QueryContext &)context_).getManager())
@@ -328,12 +481,29 @@ int LazyDIResults::reset()
 
 	try {
 		if(xqc_ != 0) delete xqc_;
-		xqc_ = ((QueryContext&)context_).createDynamicContext(((QueryContext &)context_).getOperationContext()
-			.txn(), ((QueryExpression*)expr_)->getDynamicContext());
-		if(contextItem_)
-			xqc_->setContextItem(Value::convertToItem((Value*)contextItem_, xqc_));
-
-		result_ = ((QueryExpression*)expr_)->getCompiledExpression()->execute(xqc_);
+		// additional try/catch to clean up xqc_ if there is
+		// and exception during the constructor
+		try {
+			xqc_ = new (Globals::defaultMemoryManager)
+				XQDynamicContextImpl(
+					&conf_,
+					((QueryExpression*)expr_)->getDynamicContext(),
+					Globals::defaultMemoryManager);
+			conf_.populateDynamicContext(xqc_);
+			if(contextItem_)
+				xqc_->setContextItem(
+					Value::convertToItem((Value*)contextItem_,
+							     xqc_, true));
+			
+			result_ = ((QueryExpression*)expr_)->
+				getCompiledExpression()->execute(xqc_);
+		} catch (...) {
+			if (xqc_) {
+				delete xqc_;
+				xqc_ = 0;
+			}
+			throw; // rethrow
+		}
 	}
 	CATCH_XQUERY_EXCEPTIONS
 
@@ -342,6 +512,11 @@ int LazyDIResults::reset()
 	return 0;
 }
 
+std::map<int, XmlContainer> LazyDIResults::getContainers() const
+{
+	return (*expr_).getContainers();
+}
+
 static DbWrapper::Operation mapIndexOp(XmlIndexLookup::Operation op)
 {
 	DbWrapper::Operation ret;
@@ -373,8 +548,10 @@ LazyIndexResults::LazyIndexResults(Container &container,
 				   Transaction *txn, const Index &index,
 				   const IndexLookup &il,
 				   u_int32_t flags)
-	: context_(context->copy()),
-	  container_(&((TransactedContainer &)container)),
+	: LazyResults(context->getManager(), txn),
+	  context_(context),
+	  conf_(*context, txn, 0),
+	  container_(&((Container &)container)),
 	  lowOp_(DbWrapper::PREFIX),
 	  lowKey_(((Manager&)container.getManager()).getImplicitTimezone()),
 	  highOp_(DbWrapper::NONE),
@@ -397,12 +574,11 @@ LazyIndexResults::LazyIndexResults(Container &container,
 		Name pname(il.getParentURI(), il.getParentName());
 		parentUriName = pname.getURIName();
 	}
-	((Manager &)((QueryContext &)context_).getManager())
-		.log(Log::C_QUERY, Log::L_INFO, "Starting index lookup");
+	container.log(Log::C_QUERY, Log::L_INFO, "Starting index lookup");
 
-	((QueryContext &)context_).setTransaction(txn);
-	((QueryContext &)context_).setFlags(flags & ~(DBXML_REVERSE_ORDER|DBXML_NO_INDEX_NODES|DBXML_CACHE_DOCUMENTS));
-	OperationContext &oc = ((QueryContext*)context_)->getOperationContext();
+	conf_.setFlags(flags & ~(DBXML_REVERSE_ORDER|DBXML_NO_INDEX_NODES|DBXML_CACHE_DOCUMENTS));
+	conf_.setDbMinder(dbMinder_);
+	OperationContext &oc = conf_.getOperationContext();
 
 	lowKey_.setIndex(index);
 
@@ -451,12 +627,12 @@ int LazyIndexResults::reset()
 		if (highOp_ != DbWrapper::NONE) {
 			cursor_.reset(sdb->getIndexDB()->
 				createCursor(
-					((QueryContext &)context_).getOperationContext().txn(),
+					conf_.getTransaction(),
 					lowOp_, &lowKey_, highOp_, &highKey_, reverse_));
 		} else {
 			cursor_.reset(sdb->getIndexDB()->
 				createCursor(
-					((QueryContext &)context_).getOperationContext().txn(),
+					conf_.getTransaction(),
 					lowOp_, &lowKey_, reverse_));
 		}
 		err = cursor_->error();
@@ -483,15 +659,16 @@ int LazyIndexResults::doNext(XmlValue &value, bool isPeek)
 	    docId == ((const Document&) value.asDocument()).getID())
 		document = value.asDocument();
 	else
-		docId.fetchDocument(container_, context_, document,
+		docId.fetchDocument(container_, conf_, document,
 				    (cacheDocuments_ ? &evaluationMinder_ : 0));
 
-	DOMNode *node = 0;
+	NsDomNode *node = 0;
 	if(!docOnly_ && ie_.isSpecified(IndexEntry::NODE_ID)) {
-		node = ie_.fetchNode(document);
+		node = ie_.fetchNode(document, conf_.getTransaction(),
+				     &conf_);
 	}
 
-	value = new NodeValue(node, document);
+	value = DbXmlNodeValue::makeDbXmlNodeValue(node, document, &conf_);
 
 	int err = 0;
 	if(!isPeek)
