@@ -1,9 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: Syntax.cpp,v 1.52 2006/10/31 23:39:51 jsnelson Exp $
+// $Id: Syntax.cpp,v 86f215ae8bc4 2008/08/20 17:42:30 john $
 //
 
 #include "DbXmlInternal.hpp"
@@ -36,36 +36,33 @@
 
 using namespace DbXml;
 using namespace std;
-#ifdef _WIN32
-#define strcasecmp stricmp
-#endif
 
 extern "C" {
-int lexicographical_bt_compare(Db *db, const Dbt *a, const Dbt *b)
+int lexicographical_bt_compare(DB *db, const DBT *a, const DBT *b)
 {
 	UNUSED(db);
 
-	size_t len = a->get_size() > b->get_size() ? b->get_size() : a->get_size();
-	unsigned char *p1 = (unsigned char*)a->get_data();
-	unsigned char *p2 = (unsigned char*)b->get_data();
+	size_t len = a->size > b->size ? b->size : a->size;
+	unsigned char *p1 = (unsigned char*)a->data;
+	unsigned char *p2 = (unsigned char*)b->data;
 	int cmp;
 	for (; len--; ++p1, ++p2) {
 		cmp = (int)*p1 - (int)*p2;
 		if(cmp != 0) return cmp;
 	}
-	return ((long)a->get_size() - (long)b->get_size());
+	return ((long)a->size - (long)b->size);
 }
 
-static int mapm_bt_compare(Db *db, const Dbt *dbt1, const Dbt *dbt2)
+static int mapm_bt_compare(DB *db, const DBT *dbt1, const DBT *dbt2)
 {
 	UNUSED(db);
 
-	const xmlbyte_t *p1 = (const xmlbyte_t*)dbt1->get_data();
-	const xmlbyte_t *p2 = (const xmlbyte_t*)dbt2->get_data();
+	const xmlbyte_t *p1 = (const xmlbyte_t*)dbt1->data;
+	const xmlbyte_t *p2 = (const xmlbyte_t*)dbt2->data;
 
 	// Calculate the end of the two keys
-	const xmlbyte_t *e1 = p1 + dbt1->get_size();
-	const xmlbyte_t *e2 = p2 + dbt2->get_size();
+	const xmlbyte_t *e1 = p1 + dbt1->size;
+	const xmlbyte_t *e2 = p2 + dbt2->size;
 
 	// Compare the structure part of the key
 	int res = Key::compareStructure(p1, e1, p2, e2);
@@ -81,16 +78,16 @@ static int mapm_bt_compare(Db *db, const Dbt *dbt1, const Dbt *dbt2)
 	return NsFormat::compareMarshaledMAPM(p1, p2);
 }
 
-static int duration_bt_compare(Db *db, const Dbt *dbt1, const Dbt *dbt2)
+static int duration_bt_compare(DB *db, const DBT *dbt1, const DBT *dbt2)
 {
 	UNUSED(db);
 
-	const xmlbyte_t *p1 = (const xmlbyte_t*)dbt1->get_data();
-	const xmlbyte_t *p2 = (const xmlbyte_t*)dbt2->get_data();
+	const xmlbyte_t *p1 = (const xmlbyte_t*)dbt1->data;
+	const xmlbyte_t *p2 = (const xmlbyte_t*)dbt2->data;
 
 	// Calculate the end of the two keys
-	const xmlbyte_t *e1 = p1 + dbt1->get_size();
-	const xmlbyte_t *e2 = p2 + dbt2->get_size();
+	const xmlbyte_t *e1 = p1 + dbt1->size;
+	const xmlbyte_t *e2 = p2 + dbt2->size;
 
 	// Compare the structure part of the key
 	int res = Key::compareStructure(p1, e1, p2, e2);
@@ -132,16 +129,20 @@ const char *Syntax::getName() const
 bool Syntax::hasTypeName(const char *name) const
 {
 	// do case-insensitive comparison
-	if (::strcasecmp(name, getName()) == 0)
+	if (NsUtil::stringEqualsIgnoreCase(name, getName()) == 0)
 		return true;
 	return false;
 }
 
-KeyGenerator::Ptr Syntax::getKeyGenerator(const Index &index, const char *valueBuffer, size_t valueLength) const
+KeyGenerator::Ptr Syntax::getKeyGenerator(const Index &index, const char *valueBuffer, size_t valueLength, bool generateShortKeys) const
 {
 	if(test(valueBuffer, valueLength)) {
 		return KeyGenerator::Ptr(new SingleKeyGenerator(valueBuffer, valueLength));
 	}
+	else if(generateShortKeys) {
+		// Add a consolation key, to make presence lookups work
+		return KeyGenerator::Ptr(new SingleKeyGenerator(0, 0));
+	}
 	else {
 		return KeyGenerator::Ptr(new EmptyKeyGenerator());
 	}
@@ -199,7 +200,7 @@ DbWrapper::bt_compare_fn StringSyntax::get_bt_compare() const
 	return lexicographical_bt_compare;
 }
 
-KeyGenerator::Ptr StringSyntax::getKeyGenerator(const Index &index, const char *valueBuffer, size_t valueLength) const
+KeyGenerator::Ptr StringSyntax::getKeyGenerator(const Index &index, const char *valueBuffer, size_t valueLength, bool generateShortKeys) const
 {
 	// This is a special case. We don't need to call
 	// test(), since it always returns true.
@@ -211,7 +212,7 @@ KeyGenerator::Ptr StringSyntax::getKeyGenerator(const Index &index, const char *
 		r.reset(new SingleKeyGenerator(valueBuffer, valueLength));
 		break;
 	case Index::KEY_SUBSTRING:
-		r.reset(new SubstringKeyGenerator(valueBuffer, valueLength));
+		r.reset(new SubstringKeyGenerator(valueBuffer, valueLength, generateShortKeys));
 		break;
 	default:
 		DBXML_ASSERT(0);
@@ -228,11 +229,136 @@ size_t StringSyntax::marshal(Buffer *buffer, const char *p, size_t l, int timezo
 ///////////////////////////////////////////////////////////////////////////
 // MAPMSyntax
 
+static const char *dbxml_nan = "NAN";
+static const char *dbxml_inf = "INF";
+static const char *dbxml_neginf = "-INF";
+
+// Method to validate a numeric.
+// Returns true if string is a valid double, float or decimal and
+// false otherwise.   Only syntax checking is done.
+// The rules for decimal are different from
+// double/float so a boolean parameter distinguishes them.
+//
+// See XML Schema for lexical values allowed:
+//    http://www.w3.org/TR/xmlschema11-2/#decimal
+//    http://www.w3.org/TR/xmlschema11-2/#float
+//    http://www.w3.org/TR/xmlschema11-2/#double
+// Range validation is not done here.  Casts are done
+// at execution time based on valid (or not) ranges
+//
+bool NumericSyntax::testNumeric(const char* value, size_t len,
+				bool isDecimal) const
+{
+	NsUtil::trimWhitespace(value, len);
+
+	if (value == NULL || len == 0)
+		return false;
+
+	size_t pos = 0;
+	bool gotPoint = false;
+	bool gotSign = false;
+	bool gotDigit = false;
+	bool stop = false;
+	// gotBase is needed for float/double;
+	// it will never be set true for decimal
+	bool gotBase = false; 
+
+	const char *src = value;
+	char tmpChar;
+	while (!stop && pos < len && *src != 0) {
+		++pos;
+		tmpChar = *src++;
+		switch (tmpChar) {
+		case L'+': {
+			if (gotSign || gotDigit) {
+				stop = true;
+			} else {
+				gotSign = true;
+			}
+			break;
+		}
+             
+		case L'-': {
+			if (gotSign || gotDigit) {
+				stop = true;
+			} else {
+				gotSign = true;
+			}
+			break;
+		}
+             
+		case 0x0045:
+		case 0x0065: {
+			if (!gotDigit || gotBase || isDecimal) {
+				stop = true;
+			} else {
+				gotPoint = false;
+				gotSign = false;        
+				gotBase = true;
+				gotDigit = false;
+			}
+			break;
+		}
+             
+			
+		case 0x002e: {  //This is '.'
+			if (gotPoint || gotBase) {
+				stop = true;
+			} else {
+				gotPoint = true;
+			}
+			break;
+		}
+             
+		case 0x0030:
+		case 0x0031:
+		case 0x0032:
+		case 0x0033:
+		case 0x0034:
+		case 0x0035:
+		case 0x0036:
+		case 0x0037:
+		case 0x0038:
+		case 0x0039: {
+			// valid digits
+			gotDigit = true;
+			break;
+		}
+			
+		default:
+			stop = true;
+			break;
+		}
+		// end switch
+	} // while
+
+	if (!gotDigit || stop) {
+		if (isDecimal)
+			return false;
+		// check other valid strings for float/double
+		// assumes white space has been trimmed by caller
+		if (NsUtil::stringNEqualsIgnoreCase(value, dbxml_nan, len) == 0 ||
+		    NsUtil::stringNEqualsIgnoreCase(value, dbxml_inf, len) == 0 ||
+		    NsUtil::stringNEqualsIgnoreCase(value, dbxml_neginf, len) == 0)
+			return true;
+		return false;
+	}
+	if (isDecimal)
+		return true;
+	else {
+		// Xerces-c maps overflow and underflow to
+		// + or - INF respectively so don't range
+		// check at this time.  If so it'd involve
+		// calling strtod().
+		return true;
+	}
+}
+
 bool MAPMSyntax::test(const char *v, size_t len) const
 {
 	NsUtil::trimWhitespace(v, len);
 	return getDatatypeFactory()->checkInstance(UTF8ToXMLCh(v, len).str(),
-		Globals::defaultMemoryManager);
+						   Globals::defaultMemoryManager);
 }
 
 SecondaryDatabase::bt_compare_fn MAPMSyntax::get_bt_compare() const
@@ -380,7 +506,7 @@ MAPM DateTimeSyntax::createMAPM(const char *p, size_t l, int timezone, Numeric::
 
 bool DaySyntax::hasTypeName(const char *name) const
 {
-	if (::strcasecmp(name, "day") == 0)
+	if (NsUtil::stringEqualsIgnoreCase(name, "day") == 0)
 		return true;
 	return Syntax::hasTypeName(name);
 }
@@ -408,6 +534,11 @@ Syntax::Type DecimalSyntax::getType() const
 	return Syntax::DECIMAL;
 }
 
+bool DecimalSyntax::test(const char *v, size_t len) const
+{
+	return testNumeric(v, len, true);
+}
+
 DatatypeFactory *DecimalSyntax::getDatatypeFactory() const
 {
 	return Globals::getDatatypeLookup()->getDecimalFactory();
@@ -426,6 +557,11 @@ Syntax::Type DoubleSyntax::getType() const
 	return Syntax::DOUBLE;
 }
 
+bool DoubleSyntax::test(const char *v, size_t len) const
+{
+	return testNumeric(v, len, false);
+}
+
 DatatypeFactory *DoubleSyntax::getDatatypeFactory() const
 {
 	return Globals::getDatatypeLookup()->getDoubleFactory();
@@ -463,7 +599,7 @@ size_t DurationSyntax::marshal(Buffer *buffer, const char *p, size_t l, int time
 	MAPM months, seconds;
 	ATDurationOrDerivedImpl::parseDuration(UTF8ToXMLCh(p, l).str(), months, seconds);
 
-	int length = marshalMAPMIntoBuffer(buffer, months);
+	size_t length = marshalMAPMIntoBuffer(buffer, months);
 	length += marshalMAPMIntoBuffer(buffer, seconds);
 
 	return length;
@@ -477,6 +613,11 @@ Syntax::Type FloatSyntax::getType() const
 	return Syntax::FLOAT;
 }
 
+bool FloatSyntax::test(const char *v, size_t len) const
+{
+	return testNumeric(v, len, false);
+}
+
 DatatypeFactory *FloatSyntax::getDatatypeFactory() const
 {
 	return Globals::getDatatypeLookup()->getFloatFactory();
@@ -527,7 +668,7 @@ size_t HexBinarySyntax::marshal(Buffer *buffer, const char *p, size_t l, int tim
 
 bool MonthSyntax::hasTypeName(const char *name) const
 {
-	if (::strcasecmp(name, "month") == 0)
+	if (NsUtil::stringEqualsIgnoreCase(name, "month") == 0)
 		return true;
 	return Syntax::hasTypeName(name);
 }
@@ -552,7 +693,7 @@ MAPM MonthSyntax::createMAPM(const char *p, size_t l, int timezone, Numeric::Sta
 
 bool MonthDaySyntax::hasTypeName(const char *name) const
 {
-	if (::strcasecmp(name, "monthDay") == 0)
+	if (NsUtil::stringEqualsIgnoreCase(name, "monthDay") == 0)
 		return true;
 	return Syntax::hasTypeName(name);
 }
@@ -595,7 +736,7 @@ MAPM TimeSyntax::createMAPM(const char *p, size_t l, int timezone, Numeric::Stat
 
 bool YearSyntax::hasTypeName(const char *name) const
 {
-	if (::strcasecmp(name, "year") == 0)
+	if (NsUtil::stringEqualsIgnoreCase(name, "year") == 0)
 		return true;
 	return Syntax::hasTypeName(name);
 }
@@ -620,7 +761,7 @@ MAPM YearSyntax::createMAPM(const char *p, size_t l, int timezone, Numeric::Stat
 
 bool YearMonthSyntax::hasTypeName(const char *name) const
 {
-	if (::strcasecmp(name, "yearMonth") == 0)
+	if (NsUtil::stringEqualsIgnoreCase(name, "yearMonth") == 0)
 		return true;
 	return Syntax::hasTypeName(name);
 }
