@@ -1,15 +1,14 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: Syntax.hpp,v 1.26 2006/10/31 23:39:51 jsnelson Exp $
 //
 
 #ifndef __SYNTAX_HPP
 #define	__SYNTAX_HPP
 
-#include "db_cxx.h"
+#include "db.h"
 #include "DbWrapper.hpp"
 #include "KeyGenerator.hpp"
 
@@ -19,7 +18,7 @@
 class DatatypeFactory;
 
 extern "C" {
-int lexicographical_bt_compare(Db *db, const Dbt *a, const Dbt *b);
+int lexicographical_bt_compare(DB *db, const DBT *a, const DBT *b);
 };
 
 namespace DbXml
@@ -63,7 +62,7 @@ public:
 	virtual DbWrapper::bt_compare_fn get_bt_compare() const = 0;
 	virtual KeyGenerator::Ptr getKeyGenerator(
 		const Index &index, const char *valueBuffer,
-		size_t valueLength) const;
+		size_t valueLength, bool generateShortKeys) const;
 	virtual size_t marshal(Buffer *buffer, const char *p, size_t l, int timezone) const = 0;
 };
 
@@ -87,7 +86,7 @@ public:
 	virtual DbWrapper::bt_compare_fn get_bt_compare() const;
 	virtual KeyGenerator::Ptr getKeyGenerator(
 		const Index &index, const char *valueBuffer,
-		size_t valueLength) const;
+		size_t valueLength, bool generateShortKeys) const;
 	virtual size_t marshal(Buffer *buffer, const char *p, size_t l, int timezone) const;
 };
 
@@ -104,6 +103,13 @@ public:
 	virtual size_t marshal(Buffer *buffer, const char *p, size_t l, int timezone) const;
 };
 
+class NumericSyntax : public MAPMSyntax
+{
+protected:
+	bool testNumeric(const char* value, size_t len,
+			 bool isDecimal) const;
+};
+	
 class Base64BinarySyntax : public Syntax
 {
 public:
@@ -147,18 +153,20 @@ public:
 	virtual MAPM createMAPM(const char *p, size_t l, int timezone, Numeric::State &state) const;
 };
 
-class DecimalSyntax : public MAPMSyntax
+class DecimalSyntax : public NumericSyntax
 {
 public:
 	virtual Type getType() const;
+	virtual bool test(const char *v, size_t len) const;
 	virtual DatatypeFactory *getDatatypeFactory() const;
 	virtual MAPM createMAPM(const char *p, size_t l, int timezone, Numeric::State &state) const;
 };
 
-class DoubleSyntax : public MAPMSyntax
+class DoubleSyntax : public NumericSyntax
 {
 public:
 	virtual Type getType() const;
+	virtual bool test(const char *v, size_t len) const;
 	virtual DatatypeFactory *getDatatypeFactory() const;
 	virtual MAPM createMAPM(const char *p, size_t l, int timezone, Numeric::State &state) const;
 };
@@ -172,10 +180,11 @@ public:
 	virtual size_t marshal(Buffer *buffer, const char *p, size_t l, int timezone) const;
 };
 
-class FloatSyntax : public MAPMSyntax
+class FloatSyntax : public NumericSyntax
 {
 public:
 	virtual Type getType() const;
+	virtual bool test(const char *v, size_t len) const;
 	virtual DatatypeFactory *getDatatypeFactory() const;
 	virtual MAPM createMAPM(const char *p, size_t l, int timezone, Numeric::State &state) const;
 };
