@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: SyntaxDatabase.hpp,v 1.23 2006/11/17 18:18:27 jsnelson Exp $
 //
 
 #ifndef __SYNTAXDATABASE_HPP
@@ -13,8 +12,8 @@
 #include <vector>
 #include "SharedPtr.hpp"
 #include "DbWrapper.hpp"
-#include "IDS.hpp"
 #include "IndexEntry.hpp"
+#include "Key.hpp"
 
 namespace DbXml
 {
@@ -22,6 +21,7 @@ namespace DbXml
 class KeyStatistics;
 class Syntax;
 class DbtIn;
+class Index;
 
 /**
  * Wraps an index and statistics database for a syntax.
@@ -33,8 +33,10 @@ public:
 	typedef SharedPtr<SyntaxDatabase> Ptr;
 	typedef std::vector<Ptr> Vector;
 
-	SyntaxDatabase(const Syntax *syntax, DbEnv *env, Transaction *txn, const std::string &name,
-		bool nodesIndexed, u_int32_t pageSize, u_int32_t flags, int mode);
+	SyntaxDatabase(const Syntax *syntax, DB_ENV *env, Transaction *txn,
+                       const std::string &name,
+                       bool nodesIndexed, const ContainerConfig &flags,
+                       bool usingCDB);
 	~SyntaxDatabase();
 
 	IndexDatabase *getIndexDB() { return index_.get(); }
@@ -43,10 +45,11 @@ public:
 	IndexDatabase *getStatisticsDB() { return statistics_.get(); }
 	const IndexDatabase *getStatisticsDB() const { return statistics_.get(); }
 
-	int getIndexData(OperationContext &context, IndexData::Ptr &data,
-			 SecondaryDatabase::Operation op1, const Key &k1,
-			 SecondaryDatabase::Operation op2 = SecondaryDatabase::NONE,
-			 const Key &k2 = Key(0)) const;
+	NodeIterator *createNodeIterator(
+		Container *container, Transaction *txn, bool documentIndex,
+		const LocationInfo *l, SecondaryDatabase::Operation op1, const Key &k1,
+		SecondaryDatabase::Operation op2 = SecondaryDatabase::NONE,
+		const Key &k2 = Key(0)) const;
 
 	int removeIndex(OperationContext &context, const Index &index,
 			const NameID &id);
@@ -63,11 +66,11 @@ public:
 		runner.run(txn, *statistics_);
 	}
 
-	static int dump(const Syntax *syntax, DbEnv *env, const std::string &name,
+	static int dump(const Syntax *syntax, DB_ENV *env, const std::string &name,
 		std::ostream *out);
-	static int load(const Syntax *syntax, DbEnv *env, const std::string &name,
+	static int load(const Syntax *syntax, DB_ENV *env, const std::string &name,
 		std::istream *in, unsigned long *lineno);
-	static int verify(const Syntax *syntax, DbEnv *env, const std::string &name,
+	static int verify(const Syntax *syntax, DB_ENV *env, const std::string &name,
 		std::ostream *out, u_int32_t flags);
 
 private:
@@ -78,7 +81,7 @@ private:
 			       void *buf, u_int32_t bufsize);
 
 	const Syntax *syntax_;
-	DbEnv *environment_;
+	DB_ENV *environment_;
 	std::string containerName_;
 
 	IndexDatabase::Ptr index_;
