@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: XmlUpdateContext.cpp,v 1.13 2006/10/30 17:45:54 bostic Exp $
 //
 
 #include "DbXmlInternal.hpp"
@@ -14,9 +13,6 @@
 
 using namespace DbXml;
 
-static const char *className = "XmlUpdateContext";
-#define CHECK_POINTER checkNullPointer(updateContext_,className)
-
 XmlUpdateContext::XmlUpdateContext()
 	: updateContext_(0)
 {
@@ -53,15 +49,3 @@ XmlUpdateContext &XmlUpdateContext::operator=(const XmlUpdateContext &o)
 	}
 	return *this;
 }
-
-void XmlUpdateContext::setApplyChangesToContainers(bool applyChanges)
-{
-	CHECK_POINTER;
-	updateContext_->setApplyChanges(applyChanges);
-}
-
-bool XmlUpdateContext::getApplyChangesToContainers() const
-{
-	CHECK_POINTER;
-	return updateContext_->getApplyChanges();
-}
