@@ -1,9 +1,7 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
-//
-// $Id: NsDocumentDatabase.hpp,v 1.23 2006/11/01 15:57:28 jsnelson Exp $
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
 
 #ifndef __NSDOCUMENTDATABASE_HPP
@@ -23,15 +21,14 @@ class Transaction;
 class NsDocumentDatabase : public DocumentDatabase
 {
 public:
-	NsDocumentDatabase(DbEnv *env, Transaction *txn,
-			   const std::string &name,
-			   u_int32_t pageSize, u_int32_t flags, int mode);
+	NsDocumentDatabase(DB_ENV *env, Transaction *txn,
+			   const std::string &name, const ContainerConfig &config,
+			   XmlCompression *compression);
 	virtual ~NsDocumentDatabase();
 
 	/** @name Interface to Databases */
-	DbWrapper &getNodeStorageDatabase()
-	{
-		return *nodeStorage_;
+	virtual DbWrapper *getNodeDatabase() {
+		return nodeStorage_;
 	}
 
 	virtual u_int32_t getPageSize() const {
@@ -41,32 +38,17 @@ public:
 		return nodeStorage_->getNumberOfPages();
 	}
 
-	virtual int createDocumentCursor(
-		Transaction *txn, ScopedPtr<DocumentCursor> &cursor,
-		u_int32_t flags) const;
-	
 	virtual int getContent(OperationContext &context, Document *document,
 			       u_int32_t flags) const;
 	virtual int updateContentAndIndex(Document &new_document,
 					  UpdateContext &context,
-					  KeyStash &stash);
+					  KeyStash &stash, bool validate,
+					  bool updateStats);
 	/// Also deallocates the ID
 	virtual int removeContentAndIndex(const Document &document,
 					  UpdateContext &context,
-					  KeyStash &stash);
+					  KeyStash &stash, bool updateStats);
 
-	/// Replaces if it already exists
-	int putNodeRecord(OperationContext &context, const DocID &did,
-		const NsNid *nid, const Dbt *data);
-	/// Replaces if it already exists
-	int putNodeRecord(OperationContext &context, const DocID &id,
-		const NsNode *node);
-	int delNodeRecord(OperationContext &context, const DocID &id,
-		const NsNid *nid);
-	int getNodeRecord(OperationContext &context, const DocID &id,
-		const NsNid *nid, Dbt *data, u_int32_t flags) const;
-	int getNextNodeRecord(OperationContext &context, const DocID &did,
-		const NsNid *nid, Dbt *data, u_int32_t flags) const;
 	virtual void sync() {
 		nodeStorage_->sync();
 		secondary_.sync();
@@ -88,22 +70,24 @@ public:
 		runner.run(txn, secondary_);
 		runner.run(txn, *nodeStorage_);
 	}
-	static int dump(DbEnv *env, const std::string &name,
+	static int dump(DB_ENV *env, const std::string &name,
 			std::ostream *out);
-	static int load(DbEnv *env, const std::string &name,
+	static int load(DB_ENV *env, const std::string &name,
 			std::istream *in, unsigned long *lineno);
-	static int verify(DbEnv *env, const std::string &name,
+	static int verify(DB_ENV *env, const std::string &name,
 			  std::ostream *out, u_int32_t flags);
 	// upgrade
-	static void upgrade(const std::string &name, Manager &mgr,
+	static void upgrade(const std::string &name,
+			    const std::string &tname, Manager &mgr,
 			    int old_version, int current_version);
 private:
 	int deleteAllNodes(OperationContext &context, const DocID &did);
 
 	// Private methods for upgrade from 2.2 to 2.3.  This is non-trivial
-	NsDocumentDatabase(DbEnv *env, const std::string &name,
+	NsDocumentDatabase(DB_ENV *env, const std::string &name,
 			   DbWrapper *db);
-	static void upgrade23(const std::string &name, Manager &mgr);
+	static void upgrade23(const std::string &name,
+			      const std::string &tname, Manager &mgr);
 	
 	mutable DbWrapper *nodeStorage_;
 	bool dbIsOwned_;
