@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: NsEventReader.cpp,v 1.14.2.1 2007/01/02 21:01:14 gmf Exp $
 //
 
 #include "../DbXmlInternal.hpp"
@@ -12,7 +11,7 @@
 #include "NsUtil.hpp"
 #include "NsConstants.hpp"
 #include "NsFormat.hpp"
-#include "NsDocumentDatabase.hpp"
+#include "../DbWrapper.hpp"
 #include <xqilla/framework/XPath2MemoryManager.hpp>
 #include <xercesc/framework/MemoryManager.hpp>
 
@@ -27,11 +26,14 @@
 
 #define MAKEBOOL(x) ((x)? true : false)
 
-// use 256K buffer for DB_MULTIPLE gets
-#define NS_EVENT_DB_BUFSIZE (256 * 1024)
-
 namespace DbXml {
 
+#define FORMAT_DEBUG 1
+#ifdef FORMAT_DEBUG
+// see code in NsFormat.cpp
+extern int _dumpFormatNodes;
+#endif
+
 /*
  * NsEventReaderBuf
  *
@@ -51,7 +53,8 @@ public:
 	uint32_t nComplete;
 	uint32_t nSize;
 	bool usedUp;
-	DbMultipleKeyDataIterator iter;
+	void *p;
+	DbXmlDbt dbt;
 };
 
 enum state {
@@ -67,31 +70,34 @@ class NsEventReaderNodeList
 public:
 	NsEventReaderNodeList(NsEventReaderNodeList *parent,
 			      NsEventReaderBuf *currentBuf)
-		: node(NULL), buffer(currentBuf),
-		textDone(0), state(0), started(false), parent(parent) {}
+		: buffer(currentBuf),
+		  textDone(0), state(0), started(false), parent(parent) {
+		memset(&curText, 0, sizeof(curText));
+	}
 
 	int getTextIndex() const { return textDone; }
+	nsTextEntry_t *getTextEntry() { return &curText; }
 	void incrText() {
 		// transition state when done with either
 		// leading text or child text
 		++textDone;
-		int ntext = node->getNumText();
-		if (!node->hasTextChild()) {
+		int ntext = node.getNumText();
+		if (!node.hasTextChild()) {
 			if (textDone == ntext)
 				state = START_ELEMENT;
 		} else {
 			if (textDone == ntext)
 				state = END_ELEMENT;
 			else if (textDone ==
-				 (ntext - node->getNumChildText()))
+				 (ntext - (int)node.getNumChildText()))
 				state = START_ELEMENT;
 		}
 	}
 	void setStartElemDone() {
 		started = true;
-		if (node->hasChildElem())
+		if (node.hasElemChild())
 			state = CHILD_ELEMENT;
-		else if (node->hasTextChild())
+		else if (node.hasTextChild())
 			state = TEXT;
 		else
 			state = END_ELEMENT;
@@ -100,43 +106,39 @@ public:
 		state = POP;
 	}
 	void setChildElemDone() {
-		if (node->hasTextChild())
+		if (node.hasTextChild())
 			state = TEXT;
 		else
 			state = END_ELEMENT;
 	}
 	bool hasSibling() {
-		return (node->hasNext());
+		return (node.hasNext());
 	}
 	int getState() const { return state; }
 	void setState(bool init);
 	bool getStarted() const { return started; }
 public:
-	NsNode *node;
+	NsRawNode node;
 	NsEventReaderBuf *buffer;
 	int textDone;
+	nsTextEntry_t curText;
 	int state;
 	bool started;
 	NsEventReaderNodeList *parent;
-	DbtOut curUri;
-	DbtOut curPrefix;
-	DbtOut attrUri;
-	DbtOut attrPrefix;
 };
 
 void NsEventReaderNodeList::setState(bool init)
 {
-	DBXML_ASSERT(node);
-	if (node->hasText()) {
+	if (node.hasText()) {
 		if (init) {
 			// skip leading text if this is
 			// the starting node
 			textDone +=
-				(node->getNumText() -
-				 node->getNumChildText());
+				(node.getNumText() -
+				 node.getNumChildText());
 		} else {
-			if (node->getNumText() >
-			    node->getNumChildText()) {
+			if (node.getNumText() >
+			    node.getNumChildText()) {
 				state = TEXT;
 				return;
 			}
@@ -151,33 +153,40 @@ using namespace DbXml;
 using namespace std;
 
 NsEventReader::NsEventReader(
-	Transaction *txn, NsDocumentDatabase *db, DictionaryDatabase *ddb,
+	Transaction *txn, DbWrapper *db, DictionaryDatabase *ddb,
 	const DocID &docId,
-	u_int32_t flags, XER_NS MemoryManager *memManager, const NsNid *startId)
-	: emptyElement_(false),
-	  document_(0, memManager),
-	  memMgr_(memManager),
+	int cid, u_int32_t flags, uint32_t bufferSize,
+	const NsNid *startId, CacheDatabase *cdb)
+	: localName_(0),
+	  emptyElement_(false),
+	  document_(0),
 	  doInit_(true),
 	  popElement_(false),
 	  entityCount_(0),
 	  current_(NULL),
 	  currentBuffer_(NULL),
-	  cursor_(db->getNodeStorageDatabase(),
-		  txn, CURSOR_READ, flags),
-	  cursorFlags_(DB_CURSOR_GET_MASK(flags)),
+	  cursor_(*db, txn, CURSOR_READ, "NsEventReader", flags),
+	  cursorFlags_(DB_CURSOR_GET_MASK(*db,flags)),
 	  reuseList_(NULL),
-	  freeList_(NULL)
+	  freeList_(NULL),
+	  docdb_(cdb),
+	  bufferSize_(bufferSize)
 {
 	DBXML_ASSERT(db);
-	document_.initDoc(txn, db, ddb,
-			  docId, flags);
+
+	if(db->getPageSize() > bufferSize_) {
+		bufferSize_ = db->getPageSize();
+	}
+
+	document_.initDoc(txn, db, ddb, docId, cid, flags);
 
 	// set starting NID.  If NULL, start at beginning
+	startId_.clear();
 	if (startId) {
-		memset(&startId_, 0, sizeof(startId_));
-		startId_.copyNid(memManager, startId);
+		startId_.copyNid(startId->getBytes(),
+				 startId->getLen());
 	} else {
-		startId_.setDocRootNid(NULL);
+		startId_.setDocRootNid();
 	}
 
 	// do the first event separately
@@ -186,7 +195,66 @@ NsEventReader::NsEventReader(
 		(void) doElement(true /*start*/);
 }
 
+NsEventReader::NsEventReader(NsDoc &nsDoc, uint32_t bufferSize,
+	const NsNid *startId, CacheDatabase *cdb)
+	: localName_(0),
+	  emptyElement_(false),
+	  document_(0),
+	  doInit_(true),
+	  popElement_(false),
+	  entityCount_(0),
+	  current_(NULL),
+	  currentBuffer_(NULL),
+	  cursor_(*(nsDoc.getDocDb()), nsDoc.getTxn(),
+		  CURSOR_READ, "NsEventReader", nsDoc.getFlags()),
+	  cursorFlags_(DB_CURSOR_GET_MASK(*(nsDoc.getDocDb()),nsDoc.getFlags())),
+	  reuseList_(NULL),
+	  freeList_(NULL),
+	  docdb_(cdb),
+	  bufferSize_(bufferSize)
+{
+	if(nsDoc.getDocDb()->getPageSize() > bufferSize_) {
+		bufferSize_ = nsDoc.getDocDb()->getPageSize();
+	}
+
+	document_.initDoc(nsDoc.getTxn(),
+			  nsDoc.getDocDb(),
+			  nsDoc.getDictionaryDatabase(),
+			  nsDoc.getDocID(),
+			  nsDoc.getContainerID(),
+			  nsDoc.getFlags());
+
+	// set starting NID.  If NULL, start at beginning
+	startId_.clear();
+	if (startId) {
+		startId_.copyNid(startId->getBytes(),
+				 startId->getLen());
+	} else {
+		startId_.setDocRootNid();
+	}
+
+	try {
+		// do the first event separately
+		getNode(current_);
+		if (hasNext_) // handle no-content docs
+			(void) doElement(true /*start*/);
+	} catch (...) {
+		cleanup();
+		throw;
+	}
+}
+
 NsEventReader::~NsEventReader()
+{
+	cleanup();
+}
+
+void NsEventReader::close()
+{
+	delete this;
+}
+
+void NsEventReader::cleanup()
 {
 	while(current_ != 0) {
 		NsEventReaderNodeList *tmp = current_;
@@ -194,17 +262,18 @@ NsEventReader::~NsEventReader()
 
 		if(tmp->buffer)
 			releaseNode(tmp->buffer);
-		if(tmp->node)
-			NsNode::freeNode(memMgr_, tmp->node);
 		delete tmp;
 	}
 
 	while (freeList_) {
 		NsEventReaderBuf *cur = freeList_;
 		freeList_ = cur->freeNext;
-		::free(cur);
+		NsUtil::deallocate(cur);
 	}
-	startId_.freeNid(memMgr_);
+	startId_.freeNid();
+	// make sure the cursor is closed before the CacheDatabase (if present)
+	// is closed...
+	cursor_.close();
 }
 
 //
@@ -228,23 +297,21 @@ NsEventReader::~NsEventReader()
 const unsigned char *
 NsEventReader::getNamespaceURI() const
 {
-	if (!localName_)
-		throwIllegalOperation(type_, "getNamespaceURI");
-	DBXML_ASSERT(node_ && node_ == current_->node);
+	if (!localName_ || type_ == ProcessingInstruction)
+		return 0;
+	DBXML_ASSERT(getNode() && getNode() == &current_->node);
 
 
-	if (!node_->hasUri())
+	if (!getNode()->hasURI())
 		return 0;
 
-	document_.getStringForID(node_->uriIndex(), current_->curUri);
-	return (const unsigned char *)current_->curUri.get_data();
+	return (const unsigned char *)
+		document_.getStringForID(getNode()->getURIIndex());
 }
 
 const unsigned char *
 NsEventReader::getLocalName() const
 {
-	if (!localName_)
-		throwIllegalOperation(type_, "getLocalName");
 	return localName_;
 }
 
@@ -252,17 +319,16 @@ const unsigned char *
 NsEventReader::getPrefix() const
 {
 	if (!localName_)
-		throwIllegalOperation(type_, "getPrefix");
-	DBXML_ASSERT(node_ && node_ == current_->node);
-	int32_t prefix = node_->namePrefix();
+		return 0;
+	DBXML_ASSERT(getNode() && getNode() == &current_->node);
+	int32_t prefix = getNode()->getPrefixIndex();
 	if (prefix == NS_NOPREFIX)
 		return 0;
-	document_.getStringForID(prefix, current_->curPrefix);
-	return (const unsigned char *)current_->curPrefix.get_data();
+	return 	(const unsigned char *) document_.getStringForID(prefix);
 }
 	
 const unsigned char *
-NsEventReader::getValue(int &len) const
+NsEventReader::getValue(size_t &len) const
 {
 	if (type_ == ProcessingInstruction) {
 		if (value_)
@@ -286,7 +352,8 @@ NsEventReader::needsEntityEscape(int index) const
 		return MAKEBOOL(textType_ & NS_ENTITY_CHK);
 	} else if (type_ == StartElement) {
 		ensureAttributes(index, "needsEntityEscape");
-		return node_->attrNeedsEscape(index);
+		fetchAttributes(index);
+		return ((attr_.a_flags & NS_ATTR_ENT) != 0);
 	} else if (type_ == CDATA)
 		return false;
 	throwIllegalOperation(type_, "needsEntityEscape");
@@ -299,13 +366,6 @@ NsEventReader::isEmptyElement() const {
 	return emptyElement_;
 }
 
-// EventReader extensions
-const NsNode *
-NsEventReader::getNsNode() const
-{
-	return node_;
-}
-
 const unsigned char *
 NsEventReader::getVersion() const
 {
@@ -366,48 +426,61 @@ NsEventReader::getAttributeCount() const
 	return nattrs_;
 }
 
+void
+NsEventReader::fetchAttributes(int index) const
+{
+	if (index != attrIndex_) {
+		curAttrName_ = getNode()->getNextAttr(curAttrName_,
+						  &attr_, index);
+		attrIndex_ = index;
+	}
+}
+
 bool
 NsEventReader::isAttributeSpecified(int index) const
 {
 	ensureAttributes(index, "isAttributeSpecified");
-	return node_->attrIsSpecified(index);
+	fetchAttributes(index);
+	bool notSpecified = (attr_.a_flags & NS_ATTR_NOT_SPECIFIED) != 0;
+	return (!notSpecified);
 }
 
 const unsigned char *
 NsEventReader::getAttributeLocalName(int index) const
 {
 	ensureAttributes(index, "getAttributeLocalName");
-	nsText_t &text = node_->getAttrName(index)->n_text;
-	return (const unsigned char *)text.t_chars;
+	fetchAttributes(index);
+	return curAttrName_;
 }
 
 const unsigned char *
 NsEventReader::getAttributeNamespaceURI(int index) const
 {
 	ensureAttributes(index, "getAttributeNamespaceURI");
-	int32_t uri = node_->attrUri(index);
+	fetchAttributes(index);
+	int32_t uri = attr_.a_uri;
 	if (uri == NS_NOURI)
 		return 0;
-	document_.getStringForID(uri, current_->attrUri);
-	return (const unsigned char *)current_->attrUri.get_data();
+	return 	(const unsigned char *) document_.getStringForID(uri);
 }
 
 const unsigned char *
 NsEventReader::getAttributePrefix(int index) const
 {
 	ensureAttributes(index, "getAttributePrefix");
-	int32_t prefix = node_->attrNamePrefix(index);
+	fetchAttributes(index);
+	int32_t prefix = attr_.a_name.n_prefix;
 	if (prefix == NS_NOPREFIX)
 		return 0;
-	document_.getStringForID(prefix, current_->attrPrefix);
-	return (const unsigned char *)current_->attrPrefix.get_data();
+	return (const unsigned char *) document_.getStringForID(prefix);
 }
 
 const unsigned char *
 NsEventReader::getAttributeValue(int index) const
 {
 	ensureAttributes(index, "getAttributeValue");
-	return (const unsigned char *) node_->attrValue(index);
+	fetchAttributes(index);
+	return (const unsigned char *)attr_.a_value;
 }
 
 XmlEventReader::XmlEventType
@@ -509,7 +582,8 @@ bool
 NsEventReader::doText()
 {
 	const nsTextEntry_t *entry =
-		current_->node->getTextEntry(current_->getTextIndex());
+		current_->node.getTextEntry(current_->getTextEntry(),
+					    current_->getTextIndex());
 	current_->incrText();
 
 	// initialize text event state
@@ -575,47 +649,39 @@ NsEventReader::doText()
 void
 NsEventReader::getNode(NsEventReaderNodeList *parent)
 {
-	Dbt key;
-	Dbt data;
+	DbXmlDbt key;
+	DbXmlDbt data;
 	current_ = new NsEventReaderNodeList(parent, currentBuffer_);
-	if(doInit_) {
-		nextNode(&current_->buffer, key, data, &startId_);
-		if(current_->buffer == 0) {
-			hasNext_ = false; // no-content doc
-			return;
+	try {
+		if(doInit_) {
+			nextNode(&current_->buffer, key, data, &startId_);
+			if(current_->buffer == 0) {
+				hasNext_ = false; // no-content doc
+				return;
+			}
+		} else {
+			nextNode(&current_->buffer, key, data);
+			DBXML_ASSERT(current_->buffer != 0);
+			DBXML_ASSERT(data.data);
 		}
-	} else {
-		nextNode(&current_->buffer, key, data);
-		DBXML_ASSERT(current_->buffer != 0);
-		DBXML_ASSERT(data.get_data());
+		currentBuffer_ = current_->buffer;
+		
+		current_->node.setNode(key, data);
+		current_->setState(doInit_);
+		DBXML_ASSERT(current_->node.getDocID() == document_.getDocID());
+	} catch (...) {
+		delete current_;
+		current_ = 0;
+		throw;
 	}
-	currentBuffer_ = current_->buffer;
-
-	// Because the node buffer will stick around until we've
-	// finished, we can set copyStrings to false, meaning that
-	// the NsNode strings point directly into the node buffer.
-	unsigned char *datap = (unsigned char *)data.get_data();
-
-	const NsFormat &fmt = NsFormat::getFormat((int)(*datap));
-	current_->node = fmt.unmarshalNodeData(memMgr_, datap,
-		/*adoptBuffer*/false);
-
-	current_->setState(doInit_);
-
-	// Unmarshal the key to get the node ID
-	DocID did;
-	fmt.unmarshalNodeKey(did, current_->node->getNid(), memMgr_,
-		(unsigned char *)key.get_data(), /*copyStrings*/false);
-
-	DBXML_ASSERT(did == document_.getDocId());
 
 	if(Log::isLogEnabled(Log::C_NODESTORE, Log::L_DEBUG)) {
-		ostringstream oss;
-		oss << "Fetched node: 0x";
-		current_->node->getNid()->displayNid(oss);
-		NsDocumentDatabase *ndb = document_.getDb();
-		Log::log(ndb->getDbEnv(), Log::C_NODESTORE, Log::L_DEBUG,
-			 ndb->getContainerName().c_str(), oss.str().c_str());
+		NsRawNode *n = &current_->node;
+		// raw node so cannot log the actual node state
+		NsFormat::logNodeOperation(*document_.getDocDb(),
+					   n->getDocID(), 0,
+					   n->getNodeID(), &data,
+					   "EventReader");
 	}
 }
 
@@ -627,10 +693,13 @@ NsEventReader::doElement(bool start)
 {
 	// initialize state
 	value_ = 0;
-	node_ = 0;
+	setNode(0);
+	attrIndex_ = -1;
+	curAttrName_ = 0;
 	// current_ is set by caller
-	uint32_t flags = current_->node->getFlags();
+	uint32_t flags = current_->node.getFlags();
 	if (flags & NS_ISDOCUMENT) {
+		localName_ = 0;
 		if(start) {
 			// TBD: not using sniffed encoding
 			type_ = StartDocument;
@@ -646,12 +715,11 @@ NsEventReader::doElement(bool start)
 			return false;
 
 		// even for EndElement, reassign names
-		node_ = current_->node;
-		localName_ = (const unsigned char*)
-			node_->getNameChars();
+		setNode(&current_->node);
+		localName_ = getNode()->getNodeName();
 		if(start) {
 			type_ = StartElement;
-			nattrs_ = node_->numAttrs();
+			nattrs_ = getNode()->getNumAttrs();
 		} else {
 			type_ = EndElement;
 			popElement_ = true;
@@ -680,14 +748,15 @@ NsEventReader::doElement(bool start)
 //
 
 void
-NsEventReader::nextNode(NsEventReaderBuf **bufp, Dbt &key, Dbt &data, NsNid *startId)
+NsEventReader::nextNode(NsEventReaderBuf **bufp, DbXmlDbt &key, DbXmlDbt &data, NsFullNid *startId)
 {
 	NsEventReaderBuf *buf = *bufp;
 	DBXML_ASSERT(buf || startId);
-	uint32_t bufSize = NS_EVENT_DB_BUFSIZE;
+	uint32_t bufSize = bufferSize_;
 	// are there any items left?
 	if(buf) {
-		if (!buf->iter.next(key, data)) {
+		DB_MULTIPLE_KEY_NEXT(buf->p, &buf->dbt, key.data, key.size, data.data, data.size);
+		if (buf->p == 0) {
 			buf->usedUp = true;
 			if (buf->nComplete == buf->nNodes) {
 				buf->reuseNext = reuseList_;
@@ -705,53 +774,53 @@ NsEventReader::nextNode(NsEventReaderBuf **bufp, Dbt &key, Dbt &data, NsNid *sta
 			reuseList_ = buf->reuseNext;
 		} else {
 			buf = (NsEventReaderBuf *)
-				malloc(bufSize + sizeof(NsEventReaderBuf));
-			if (!buf) {
-				NsUtil::nsThrowException(
-					XmlException::NO_MEMORY_ERROR,
-					"Malloc failed",
-					__FILE__, __LINE__);
-				return;
-			}
+				NsUtil::allocate(bufSize +
+						 sizeof(NsEventReaderBuf));
 			buf->nSize = bufSize;
 			buf->freeNext = freeList_;
 			freeList_ = buf;
+			// placement new to initialize DbXmlDbt
+			(void) new (&(buf->dbt)) DbXmlDbt();
 		}
 		buf->reuseNext = 0;
 		buf->nNodes = 0;
 		buf->nComplete = 0;
 		buf->usedUp = false;
 
-		Dbt data1; // does not need to live past this block
-		data1.set_flags(DB_DBT_USERMEM);
-		data1.set_ulen(buf->nSize);
-		data1.set_data(buf + 1);
+		buf->dbt.set_flags(DB_DBT_USERMEM);
+		buf->dbt.ulen = buf->nSize;
+		buf->dbt.data = buf + 1;
 		int ret = 0;
-		if (startId) {
-			NsFormat::marshalNodeKey(document_.getDocId(), startId, docKey_);
-			ret = cursor_.get(
-				&docKey_,
-				&data1,
-				cursorFlags_|DB_SET|DB_MULTIPLE_KEY);
-		} else {
-			ret = cursor_.get(
-				&docKey_,
-				&data1,
-				cursorFlags_|DB_NEXT|DB_MULTIPLE_KEY);
+		try {
+			if (startId) {
+				NsFormat::marshalNodeKey(document_.getDocID(),
+							 NsNid(startId), docKey_);
+				ret = cursor_.get(
+					docKey_,
+					buf->dbt,
+					cursorFlags_|DB_SET|DB_MULTIPLE_KEY);
+			} else {
+				ret = cursor_.get(
+					docKey_,
+					buf->dbt,
+					cursorFlags_|DB_NEXT|DB_MULTIPLE_KEY);
+			}
+		} catch (XmlException &xe) {
+			ret = xe.getDbErrno();
 		}
 		if (ret != 0) {
 			// if buf was just allocated, free it; otherwise,
 			// put it back on the reuse list
 			if (freeList_ == buf) {
 				freeList_ = buf->freeNext;
-				::free(buf);
+				NsUtil::deallocate(buf);
 			} else {
 				buf->reuseNext = reuseList_;
 				reuseList_ = buf;
 			}
 			if (ret == DB_BUFFER_SMALL) {
-				// necessary size is in data1.get_size()
-				while (bufSize < data1.get_size())
+				// necessary size is in buf->dbt.size
+				while (bufSize < buf->dbt.size)
 					bufSize <<= 1;
 				buf = 0;
 				continue;
@@ -766,17 +835,28 @@ NsEventReader::nextNode(NsEventReaderBuf **bufp, Dbt &key, Dbt &data, NsNid *sta
 			*bufp = buf; // 0
 			return;
 		}
-		// placement new to initialize multiple iter
-		(void) new (&(buf->iter)) DbMultipleKeyDataIterator(data1);
-		if (!buf->iter.next(key, data)) {
+		// initialize multiple iter
+		DB_MULTIPLE_INIT(buf->p, &buf->dbt);
+		DB_MULTIPLE_KEY_NEXT(buf->p, &buf->dbt, key.data, key.size, data.data, data.size);
+		if (buf->p == 0) {
 			NsUtil::nsThrowException(XmlException::INTERNAL_ERROR,
 						 "Failed to find node.",
 						 __FILE__, __LINE__);
 			return;
 		}
-
 		*bufp = buf;
 	}
+#ifdef FORMAT_DEBUG
+	if (_dumpFormatNodes) {
+		unsigned char *datap = (unsigned char *)data.data;
+		const NsFormat &fmt = NsFormat::getFormat((int)(*datap));
+		NsNode *nsNode = fmt.unmarshalNodeData(
+			datap, /*adoptBuffer*/false);
+		nsNode->acquire();
+		nsNode->release();
+	}
+#endif
+
 	buf->nNodes++;
 }
 
@@ -794,7 +874,6 @@ NsEventReader::endElement()
 	} else
 		current_ = tmp->parent;
 	releaseNode(tmp->buffer);
-	NsNode::freeNode(memMgr_, tmp->node);
 	delete tmp;
 }
 	
