@@ -1,15 +1,21 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: NsFormat.cpp,v 1.15 2006/11/01 15:57:28 jsnelson Exp $
 //
 #include "NsNode.hpp"
 #include "NsFormat.hpp"
 #include "NsUtil.hpp"
+#include "NsConstants.hpp"
 #include "../DocID.hpp"
 #include "../ScopedDbt.hpp"
+// next 3 are for the Db operations put/get/del
+#include "../DbWrapper.hpp"
+#include "../OperationContext.hpp"
+#include "../Cursor.hpp"
+#include "../Log.hpp"
+#include "../Buffer.hpp"
 
 #define FORMAT_DEBUG 1
 
@@ -167,22 +173,25 @@ using namespace DbXml;
 
 namespace DbXml {
 	
+int _dumpFormatNodes = 0;
 #ifdef FORMAT_DEBUG
-static int _dumpNodes = 0;
 void dumpNode(NsNode *node);
 #endif	
 
 class NsFormat2: public NsFormat {
 public:
-	virtual int marshalNodeKey(const DocID &did, const NsNid *nid,
+	virtual size_t marshalNodeKey(
+		const DocID &did, const NsNid &nid,
 		unsigned char *buf, bool count) const;
-	virtual void unmarshalNodeKey(DocID &did, NsNid *nid, XER_NS MemoryManager *mmgr,
+	virtual void unmarshalNodeKey(
+		DocID &did, NsFullNid *nid,
 		unsigned char *buf, bool copyStrings) const;
 
-	virtual int marshalNodeData(const NsNode *node,
-				unsigned char *buf, bool count) const;
-	virtual  NsNode *unmarshalNodeData(XER_NS MemoryManager *mmgr,
-				       unsigned char *buf, bool adoptBuffer) const;
+	virtual size_t marshalNodeData(
+		const NsNode *node,
+		unsigned char *buf, bool count) const;
+	virtual  NsNode *unmarshalNodeData(
+		unsigned char *buf, bool adoptBuffer, nodeRecordSizes *sizes = 0) const;
 };
 
 // NOTE: format 1 is not implemented here.  See NsUpgrade.*	
@@ -203,13 +212,261 @@ XERCES_CPP_NAMESPACE_USE
 	sizeof(nsAttrList_t) +		  \
 	sizeof(nsNav_t)
 
-/*
- * internal functions/macros to count bytes for marshaling
- */
-
 /* t_len is in UTF-8 bytes */
 #define _nsTextSize(t) ((t).t_len + 1) /* include null */
 
+/*
+ * node put/get/delete
+ */
+#define STACK_BUF_SIZE 512
+
+class NodeMarshalBuffer {
+public:
+	NodeMarshalBuffer(const NsNode *node, size_t nodeSize = 0)
+	{
+		const NsFormat &fmt = *node->getFormat();
+
+		data.data = stackBuf;
+		if (nodeSize)
+			data.size = (uint32_t)nodeSize;
+		else
+			data.size = (uint32_t)
+				fmt.marshalNodeData(node, NULL, true);
+		
+		if(data.size > STACK_BUF_SIZE) {
+			data.data = malloc(data.size);
+			if(!data.data)
+				NsUtil::nsThrowException(XmlException::NO_MEMORY_ERROR,
+					"NodeMarshalBuffer::NodeMarshalBuffer",
+					__FILE__, __LINE__);
+		}
+		fmt.marshalNodeData(node, (unsigned char*)data.data, false);
+	}
+
+	~NodeMarshalBuffer()
+	{
+		if(data.size > STACK_BUF_SIZE)
+			free(data.data);
+	}
+
+	DbXml::DbXmlDbt data;
+	unsigned int stackBuf[STACK_BUF_SIZE];
+};
+
+void NsFormat::logNodeOperation(DbWrapper &db, const DocID &did,
+				const NsNode *node, const NsNid &nid,
+				const DbXmlDbt *data, const char *op, int err)
+{
+	if(Log::isLogEnabled(Log::C_NODESTORE, Log::L_DEBUG)) {
+		ostringstream oss;
+		if (err != 0)
+			oss << "Failed node op: " << op << ", err " << err;
+		else
+			oss << op;
+		oss << ", did:nid: " << did.asString() << ":";
+		nid.displayNid(oss);
+		if (node) {
+			oss << ", node: ";
+			node->displayNode(oss);
+		} else if (data) {
+			oss << ", data: ";
+			Buffer b((const xmlbyte_t*)data->data, data->size, true);
+			oss << b.asString();
+		}
+		// TBD: get container name (from caller?)
+		std::string dbname = db.getDatabaseName();
+		if (!dbname.size())
+			dbname = "temp";
+		Log::log(db.getEnvironment(), Log::C_NODESTORE, Log::L_DEBUG,
+			 dbname.c_str(),
+			 oss.str().c_str());
+	}
+}
+
+int NsFormat::putNodeRecord(DbWrapper &db, OperationContext &context,
+			    const DocID &did, const NsNid &nid,
+			    const DbXmlDbt *data)
+{
+	marshalNodeKey(did, nid, context.key());
+	int ret = 0;
+	try {
+		// use bulk put in preference to a write cursor because it
+		// appears to be faster (write cursor will only be set for
+		// cache databases)
+		
+		// don't allocate if not already done
+		BulkPut *bp = context.getBulkPut(false);
+		if (bp) {
+			bp->setDbWrapper(&db);
+			if (bp->store(context.txn(), &context.key(),
+				      const_cast<DbXmlDbt*>(data)))
+				return 0;
+			// above will fail if node is too large for bulk buffer
+		}	
+		
+		Cursor *cursor = db.getWriteCursor();
+		if (cursor)
+			ret = cursor->put(context.key(),
+					  const_cast<DbXmlDbt&>(*data), DB_KEYFIRST);
+		else {
+			ret = db.put(context.txn(), &context.key(),
+				     const_cast<DbXmlDbt*>(data), 0);
+		}
+	} catch (XmlException &xe) {
+		if (xe.getExceptionCode() == XmlException::DATABASE_ERROR)
+			ret = xe.getDbErrno();
+		else
+			throw; // re-throw
+	}
+	return ret;
+}
+
+int NsFormat::putNodeRecord(DbWrapper &db, OperationContext &context,
+			    const DocID &id, const NsNode *node, bool add, size_t nodeSize)
+{
+	NodeMarshalBuffer nmb(node, nodeSize);
+	if(Log::isLogEnabled(Log::C_NODESTORE, Log::L_DEBUG))
+		logNodeOperation(db, id,
+				 node,
+				 const_cast<NsNode*>(node)->getNid(),
+				 NULL, // don't pass data
+				 (add ? "add" : "updating"));
+	return putNodeRecord(db, context, id, const_cast<NsNode*>(node)->getNid(),
+			     &nmb.data);
+}
+
+int NsFormat::delNodeRecord(DbWrapper &db, OperationContext &context,
+			    const DocID &did, const NsNid &nid)
+{
+	if(Log::isLogEnabled(Log::C_NODESTORE, Log::L_DEBUG))
+		logNodeOperation(db, did, 0, nid, 0, "deleting");
+
+	marshalNodeKey(did, nid, context.key());
+	int ret = 0;
+	try {
+		ret = db.del(context.txn(), &context.key(), 0);
+	} catch (XmlException &xe) {
+		if (xe.getExceptionCode() == XmlException::DATABASE_ERROR)
+			ret = xe.getDbErrno();
+		else
+			throw; // re-throw
+	}
+	return ret;
+}
+
+int NsFormat::getNodeRecord(DbWrapper &db, OperationContext &context,
+			    const DocID &did, const NsNid &nid, DbXmlDbt *data,
+			    u_int32_t flags)
+{
+	marshalNodeKey(did, nid, context.key());
+	int ret = 0;
+	try {
+		Cursor *cursor = db.getReadCursor();
+		if (cursor)
+			ret = cursor->get(context.key(), *data, DB_SET);
+		else
+			ret = db.get(context.txn(), &context.key(), data, flags);
+	} catch (XmlException &xe) {
+		if (xe.getExceptionCode() == XmlException::DATABASE_ERROR)
+			ret = xe.getDbErrno();
+		else
+			throw; // re-throw
+	}
+	return ret;
+}
+
+int NsFormat::getNextNodeRecord(DbWrapper &db, OperationContext &context,
+				const DocID &did, const NsNid &nid, DbXmlDbt *data,
+				u_int32_t flags)
+{
+	Cursor cursor(db, context.txn(), CURSOR_READ, 0);
+	if(cursor.error() != 0) return cursor.error();
+
+	// Marshal a key that comes directly after
+	// the given node ID
+	marshalNextNodeKey(did, nid, context.key());
+
+	// Find the next node record
+	int ret = 0;
+	try {
+		ret = cursor.get(context.key(), *data, DB_SET_RANGE);
+	} catch (XmlException &xe) {
+		if (xe.getExceptionCode() == XmlException::DATABASE_ERROR)
+			ret = xe.getDbErrno();
+		else
+			throw; // re-throw
+	}
+	cursor.close(); // avoid close in dtor
+	return ret;
+}
+
+int NsFormat::deleteAllNodes(DbWrapper &db, OperationContext &context,
+			     const DocID &did)
+{
+#if DBVER >= 48
+	Cursor cursor(db, context.txn(), CURSOR_WRITE, "bulkdel", DB_CURSOR_BULK);
+#else
+	Cursor cursor(db, context.txn(), CURSOR_WRITE, 0);
+#endif
+	if(cursor.error() != 0) return cursor.error();
+	int err = 0;
+	try {
+		DbtOut data;
+		data.set_flags(DB_DBT_PARTIAL); // only want keys
+		
+		marshalNodeKey(did, NsNid(), context.key());
+		err = cursor.get(context.key(), data, DB_SET_RANGE);
+		
+		DocID tmp;
+		while (err == 0) {
+			tmp.setThisFromDbt(context.key());
+			if(tmp != did) {
+				break;
+			}
+			err = cursor.del(0);
+			if(err == 0)
+				err = cursor.get(context.key(), data, DB_NEXT);
+		}
+		
+		if(err == DB_NOTFOUND)
+			err = 0;
+			
+	} catch (...) {
+		// deadlock probably
+		cursor.close();
+		throw;
+	}
+	cursor.close(); // avoid close in dtor
+	return err;
+}
+
+// fetch an unmarshaled node
+//static
+NsNode *NsFormat::fetchNode(const NsNid &nid,
+			    const DocID &did,
+			    DbWrapper &db,
+			    OperationContext &oc,
+			    bool forWrite)
+{
+	DbXmlDbt data;
+	data.set_flags(DB_DBT_MALLOC);
+	u_int32_t flags = ((forWrite && oc.txn() && db.isTransacted()) ? DB_RMW : 0);
+	int err = getNodeRecord(db, oc, did, nid,
+				&data, flags);
+	if (err)
+		throw XmlException(err);
+	unsigned char *buf = (unsigned char *)data.data;
+	const NsFormat &fmt = NsFormat::getFormat((int)(*buf));
+	NsNode *newNode = fmt.unmarshalNodeData(buf,
+						true /* adoptBuffer */);
+	newNode->getFullNid()->copyNid(nid.getBytes(), nid.getLen());
+	if(Log::isLogEnabled(Log::C_NODESTORE, Log::L_DEBUG))
+		logNodeOperation(db, did, newNode, nid,
+				 NULL, // don't pass data
+				 "fetchNode");
+	return newNode;
+}
+
 /*
  * marshal UTF-8 text, including trailing null.
  * The text has already been transcoded to UTF-8.
@@ -218,7 +475,7 @@ XERCES_CPP_NAMESPACE_USE
  * chance of buffer overflow.
  *
  */
-static inline int _nsMarshText(unsigned char *ptr, const nsText_t *text)
+static inline size_t _nsMarshText(unsigned char *ptr, const nsText_t *text)
 {
 	memcpy(ptr, text->t_chars, text->t_len + 1);
 	return (text->t_len + 1);
@@ -230,20 +487,21 @@ static inline int _nsMarshText(unsigned char *ptr, const nsText_t *text)
  * where the various indexes (ints) are based on their related flags fields.
  * unionTypeIndex can only exist if typeIndex is set, also.
  */
-static inline int _nsMarshAttr(unsigned char *ptr, const nsAttr_t *attr)
+static inline size_t _nsMarshAttr(unsigned char *ptr, const nsAttr_t *attr)
 {
 	// nsText_t field of the attribute name has entire
 	// name/value string, contiguous.
 	unsigned char *tptr = ptr;
-	tptr += NsFormat::marshalInt(tptr, attr->a_flags);
-	if (attr->a_flags & NS_ATTR_PREFIX)
+	uint32_t aflags = attr->a_flags & NS_ATTR_MASK;
+	tptr += NsFormat::marshalInt(tptr, aflags);
+	if (aflags & NS_ATTR_PREFIX)
 		tptr += NsFormat::marshalInt(tptr, attr->a_name.n_prefix);
-	if (attr->a_flags & NS_ATTR_URI)
+	if (aflags & NS_ATTR_URI)
 		tptr += NsFormat::marshalInt(tptr, attr->a_uri);
 #ifdef NS_USE_SCHEMATYPES	
-	if (attr->a_flags & NS_ATTR_TYPE) {
+	if (aflags & NS_ATTR_TYPE) {
 		tptr += NsFormat::marshalInt(tptr, attr->a_name.n_type);
-		if (attr->a_flags & NS_ATTR_UNION_TYPE)
+		if (aflags & NS_ATTR_UNION_TYPE)
 			tptr += NsFormat::marshalInt(tptr, attr->a_name.n_utype);
 	}
 #endif
@@ -256,22 +514,22 @@ static inline int _nsMarshAttr(unsigned char *ptr, const nsAttr_t *attr)
  * allocate "space" for xmlbyte_t from end of array.
  * endPP points to last *available* memory.
  */
-static inline void *_allocChars(xmlbyte_t **endPP, int len) {
+static inline xmlbyte_t *_allocChars(xmlbyte_t **endPP, size_t len) {
 	*endPP -= (len);
 	xmlbyte_t *ret = *endPP + 1;
-	return (void*) ret;
+	return (xmlbyte_t *) ret;
 }
 
 #define _nsAllocChars(dest, endPP, len) (dest) = _allocChars((endPP), (len))
 
-uint32_t
-NsFormat::unmarshalId(MemoryManager *mmgr, const xmlbyte_t *ptr, NsNid *id)
+size_t
+NsFormat::unmarshalId(const xmlbyte_t *ptr, NsFullNid *id)
 {
-	uint32_t len = NsUtil::nsStringLen(ptr) + 1;
+	size_t len = NsUtil::nsStringLen(ptr) + 1;
 	if(len == 1) {
-		id->freeNid(mmgr);
+		id->freeNid();
 	} else {
-		id->copyNid(mmgr, ptr, len);
+		id->copyNid(ptr, (unsigned int)len);
 	}
 	return len;
 }
@@ -281,12 +539,13 @@ NsFormat::unmarshalId(MemoryManager *mmgr, const xmlbyte_t *ptr, NsNid *id)
  * Eventually, this will use space off of endPP, which is
  * why it's passed.
  */
-uint32_t
-NsFormat::unmarshalId(const xmlbyte_t *ptr, NsNid *id, xmlbyte_t ** endPP, bool copyStrings)
+size_t
+NsFormat::unmarshalId(const xmlbyte_t *ptr, NsFullNid *id,
+		      xmlbyte_t ** endPP, bool copyStrings)
 {
-	uint32_t len = NsUtil::nsStringLen(ptr) + 1;
-	id->setLen(len, /*alloced*/false);
-	if (len > NsNid::nidStaticSize()) {
+	size_t len = NsUtil::nsStringLen(ptr) + 1;
+	id->setLen((unsigned int)len, /*alloced*/false);
+	if (len > NsFullNid::nidStaticSize()) {
 		xmlbyte_t *dest;
 		if(copyStrings) {
 			dest = (xmlbyte_t*)_allocChars(endPP, len);
@@ -309,11 +568,11 @@ NsFormat::unmarshalId(const xmlbyte_t *ptr, NsNid *id, xmlbyte_t ** endPP, bool
  */
 
 //static
-uint32_t
+size_t
 NsFormat::unmarshText(unsigned char *ptr, nsText_t *text,
 		      xmlbyte_t **endPP, bool copyStrings)
 {
-	uint32_t len = NsUtil::nsStringLen(ptr) + 1;
+	size_t len = NsUtil::nsStringLen(ptr) + 1;
 
 	if(copyStrings) {
 		/* allocate UTF-16 memory */
@@ -321,7 +580,7 @@ NsFormat::unmarshText(unsigned char *ptr, nsText_t *text,
 		memcpy(text->t_chars, ptr, len);
 	}
 	else {
-		text->t_chars = (void*)ptr;
+		text->t_chars = (xmlbyte_t *)ptr;
 	}
 	text->t_len = len - 1;
 	return len;
@@ -341,18 +600,20 @@ NsFormat::unmarshAttrs(unsigned char *ptr, nsAttrList_t *attrs,
 	for (uint32_t i = 0; i < attrs->al_nattrs; i++) {
 		nsAttr_t *attr = &attrs->al_attrs[i];
 		nsName_t *name = &attr->a_name;
-
 		ptr += NsFormat::unmarshalInt(ptr,
 					    (uint32_t*)&(attr->a_flags));
-		if (attr->a_flags & NS_ATTR_PREFIX)
+		if (attr->a_flags & NS_ATTR_PREFIX) {
 			ptr += NsFormat::unmarshalInt(
 				ptr, (uint32_t*)&(name->n_prefix));
-		else
+		} else {
 			name->n_prefix = NS_NOPREFIX;
-		if (attr->a_flags & NS_ATTR_URI)
+		}
+		if (attr->a_flags & NS_ATTR_URI) {
+			// TBD GMF think about using the xmlns uri index, if
+			// reliable, to set NS_ATTR_IS_DECL...
 			ptr += NsFormat::unmarshalInt(ptr,
 					     (uint32_t*)&(attr->a_uri));
-		else
+		} else
 			attr->a_uri = NS_NOURI;
 #ifdef NS_USE_SCHEMATYPES		
 		if (attr->a_flags & NS_ATTR_TYPE) {
@@ -365,18 +626,17 @@ NsFormat::unmarshAttrs(unsigned char *ptr, nsAttrList_t *attrs,
 			name->n_type = NS_NOTYPE;
 #endif
 
-		int name_len = NsUtil::nsStringLen(ptr) + 1; /* add null */
-		int alen = name_len + NsUtil::nsStringLen(ptr + name_len) + 1;
+		size_t name_len = NsUtil::nsStringLen(ptr) + 1; /* add null */
+		size_t alen = name_len + NsUtil::nsStringLen(ptr + name_len) + 1;
 
 		if(copyStrings) {
 			_nsAllocChars(name->n_text.t_chars, endPP, alen);
 			memcpy(name->n_text.t_chars, ptr, alen);
 		}
 		else {
-			name->n_text.t_chars = (void*)ptr;
+			name->n_text.t_chars = (xmlbyte_t *)ptr;
 		}
 		name->n_text.t_len = alen - 1;
-
 		// set value
 		attr->a_value = ((xmlbyte_t*)name->n_text.t_chars) + name_len;
 
@@ -402,7 +662,7 @@ NsFormat::unmarshTextList(unsigned char *ptr, nsTextList_t *text,
 	for (uint32_t i = 0; i < text->tl_ntext; i++) {
 		nsTextEntry_t *entry = &text->tl_text[i];
 		entry->te_type = (uint32_t) *ptr++;
-		int tlen = NsUtil::nsStringLen(ptr) + 1; /* add null */
+		size_t tlen = NsUtil::nsStringLen(ptr) + 1; /* add null */
 		if (nsTextType(entry->te_type) == NS_PINST)
 			tlen += NsUtil::nsStringLen(ptr + tlen) + 1;
 		if(copyStrings) {
@@ -410,7 +670,7 @@ NsFormat::unmarshTextList(unsigned char *ptr, nsTextList_t *text,
 			memcpy(entry->te_text.t_chars, ptr, tlen);
 		}
 		else {
-			entry->te_text.t_chars = (void*)ptr;
+			entry->te_text.t_chars = (xmlbyte_t *)ptr;
 		}
 		entry->te_text.t_len = tlen - 1;
 		/* tl_len includes all nulls */
@@ -426,60 +686,60 @@ NsFormat::unmarshTextList(unsigned char *ptr, nsTextList_t *text,
 
 // static
 void
-NsFormat::marshalNodeKey(const DocID &did, const NsNid *nid, DbtOut &dbt)
+NsFormat::marshalNodeKey(const DocID &did, const NsNid &nid, DbtOut &dbt)
 {
 	const NsFormat &fmt = NsFormat::getFormat(NS_PROTOCOL_VERSION);
 	
-	int size = fmt.marshalNodeKey(did, nid, 0, /*count*/true);
+	size_t size = fmt.marshalNodeKey(did, nid, 0, /*count*/true);
 	dbt.set(0, size);
 
-	fmt.marshalNodeKey(did, nid, (unsigned char*)dbt.get_data(),
+	fmt.marshalNodeKey(did, nid, (unsigned char*)dbt.data,
 		/*count*/false);
 }
 
 // static
 void
-NsFormat::marshalNextNodeKey(const DocID &did, const NsNid *nid, DbtOut &dbt)
+NsFormat::marshalNextNodeKey(const DocID &did, const NsNid &nid, DbtOut &dbt)
 {
 	const NsFormat &fmt = NsFormat::getFormat(NS_PROTOCOL_VERSION);
 	
-	int size = fmt.marshalNodeKey(did, nid, 0, /*count*/true);
+	size_t size = fmt.marshalNodeKey(did, nid, 0, /*count*/true);
 	dbt.set(0, size + 1);
 
-	fmt.marshalNodeKey(did, nid, (unsigned char*)dbt.get_data(),
+	fmt.marshalNodeKey(did, nid, (unsigned char*)dbt.data,
 		/*count*/false);
 
 	// We add an extra 0 at the end of the key - this turns it into an
 	// invalid node ID, but that doesn't matter as we just want to use
 	// this key with DB_SET_RANGE to get the next entry in the BTree.
-	((unsigned char*)dbt.get_data())[size] = 0;
+	((unsigned char*)dbt.data)[size] = 0;
 }
 
-int NsFormat2::marshalNodeKey(const DocID &did, const NsNid *nid,
+size_t NsFormat2::marshalNodeKey(const DocID &did, const NsNid &nid,
 	unsigned char *ptr, bool count) const
 {
-	int size = 0;
+	size_t size = 0;
 
 	if(count) {
 		size = did.marshalSize();
-		if(nid != 0)
+		if(!nid.isNull())
 			size += countId(nid);
 	} else {
 		ptr += did.marshal(ptr);
-		if(nid != 0)
+		if(!nid.isNull())
 			ptr += NsFormat::marshalId(ptr, nid);
 	}
 
 	return size;
 }
 
-void NsFormat2::unmarshalNodeKey(DocID &did, NsNid *nid,
-	MemoryManager *mmgr, unsigned char *ptr, bool copyStrings) const
+void NsFormat2::unmarshalNodeKey(DocID &did, NsFullNid *nid,
+	unsigned char *ptr, bool copyStrings) const
 {
 	ptr += did.unmarshal(ptr);
 
 	if(copyStrings) {
-		ptr += unmarshalId(mmgr, ptr, nid);
+		ptr += unmarshalId(ptr, nid);
 	} else {
 		ptr += unmarshalId(ptr, nid, 0, copyStrings);
 	}
@@ -511,13 +771,13 @@ void NsFormat2::unmarshalNodeKey(DocID &did, NsNid *nid,
 // if has child elems
 //    last child nid
 //
-int
+size_t
 NsFormat2::marshalNodeData(const NsNode *nodep, unsigned char *buf, bool count) const
 {
 	// accessors to NsNode are non-const, but safe
 	NsNode *node = (NsNode *)nodep;
 	unsigned char *ptr = buf;
-	int size = 0;
+	size_t size = 0;
 	// nattrs and ntext go in header to better enable
 	// unmarshal to estimate memory requirements
 	int32_t nattrs = 0;
@@ -530,13 +790,14 @@ NsFormat2::marshalNodeData(const NsNode *nodep, unsigned char *buf, bool count)
 	// set last_is_last_desc in flags
 	// before counting flags.  These flags are *only* used during
 	// marshal/unmarshal, and are not part of normal state
-	if (node->hasChildElem() && count) {
+	uint32_t tflags = 0;
+	if (node->hasChildElem()) {
 		if (*node->getLastChildNid() == *node->getLastDescendantNid())
-			node->setFlag(NS_LAST_IS_LAST_DESC);
+			tflags = NS_LAST_IS_LAST_DESC;
 	}
-	
-	uint32_t tflags = node->getFlags();
-	DBXML_ASSERT((tflags & NS_UTF16) == 0);
+
+	// add in "normal" flags
+	tflags |= node->getFlags();
 	if (count) {
 		size += 1; // Protocol version
 		size += countInt(tflags & NS_DBFLAGMASK);
@@ -599,31 +860,39 @@ NsFormat2::marshalNodeData(const NsNode *nodep, unsigned char *buf, bool count)
 	//  attributes
 	//  previous sibling, last child
 	//
-
-	// text: ntext is in header
+	// text: ntext is in header, acquired above
 	// <skipSize, [numChildText] (textType, text, 0)+>
+	//
 	if (ntext) {
 		nsTextList_t *list = node->getTextList();
-		uint32_t msize = list->tl_len +
+		size_t msize = list->tl_len +
 			list->tl_ntext; // type bytes
 		if (count) {
 			// ntext already counted
 			size += msize;
-			size += countInt(msize);
+			size += countInt((uint32_t)msize);
 			if (node->hasTextChild()) {
 				DBXML_ASSERT(list->tl_nchild);
 				size += countInt(list->tl_nchild);
 			}
 		} else {
-			ptr += marshalInt(ptr, msize);
+			ptr += marshalInt(ptr, (uint32_t)msize);
 			if (node->hasTextChild()) {
 				DBXML_ASSERT(list->tl_nchild);
 				ptr += marshalInt(ptr, list->tl_nchild);
 			}
-			for (int i = 0; i < ntext; i++) {
-				*ptr++ = (unsigned char)list->tl_text[i].te_type;
+			// skip delete text nodes
+			int index = 0;
+			int i = 0;
+			while (i < ntext) {
+				if (list->tl_text[index].te_type == NS_DELETEDTEXT) {
+					index++;
+					continue;
+				}
+				*ptr++ = (unsigned char)list->tl_text[index].te_type;
 				ptr += _nsMarshText(ptr,
-					    &(list->tl_text[i].te_text));
+						    &(list->tl_text[index].te_text));
+				i++; index++;
 			}
 		}
 	}
@@ -633,35 +902,40 @@ NsFormat2::marshalNodeData(const NsNode *nodep, unsigned char *buf, bool count)
 	if (nattrs) {
 		nsAttrList_t *attrs = node->getAttrList();
 		if (count) {
-			uint32_t msize = attrs->al_len;
+			size_t msize = attrs->al_len;
 			// num attrs already counted
 			// need to count optional:
 			//  prefixes, uris, and types
 			for (int i = 0; i < nattrs; i++) {
 				nsAttr_t *attr = &(attrs->al_attrs[i]);
+				// if attr value is 0, it's been removed, continue
+				if (!attr->a_value) { ++nattrs; continue;}
 				msize += 1; // a_flags are ALWAYS 1 byte
-				if (attr->a_flags & NS_ATTR_PREFIX)
+				uint32_t aflags = attr->a_flags & NS_ATTR_MASK;
+				if (aflags & NS_ATTR_PREFIX)
 					msize += countInt(attr->a_name.n_prefix);
-				if (attr->a_flags & NS_ATTR_URI)
+				if (aflags & NS_ATTR_URI)
 					msize += countInt(attr->a_uri);
 #ifdef NS_USE_SCHEMATYPES				
-				if (attr->a_flags & NS_ATTR_TYPE) {
+				if (aflags & NS_ATTR_TYPE) {
 					msize += countInt(attr->a_name.n_type);
-					if (attr->a_flags &
+					if (aflags &
 					    NS_ATTR_UNION_TYPE)
 						msize += countInt(attr->a_name.n_utype);
 				}
 #endif
 			}
 			size += msize;
-			size += countInt(msize);
+			size += countInt((uint32_t)msize);
 			attrs->al_marshalLen = msize;
 		} else {
 			// TBD: either use skip size or nuke it...
-			ptr += marshalInt(ptr, attrs->al_marshalLen);
+			ptr += marshalInt(ptr, (uint32_t)attrs->al_marshalLen);
 			for (int i = 0; i < nattrs; i++) {
-				ptr += _nsMarshAttr(ptr,
-						    &(attrs->al_attrs[i]));
+				nsAttr_t *attr = &(attrs->al_attrs[i]);
+				// if attr value is 0, it's been removed, continue
+				if (!attr->a_value) { ++nattrs; continue;}
+				ptr += _nsMarshAttr(ptr, attr);
 			}
 		}
 	}
@@ -670,7 +944,7 @@ NsFormat2::marshalNodeData(const NsNode *nodep, unsigned char *buf, bool count)
 	if (count) {
 		if (node->hasPrev())
 			size += countId(node->getPrevNid());
-		if (node->hasChildElem() && !node->lastIsLastDescendant()) {
+		if (node->hasChildElem() && !(tflags & NS_LAST_IS_LAST_DESC)) {
 
 			size += countId(node->getLastChildNid());
 
@@ -680,26 +954,27 @@ NsFormat2::marshalNodeData(const NsNode *nodep, unsigned char *buf, bool count)
 	} else {
 		if (node->hasPrev())
 			ptr += marshalId(ptr, node->getPrevNid());
-		if (node->hasChildElem() && !node->lastIsLastDescendant()) {
+		if (node->hasChildElem() && !(tflags & NS_LAST_IS_LAST_DESC)){
 			ptr += marshalId(ptr, node->getLastChildNid());
 		}
 	}
-	
+
 	return size;
 }
 
 
 NsNode *
-NsFormat2::unmarshalNodeData(MemoryManager *mmgr, unsigned char *buf,
-	bool adoptBuffer) const
+NsFormat2::unmarshalNodeData(unsigned char *buf,
+			     bool adoptBuffer, nodeRecordSizes *sizes) const
 {
-	uint32_t allocSize = NS_UNMARSH_MIN_2;
+	size_t allocSize = NS_UNMARSH_MIN_2;
 	uint32_t flags, dummy;
 	uint32_t nattrs = 0;
 	uint32_t ntext = 0;
 	NsNode *node;
 	char *curP;
 	unsigned char *ptr = buf;
+	unsigned char *prev = ptr;
 	xmlbyte_t *endP;
 
 	if (*ptr++ != NS_PROTOCOL_VERSION)
@@ -723,17 +998,12 @@ NsFormat2::unmarshalNodeData(MemoryManager *mmgr, unsigned char *buf,
 	}
 
 	// Allocate memory for the node
-	DBXML_ASSERT(mmgr);
-	curP = (char *)mmgr->allocate(allocSize);
-	if (!curP)
-		NsUtil::nsThrowException(XmlException::NO_MEMORY_ERROR,
-					 "unmarshalNode failed to allocate memory",
-					 __FILE__, __LINE__);
+	curP = (char *)NsUtil::allocate(allocSize);
 	memset(curP, 0, allocSize);
 
 	node = (NsNode*)curP;
 	node->setFormat(this);
-	*node->getFlagsPtr() = flags;
+	*node->getFlagsPtr() = flags & ~(NS_LAST_IS_LAST_DESC);
 
 	if(adoptBuffer) node->setMemory(buf);
 
@@ -771,12 +1041,18 @@ NsFormat2::unmarshalNodeData(MemoryManager *mmgr, unsigned char *buf,
 		} else
 			nm->n_type = NS_NOTYPE;
 #endif
+		prev = ptr;
 		ptr += unmarshText(ptr, &(nm->n_text),
 				   0, /*copyStrings*/false);
+		if(sizes)
+			sizes->name += ptr - prev;
 	} else
 		node->setLevel(0); // document node is level 0
 
-
+	if(sizes) {
+		sizes->header += ptr - buf;
+		prev = ptr;
+	}
 
 	//
 	// TBD: unmarshal on demand -- either that, or drop skip size
@@ -799,6 +1075,10 @@ NsFormat2::unmarshalNodeData(MemoryManager *mmgr, unsigned char *buf,
 		curP += ((ntext - 1) * sizeof(nsTextEntry_t));
 		// TBD: handle external text
 		ptr = unmarshTextList(ptr, text, &endP, /*copyStrings*/false);
+		if(sizes) {
+			sizes->text += ptr - prev;
+			prev = ptr;
+		}
 	}
 
 	// attributes: nattrs is in header
@@ -815,6 +1095,10 @@ NsFormat2::unmarshalNodeData(MemoryManager *mmgr, unsigned char *buf,
 		curP += ((nattrs - 1) * sizeof(nsAttr_t));
 		/* unmarshal the attributes */
 		ptr = unmarshAttrs(ptr, attrs, &endP, /*copyStrings*/false);
+		if(sizes) {
+			sizes->attributes += ptr - prev;
+			prev = ptr;
+		}
 	}
 
 	if (node->needsNav()) {
@@ -830,14 +1114,18 @@ NsFormat2::unmarshalNodeData(MemoryManager *mmgr, unsigned char *buf,
 
 	// last child
 	if (node->hasChildElem()) {
-		if(node->lastIsLastDescendant()) {
-			node->getLastChildNid()->copyNid(mmgr,
+		if (flags & NS_LAST_IS_LAST_DESC) {
+			node->getLastChildNid()->copyNid(
 				node->getLastDescendantNid());
 		} else {
 			ptr += unmarshalId(ptr, node->getLastChildNid(),
 				&endP, /*copyStrings*/false);
 		}
 	}
+	if(sizes) {
+		sizes->navigation += ptr - prev;
+		prev = ptr;
+	}
 
 	/* debug */
 	if (curP >= (char *)endP) {
@@ -845,10 +1133,8 @@ NsFormat2::unmarshalNodeData(MemoryManager *mmgr, unsigned char *buf,
 					 "unmarshalNode overlap",
 					 __FILE__, __LINE__);
 	}
-	// clear LAST_IS_LAST_DESC flags
-	node->clearFlag(NS_LAST_IS_LAST_DESC);
 #ifdef FORMAT_DEBUG
-	if (_dumpNodes)
+	if (_dumpFormatNodes && node)
 		dumpNode(node);
 #endif	
 	return node;
@@ -916,7 +1202,7 @@ int NsFormat::marshalInt64(xmlbyte_t *buf, uint64_t i)
 
 int NsFormat::unmarshalInt64(const xmlbyte_t *buf, uint64_t *i)
 {
-	int len;
+	int len = 0;
 	uint64_t tmp = 0;
 	unsigned char *p = (unsigned char *) &tmp;
 	unsigned char c = buf[0];
@@ -977,34 +1263,61 @@ int NsFormat::unmarshalInt64(const xmlbyte_t *buf, uint64_t *i)
 	return len;
 }
 
+// static
+size_t NsFormat::getNodeDataSize(const NsNode *node)
+{
+	const NsFormat &fmt =
+		NsFormat::getFormat(NS_PROTOCOL_VERSION);
+	return fmt.marshalNodeData(node, 0, /*count*/true);
+}
+
 #ifdef FORMAT_DEBUG
 namespace DbXml {
+
+void dumpFlags(uint32_t flags, ostringstream &out)
+{
+	out << "\n\tFlags: (" << hex << flags << ")" <<
+		(flags & NS_HASCHILD ? "NS_HASCHILD," : "") <<
+		(flags & NS_HASTEXT ? "NS_HASTEXT," : "") <<
+		(flags & NS_HASTEXTCHILD ? "NS_HASTEXTCHILD," : "") <<
+		(flags & NS_HASNEXT ? "NS_HASNEXT," : "") <<
+		(flags & NS_HASPREV ? "NS_HASPREV," : "");
+	// NS_LAST_IS_LAST_DESC will have been cleared at this point
+}
+	
 void dumpNode(NsNode *node)
 {
 	ostringstream out;
-	out << "Node: " << (const char *)node->getNameChars() << ",";
-	node->getNid()->displayNid(out);
-	if (!node->isDoc()) {
-		out << "\n\tparent: ";
+	if (node->isDoc()) {
+		out << "Node: Document";
+	} else {
+		const char *name = (const char *)node->getNameChars();
+		out << "Node:\n\tName, uri, level: " << (name ? name : "no name")
+		    << "," << node->uriIndex() << ", " << node->getLevel();
+		out << "\n\tparent nid: ";
 		node->getParentNid()->displayNid(out);
-	} else
-		out << "\n\tDocument";
-	out << "\n\tlast child: ";
-	if (node->getLastChildNid())
+	}
+	/* Nid isn't set
+	   node->getNid().displayNid(out);
+	*/
+	out << "\n\tlast child, last descendant: ";
+	if (node->hasChildElem()) {
 		node->getLastChildNid()->displayNid(out);
-	else
+		out << ", ";
+		node->getLastDescendantNid()->displayNid(out);
+	} else
 		out << "null";
 
-	out << "\n\tprev/lastDesc: ";
-	if (node->getPrevNid())
+	out << "\n\tprev: ";
+	if (node->hasPrev())
 		node->getPrevNid()->displayNid(out);
 	else
 		out << "null";
-	out << "/";
-	if (node->getLastDescendantNid())
-		node->getLastDescendantNid()->displayNid(out);
+	if (node->hasAttributes())
+		out << "\n\thas attributes -- " << node->numAttrs();
 	else
-		out << "null";
+		out << "\n\tno attributes";
+	dumpFlags(node->getFlags(), out);
 	out << "\n\t";
 
 	std::cout << out.str() << endl;
