@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: NsTypes.hpp,v 1.33 2006/10/30 17:45:57 bostic Exp $
 //
 
 /*
@@ -15,19 +14,18 @@
 #ifndef __DBXMLNSTYPES_HPP
 #define __DBXMLNSTYPES_HPP
 
-#include <dbxml/DbXmlFwd.hpp>
+#include "../DbXmlInternal.hpp"
 #include <xercesc/framework/MemoryManager.hpp>
 
 // Put all Xerces forward decls in one place (TBD: find better place)
-namespace XERCESC {
+namespace XERCES_CPP_NAMESPACE {
 	class MemoryManager;
 };
-#define XER_NS XERCESC::
 
 /*
  * Atomic types for node storage.  Size matters.
  */
-#include <db_cxx.h>
+#include <db.h>
 #define uint8_t u_int8_t
 #define uint16_t u_int16_t
 #define uint32_t u_int32_t
