@@ -1,7 +1,7 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
 // Some of the transcoding code is from Xerces, and is
 // under the Apache license:
@@ -11,7 +11,6 @@
 // Copyright (c) 1999-2004 The Apache Software Foundation.  All rights
 //  reserved.
 //
-// $Id: NsUtil.cpp,v 1.63 2006/10/30 17:45:57 bostic Exp $
 //
 
 #include "NsUtil.hpp"
@@ -19,15 +18,38 @@
 #include "NsConstants.hpp"
 #include "Manager.hpp"
 #include "Buffer.hpp"
-#include <db_cxx.h>
+#include <db.h>
 #include <xercesc/util/XMLUTF8Transcoder.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/sax/SAXParseException.hpp>
+#include <xqilla/framework/XPath2MemoryManager.hpp>
 
 using namespace DbXml;
 
 XERCES_CPP_NAMESPACE_USE
 
+//static
+void *NsUtil::allocate(size_t size, const char *loc)
+{
+	void *ret = ::malloc(size);
+	if (!ret) {
+		std::string msg = "allocation failed";
+		if (loc) {
+			msg += ": ";
+			msg += loc;
+		}
+		NsUtil::nsThrowException(XmlException::NO_MEMORY_ERROR,
+					 msg.c_str());
+	}
+	return ret;
+}
+//static
+void NsUtil::deallocate(void *obj)
+{
+	::free(obj);
+}
+
+
 //
 // Transcoding
 //
@@ -106,31 +128,25 @@ const xmlbyte_t NsUtil::gFirstByteMark[7] =
  *  o returns number of *bytes* in the transcoded buffer.
  *  o code is derived from the Xerces class, XMLUTF8Transcoder
  *  o returns true or false in hasEntity if a special entity is
- *  encountered: &, <, >, ", '
+ *  encountered: '&', '<', '>', '"', ''
  *  No error checking -- the UTF-16 input was created by Xerces, and
  *  is trusted.
  */
 
-int
-NsUtil::nsToUTF8(MemoryManager *mmgr, xmlbyte_t **dest, const xmlch_t *src,
-		 uint32_t nchars, uint32_t maxbytes, bool *hasEntity,
+size_t
+NsUtil::nsToUTF8(xmlbyte_t **dest, const xmlch_t *src,
+		 size_t nchars, size_t maxbytes, bool *hasEntity,
 		 enum checkType type)
 {
-	uint32_t bufsize;
+	size_t bufsize;
+	size_t nrb = 0;
 	xmlbyte_t *outPtr = *dest;
 	if (hasEntity)
 		*hasEntity = false;
 	if (!outPtr) {
 		bufsize = nchars * 3;
-		// caller needs to know if using manager or new for allocation
-		if (mmgr)
-			outPtr = (xmlbyte_t *) mmgr->allocate(bufsize);
-		else
-			outPtr = new xmlbyte_t[bufsize];
-		if (!outPtr)
-			nsThrowException(XmlException::NO_MEMORY_ERROR,
-				 "Unable to allocate memory for transcoding",
-					 __FILE__, __LINE__);
+		outPtr = (xmlbyte_t *)NsUtil::allocate(
+			bufsize * sizeof(xmlbyte_t));
 		*dest = outPtr;
 	} else {
 		bufsize = maxbytes;
@@ -149,24 +165,16 @@ NsUtil::nsToUTF8(MemoryManager *mmgr, xmlbyte_t **dest, const xmlch_t *src,
 		src++;
 		CHECK_ENT(curVal);
 		// how many bytes
-		uint32_t nbytes = codepointToUTF8Size(curVal);
+		int nbytes = codepointToUTF8Size(curVal);
 
 		if (outPtr + nbytes > outEnd) {
 			// reallocate (double size), or fail
-			if (!mmgr)
-				nsThrowException(XmlException::INTERNAL_ERROR,
-						 "nsToUTF8: no memory manager",
-						 __FILE__, __LINE__);
-			uint32_t count = (outPtr - *dest);
+			size_t count = (outPtr - *dest);
 			xmlbyte_t *newmem =
-				(xmlbyte_t *) mmgr->allocate(bufsize << 1);
-			if (!newmem)
-				nsThrowException(XmlException::NO_MEMORY_ERROR,
-						 "nsToUTF8 failed to allocate",
-						 __FILE__, __LINE__);
+				(xmlbyte_t *) NsUtil::allocate(bufsize << 1);
 			memcpy(newmem, *dest, bufsize);
 			bufsize <<= 1;
-			mmgr->deallocate(*dest);
+			NsUtil::deallocate(*dest);
 			*dest = newmem;
 			outEnd = newmem + bufsize;
 			outPtr = newmem + count;
@@ -203,9 +211,9 @@ static inline void checkTrailingBytes(const xmlbyte_t toCheck)
  *
  * The string is not entirely trusted as a valid utf-8 encoding.
  */
-static int
+static size_t
 _nsFromUTF8(xmlch_t *dest, const unsigned char *src,
-	    uint32_t nbytes, uint32_t maxchars, uint32_t &eaten)
+	    size_t nbytes, size_t maxchars, size_t &eaten)
 {
 	DBXML_ASSERT(nbytes);
 
@@ -223,7 +231,7 @@ _nsFromUTF8(xmlch_t *dest, const unsigned char *src,
 		}
 
 		// how many trailing src bytes will this sequence require
-		const uint32_t utf8size = NsUtil::nsCharSizeUTF8(*srcP);
+		const int utf8size = (int)NsUtil::nsCharSizeUTF8(*srcP);
 
 		// if not enough source to do an entire character, break out
 		// before processing the current one
@@ -256,7 +264,7 @@ _nsFromUTF8(xmlch_t *dest, const unsigned char *src,
 		 * where E0 is legal as a first byte, 9F is not legal
 		 * as a second byte.
 		 *
-		 * The byte sequence <F4 80 83 92> is legal,
+		 * The byte sequence '<F4 80 83 92>' is legal,
 		 * since every byte in that sequence matches
 		 * a byte range in a row of the table (the last row).
 		 *
@@ -412,24 +420,17 @@ _nsFromUTF8(xmlch_t *dest, const unsigned char *src,
  * characters that resulted, including NULLs, if any are included
  * in nbytes.
  */
-int
-NsUtil::nsFromUTF8(MemoryManager *mmgr, xmlch_t **dest, const xmlbyte_t *src,
-		  uint32_t nbytes, uint32_t maxchars)
+size_t
+NsUtil::nsFromUTF8(xmlch_t **dest, const xmlbyte_t *src,
+		   size_t nbytes, size_t maxchars)
 {
-	uint32_t bufsize;
+	size_t bufsize;
 	xmlch_t *outPtr = *dest;
-	uint32_t eaten;
+	size_t eaten;
 	if (!outPtr) {
 		bufsize = nbytes << 1; // multiply by 2
 		// caller needs to know if using manager or new for allocation
-		if (mmgr)
-			outPtr = (xmlch_t *) mmgr->allocate(bufsize);
-		else
-			outPtr = new xmlch_t[nbytes];
-		if (!outPtr)
-			nsThrowException(XmlException::NO_MEMORY_ERROR,
-					 "nsFromUTF8: allocation failed",
-					 __FILE__, __LINE__);
+		outPtr = (xmlch_t *) NsUtil::allocate(bufsize);
 		*dest = outPtr;
 	}
 	if (!maxchars)
@@ -441,34 +442,24 @@ NsUtil::nsFromUTF8(MemoryManager *mmgr, xmlch_t **dest, const xmlbyte_t *src,
 // String routines
 
 xmlbyte_t *
-NsUtil::nsStringDup(MemoryManager *mmgr, const xmlbyte_t *str, int *lenP)
+NsUtil::nsStringDup(const xmlbyte_t *str, size_t *lenP)
 {
 	if(str == 0) return 0;
-	int len = nsStringLen(str) + 1;
-	xmlbyte_t *copy = (xmlbyte_t *) mmgr->allocate(len);
-	if (copy)
-		memcpy(copy, str, len);
-	else
-		nsThrowException(XmlException::NO_MEMORY_ERROR,
-				 "nsStringDup failed to allocate memory",
-				 __FILE__, __LINE__);
+	size_t len = nsStringLen(str) + 1;
+	xmlbyte_t *copy = (xmlbyte_t *) NsUtil::allocate(len);
+	memcpy(copy, str, len);
 	if (lenP)
 		*lenP = len;
 	return copy;
 }
 
 xmlch_t *
-NsUtil::nsStringDup(MemoryManager *mmgr, const xmlch_t *str, int *lenP)
+NsUtil::nsStringDup(const xmlch_t *str, size_t *lenP)
 {
 	if(str == 0) return 0;
-	int len = (nsStringLen(str) + 1) << 1;
-	xmlch_t *copy = (xmlch_t *) mmgr->allocate(len);
-	if (copy)
-		memcpy(copy, str, len);
-	else
-		nsThrowException(XmlException::NO_MEMORY_ERROR,
-				 "nsStringDup failed to allocate memory",
-				 __FILE__, __LINE__);
+	size_t len = (nsStringLen(str) + 1) << 1;
+	xmlch_t *copy = (xmlch_t *) NsUtil::allocate(len);
+	memcpy(copy, str, len);
 	if (lenP)
 		*lenP = (len >> 1); // divide bytes by 2
 	return copy;
@@ -489,31 +480,77 @@ NsUtil::nsStringEqual(const xmlch_t *str1, const xmlch_t *str2)
 	return false;
 }
 
+int NsUtil::stringEqualsIgnoreCase(const char *s1, const char *s2)
+{
+	u_char s1ch, s2ch;
+
+	for (;;) {
+		s1ch = *s1++;
+		s2ch = *s2++;
+		if (s1ch >= 'A' && s1ch <= 'Z')		/* tolower() */
+			s1ch += 32;
+		if (s2ch >= 'A' && s2ch <= 'Z')		/* tolower() */
+			s2ch += 32;
+		if (s1ch != s2ch)
+			return (s1ch - s2ch);
+		if (s1ch == '\0')
+			return (0);
+	}
+	/* NOTREACHED */
+}
+
+int NsUtil::stringNEqualsIgnoreCase(const char *s1, const char *s2, size_t n)
+{
+	u_char s1ch, s2ch;
+
+	for (; n != 0; --n) {
+		s1ch = *s1++;
+		s2ch = *s2++;
+		if (s1ch >= 'A' && s1ch <= 'Z')		/* tolower() */
+			s1ch += 32;
+		if (s2ch >= 'A' && s2ch <= 'Z')		/* tolower() */
+			s2ch += 32;
+		if (s1ch != s2ch)
+			return (s1ch - s2ch);
+		if (s1ch == '\0')
+			return (0);
+	}
+	return (0);
+}
+
 /*
- * Escape < & "
+ * Escape < & " and sometimes >
  */
 static uint32_t _amp = '&';
 static uint32_t _lt = '<';
 static uint32_t _gt = '>';
 static uint32_t _dq = '"';
 static uint32_t _sp = ' ';
+static uint32_t _rb = ']';
 static const char *_ampStr = "&amp;";
 static const char *_ltStr = "&lt;";
+static const char *_gtStr = "&gt;";
 static const char *_dquotStr = "&quot;";
 #define _isNum(ch) (((ch) >= '0') && ((ch) <= '9'))
 //
-// only escape &, <, and "
+// only escape '&', '<', and '"'
+// and '>' if it follows 2 ']' characters (per XML spec)
 //
-uint32_t
-NsUtil::nsEscape(char *dest, const xmlbyte_t *src, int len, bool isAttr)
+size_t
+NsUtil::nsEscape(char *dest, const xmlbyte_t *src, size_t len, bool isAttr)
 {
 	char *start = dest;
 	uint32_t ch;
+	size_t nrb = 0; // num right bracket
 	while (--len != 0 && (ch = *src++) != 0) {
 		// reduce odds on common characters.  adds instructions
 		// for real escapes, and some other characters,
 		// but they are presumed uncommon
 		if (ch > _gt || ch == _sp || _isNum(ch)) {
+			if (ch == _rb)
+				++nrb;
+			else
+				nrb = 0;
 			*dest++ = ch;
 		} else {
 			if (ch == _amp) {
@@ -525,9 +562,13 @@ NsUtil::nsEscape(char *dest, const xmlbyte_t *src, int len, bool isAttr)
 			} else if ((ch == _dq) && isAttr) {
 				memcpy(dest, _dquotStr, 6);
 				dest += 6;
+			} else if ((ch == _gt) && nrb >= 2) {
+				memcpy(dest, _gtStr, 4);
+				dest += 4;
 			} else {
 				*dest++ = ch;
 			}
+			nrb = 0;
 		}
 	}
 	*dest = '\0';
@@ -547,12 +588,6 @@ NsUtil::nsThrowException(XmlException::ExceptionCode code, const char *desc,
 	throw XmlException(code, desc, file, line);
 }
 
-void
-NsUtil::nsThrowDbException(const DbException &e, const char *file, int line)
-{
-	throw XmlException(e, file, line);
-}
-
 void
 NsUtil::nsThrowParseException(const char *message)
 {
@@ -565,7 +600,7 @@ NsUtil::nsThrowParseException(const char *message)
 // strings to other objects. This happens during node storage parsing.
 //
 
-NsDonator::NsDonator(MemoryManager *mmgr, const xmlch_t *src, uint32_t len,
+NsDonator::NsDonator(const xmlch_t *src, size_t len,
 		     enum checkType type)
 	: _str(0), _str2(0), _len(0),
 	  _hasEntity(false)
@@ -575,40 +610,38 @@ NsDonator::NsDonator(MemoryManager *mmgr, const xmlch_t *src, uint32_t len,
 		return;
 	bool checkEntity = (type == ignore ? false : true);
 	if (src && *src)
-		_len = NsUtil::nsToUTF8(mmgr, &_str, src,
+		_len = NsUtil::nsToUTF8(&_str, src,
 					len+1, 0, (checkEntity ?
 						   &_hasEntity : 0), type) - 1;
 	// if no src, parts will be uninitialized, and _str is null
 }
 
-NsDonator::NsDonator(MemoryManager *mmgr, const xmlch_t *src1,
-		     const xmlch_t *src2, enum checkType type)
+NsDonator::NsDonator(const xmlch_t *src1,
+		     const xmlch_t *src2,
+		     XPath2MemoryManager *mmgr,
+		     enum checkType type)
 	:_str(0), _str2(0), _len(0), _hasEntity(false)
 {
 	if(src1 != 0 || src2 != 0) {
 		bool checkEntity = (type == ignore ? false : true);
-		int nlen = src1 ? NsUtil::nsStringLen(src1) : 0;
-		int vlen = src2 ? NsUtil::nsStringLen(src2) : 0;
+		size_t nlen = src1 ? NsUtil::nsStringLen(src1) : 0;
+		size_t vlen = src2 ? NsUtil::nsStringLen(src2) : 0;
 
-		int sz = (nlen + vlen);
+		size_t sz = (nlen + vlen);
 		// make plenty of space for transcoding (3x) and 2 nulls.
 		sz = (sz * 3) + 4;
-		xmlbyte_t *str = (xmlbyte_t *) mmgr->allocate(sz);
-		if (!str)
-			NsUtil::nsThrowException(XmlException::NO_MEMORY_ERROR,
-				"NsDonator: allocation failed",
-				__FILE__, __LINE__);
-
-		uint32_t tlen = 0;
+		xmlbyte_t *str = (xmlbyte_t *) (mmgr ? mmgr->allocate(sz) :
+						NsUtil::allocate(sz));
+		size_t tlen = 0;
 		if(src1 != 0) {
 			_str = str;
 			// never check for entities on first string
-			tlen = NsUtil::nsToUTF8(NULL, &str, src1, nlen + 1, sz, 0);
+			tlen = NsUtil::nsToUTF8(&str, src1, nlen + 1, sz, 0);
 			str += tlen;
 		}
 		if(src2 != 0) {
 			_str2 = str;
-			tlen += NsUtil::nsToUTF8(NULL, &str, src2, vlen + 1, sz - tlen,
+			tlen += NsUtil::nsToUTF8(&str, src2, vlen + 1, sz - tlen,
 				(checkEntity ? &_hasEntity : 0), type);
 		}
 		_len = tlen;
@@ -797,3 +830,23 @@ void NsUtil::decodeHexBinary(Buffer *buffer, const char *p, size_t l)
 	}
 }
 
+NsString::NsString(const xmlch_t *text)
+{
+	text_ = NsUtil::nsStringDup(text, 0);
+	owned_ = true;
+}
+
+void NsString::set(const xmlch_t *text, bool owned)
+{
+	if (text_ && owned_)
+		::free((void *)text_);
+	text_ = text;
+	owned_ = owned;
+}
+
+void NsString::set(const xmlch_t *text)
+{
+	clear();
+	text_ = NsUtil::nsStringDup(text, 0);
+	owned_ = true;
+}
