@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: NsUtil.hpp,v 1.42 2006/10/30 17:45:57 bostic Exp $
 //
 
 #ifndef __DBXMLNSUTIL_HPP
@@ -30,22 +29,31 @@
  * NsUtil, for namespace and convenience.
  */
 
-class DbException;
 /*
  * The only issues in text and attribute values are
- * for the characters: & and <
+ * *mostly* for the characters: & and <.
  * >, ', and " can be parsed in most contexts in XML.
+ * '>' following "]]" needs to be escaped, too.  That is
+ * handled as a special case.
+ *
  * The only issue is " vs ' in attribute values.  DB XML
  * uses the convention of attr="value" so it will escape
  * any double-quote (") in attribute values, on output.
  */
-#define CHECK_ENT(c) \
-if (hasEntity && ((c) < 'A')) {  \
-	if (((c) == '&') || ((c) == '<')) \
-		*hasEntity = true; \
-	else if ((c) == '"' && type == isAttrVal) \
-		*hasEntity = true; \
-}
+#define CHECK_ENT(c)							\
+	if ((c) == ']')							\
+		++nrb;							\
+	else if (c) {							\
+		if (hasEntity && ((c) < 'A')) {				\
+			if (((c) == '&') || ((c) == '<'))		\
+				*hasEntity = true;			\
+			else if ((c) == '"' && type == isAttrVal)	\
+				*hasEntity = true;			\
+			else if ((c) == '>' && nrb >= 2)		\
+				*hasEntity = true;			\
+		}							\
+		nrb = 0;						\
+	}								
 
 enum checkType {
 	ignore = 0,
@@ -53,6 +61,8 @@ enum checkType {
 	isAttrVal = 2
 };
 
+class XPath2MemoryManager;
+
 namespace DbXml
 {
 
@@ -61,13 +71,13 @@ class Buffer;
 class NsUtil {
 public:
 	// transcoding
-	static int nsToUTF8(XER_NS MemoryManager *mmgr, xmlbyte_t **dest,
-			    const xmlch_t *src, uint32_t nchars,
-			    uint32_t maxbytes,
-			    bool *hasEntity = 0, enum checkType type = ignore);
-	static int nsFromUTF8(XER_NS MemoryManager *mmgr, xmlch_t **dest,
-			      const xmlbyte_t *src,
-			      uint32_t nbytes, uint32_t maxchars);
+	static size_t nsToUTF8(xmlbyte_t **dest,
+			       const xmlch_t *src, size_t nchars,
+			       size_t maxbytes,
+			       bool *hasEntity = 0, enum checkType type = ignore);
+	static size_t nsFromUTF8(xmlch_t **dest,
+				 const xmlbyte_t *src,
+				 size_t nbytes, size_t maxchars);
 
 	/// Returns the number of bytes in the UTF-8 character
 	static int codepointToUTF8Size(uint32_t ch);
@@ -86,19 +96,22 @@ public:
 	static bool nsStringEqual(const xmlch_t *str1, const xmlch_t *str2);
 	static bool nsStringEqual(const xmlbyte_t *str1,
 				  const xmlbyte_t *str2);
-	static int nsStringLen(const xmlch_t *str);
-	static int nsStringLen(const unsigned char *str);
-	static xmlbyte_t *nsStringDup(XER_NS MemoryManager *mmgr,
-				      const xmlbyte_t *str, int *lenP);
-	static xmlch_t *nsStringDup(XER_NS MemoryManager *mmgr,
-				    const xmlch_t *str,
-				    int *lenP);
+	static size_t nsStringLen(const xmlch_t *str);
+	static size_t nsStringLen(const unsigned char *str);
+	static xmlbyte_t *nsStringDup(const xmlbyte_t *str, size_t *lenP);
+	static xmlch_t *nsStringDup(const xmlch_t *str,
+				    size_t *lenP);
+	static int stringEqualsIgnoreCase(const char *s1, const char *s2);
+	static int stringNEqualsIgnoreCase(const char *s1, const char *s2, 
+						size_t n);
+
 	// Escape < & " (latter only if isAttr)
-	static uint32_t nsEscape(char *dest, const xmlbyte_t *src, int len,
-				 bool isAttr = false);
+	static size_t nsEscape(char *dest, const xmlbyte_t *src, size_t len,
+			       bool isAttr = false);
 
 	// Whitespace munging
 	static bool isWhitespace(char c);
+	static bool isWhitespace(const char *c);
 	static void trimWhitespace(const char *&p, size_t &l);
 	/**
 	 * Performs in place trim of leading and trailing
@@ -112,15 +125,30 @@ public:
 	static void decodeBase64Binary(Buffer *buffer, const char *p, size_t l);
 	static void decodeHexBinary(Buffer *buffer, const char *p, size_t l);
 
+	// Memory mgmt
+	static void *allocate(size_t size, const char *loc = 0);
+	static void deallocate(void *obj);
+        class AutoDealloc {
+	public:
+		AutoDealloc(void*p) {p_ = p;}
+		~AutoDealloc() {
+			if (p_)
+				deallocate(p_);
+		}
+		void dealloc() {
+			if (p_) {
+				deallocate(p_);
+				p_ = 0;
+			}
+		}
+	private:
+		void *p_;
+	};
 
 	// error handling
-	// TBD: change code to XmlException::ExceptionCode
 	static void nsThrowException(XmlException::ExceptionCode,
 				     const char *desc,
 				     const char *file =0, int line = 0);
-	static void nsThrowDbException(const DbException &e,
-				       const char *file =0,
-				       int line = 0);
 	static void nsThrowParseException(const char *message);
 
 private:
@@ -141,31 +169,50 @@ private:
  *
  * len passed in does not include trailing NULL.  This must be added
  * for nsToUTF8.
- *
- * IMPORTANT: the same MemoryManager instance must be used for this
- * class as is used by the related classes (NsDocument, etc), as it
- * will be used for deallocation of the donated memory.
  */
 class NsDonator {
 public:
-	NsDonator(XER_NS MemoryManager *mmgr, const xmlch_t *src, uint32_t len,
+	NsDonator(const xmlch_t *src, size_t len,
 		  enum checkType type = ignore);
 	// multi-string constructor (creates src10src20)
-	NsDonator(XER_NS MemoryManager *mmgr, const xmlch_t *src1,
-		  const xmlch_t *src2, enum checkType type = ignore);
+	NsDonator(const xmlch_t *src1,
+		  const xmlch_t *src2,
+		  XPath2MemoryManager *mmgr = 0,
+		  enum checkType type = ignore);
 	xmlbyte_t *getStr() const { return _str; }
 	xmlbyte_t *getStr2() const { return _str2; }
-	uint32_t getLen() const { return _len; }
+	size_t getLen() const { return _len; }
 	bool getHasEntity() const { return _hasEntity; }
 private:
 	xmlbyte_t *_str;
 	xmlbyte_t *_str2;
-	uint32_t _len;
+	size_t _len;
 	bool _hasEntity;
 };
 
+		
+// utility class to cache UTF-16 strings
+class NsString {
+public:
+	NsString(const xmlch_t *text);
+	NsString() : text_(0), owned_(false) {}
+	~NsString() {
+		if (text_ && owned_)
+			::free((void*)text_);
+	}
+	void set(const xmlch_t *text);
+	void set(const xmlch_t *text, bool owned);
+	void clear();
+	const xmlch_t *get() const;
+	operator const XMLCh *() const { return text_; }
+	NsString &operator=(const XMLCh *text) { set(text); return *this; }
+private:
+	const xmlch_t *text_;
+	bool owned_;
+};
+
 // inlines
-inline int
+inline size_t
 NsUtil::nsStringLen(const xmlch_t *str)
 {
 	int ret = 0;
@@ -174,10 +221,10 @@ NsUtil::nsStringLen(const xmlch_t *str)
 	return ret;
 }
 
-inline int
+inline size_t
 NsUtil::nsStringLen(const unsigned char *str)
 {
-	return strlen((const char *)str);
+	return str == 0 ? 0 : strlen((const char *)str);
 }
 
 inline bool
@@ -209,6 +256,18 @@ inline bool NsUtil::isWhitespace(char c)
 	}
 	return false;
 }
+	
+inline bool NsUtil::isWhitespace(const char *c)
+{
+	if (c && *c) {
+		while (*c != '\0') {
+			if (!isWhitespace(*c++))
+				return false;
+		}
+		return true;
+	}
+	return false;
+}
 
 inline void NsUtil::trimWhitespace(const char *&p, size_t &l)
 {
@@ -329,5 +388,18 @@ inline int NsUtil::UTF8ToCodepoint(const xmlbyte_t *src, uint32_t &ch)
 	return size;
 }
 
+inline const xmlch_t *NsString::get() const
+{
+	return text_;
+}
+
+inline void NsString::clear()
+{
+	if (text_ && owned_)
+		::free((void*)text_);
+	text_ = 0;
+	owned_ = false;
+}
+
 }
 #endif
