@@ -1,15 +1,15 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: NsXercesTranscoder.cpp,v 1.39 2006/11/17 18:18:29 jsnelson Exp $
 //
 
 #include "NsXercesTranscoder.hpp"
 #include "NsDom.hpp"
 #include "EventWriter.hpp"
 #include "NsDocument.hpp"
+#include "NsDocumentDatabase.hpp"
 #include "NsConstants.hpp"
 #include "NsNode.hpp"
 #include "../UTF8.hpp"
@@ -17,20 +17,20 @@
 
 #define nsSetFlag(node, flag) (node)->nd_header.nh_flags |= (flag)
 
+using namespace std;
 using namespace DbXml;
 XERCES_CPP_NAMESPACE_USE
 
-NsXercesTranscoder::NsXercesTranscoder(Transaction *txn, Container *container,
-				       const DocID &docId, u_int32_t flags,
-				       XER_NS MemoryManager *mmgr)
-	: NsHandlerBase(mmgr, &_document, 0), _document(0, mmgr),
+NsXercesTranscoder::NsXercesTranscoder(Transaction *txn, DbWrapper *db,
+				       DictionaryDatabase *ddb,
+				       const DocID &docId, int cid,
+				       u_int32_t flags)
+	: NsHandlerBase(&_document, 0), _document(0),
 	  _needsStart(false)
 {
-	NsDocumentDatabase *ndb = container->getDocumentDB()->getNsDocumentDatabase();
-	// NOTE: it's ok for ndb to be null, which is the case for
+	// NOTE: it's ok for db to be null, which is the case for
 	// node indexes on wholedoc containers
-	_document.initDoc(txn, ndb, container->getDictionaryDB(), docId,
-			  flags);
+	_document.initDoc(txn, db, ddb, docId, cid, flags);
 }
 
 NsXercesTranscoder::~NsXercesTranscoder()
@@ -41,12 +41,13 @@ void
 NsXercesTranscoder::startDocument(const xmlch_t *sniffedEncoding)
 {
 	// create the document node
-	NsNode *node = NsNode::allocNode(_memManager, 0, NS_STANDALONE);
+	NsNode *node = NsNode::allocNode(0, NS_STANDALONE);
 	node->setFlag(NS_ISDOCUMENT);
 	_current = node;
+	node->acquire();
 	
-	nextId(node->getNid());
-	DBXML_ASSERT(node->getNid()->isDocRootNid());
+	nextId(node->getFullNid());
+	DBXML_ASSERT(node->getNid().isDocRootNid());
 
 	if (sniffedEncoding && *sniffedEncoding) {
 		_doc->setSniffedEncodingStr((const xmlbyte_t *)XMLChToUTF8Null(sniffedEncoding).str());
@@ -122,21 +123,26 @@ NsXercesTranscoder::startElement(const xmlch_t *localName,
 		doStart();
 
 	// create the node
-	NsNode *node = NsNode::allocNode(_memManager,
-					 attrCount, NS_STANDALONE);
-	nextId(node->getNid());
+	NsNode *node = NsNode::allocNode(attrCount, NS_STANDALONE);
+	nextId(node->getFullNid());
 	
-	NsDonator lname8(_memManager, localName,
+	NsDonator lname8(localName,
 			 NsUtil::nsStringLen(localName));
 
 	XMLChToUTF8Null uri8(uri);
 	XMLChToUTF8Null prefix8(prefix);
 
+	node->acquire();
 	// initialize the node plumbing
-	startElem(node, lname8.getStr(),
-		  uri8.str(), prefix8.str(), true, true);
+	try {
+		startElem(node, lname8.getStr(),
+			  uri8.str(), prefix8.str(), true);
+	} catch (...) {
+		node->release();
+		throw;
+	}
 
-	if (attrCount != 0) {	
+	if (attrCount != 0) {
 		for (unsigned int i = 0; i < attrCount; i++) {
 			node->addAttr(_doc, attrs->prefix(i),
 				      attrs->uri(i), attrs->localName(i),
@@ -145,20 +151,22 @@ NsXercesTranscoder::startElement(const xmlch_t *localName,
 		}
 	}
 
+	// endElem() must not delete node
+	if (isEmpty)
+		endElem();
+
 	// event writer
 	if (_ewriter) {
 		const xmlbyte_t *tname = (const xmlbyte_t *) node->getNameChars();
 		// constructor is ok with empty attr list.
 		NsEventNodeAttrList alist(node->getAttrList(), *_doc);
+		NsNodeIndexNodeInfo ninfo(node);
 		_ewriter->writeStartElementWithAttrs(
 			tname, (const unsigned char *)prefix8.str(),
 			(const unsigned char *)uri8.str(), attrCount,
 			(attrCount ? &alist : 0),
-			node, isEmpty);
+			&ninfo, isEmpty);
 	}
-	
-	if (isEmpty)
-		endElem();
 }
 
 void
@@ -166,25 +174,30 @@ NsXercesTranscoder::endElement(const xmlch_t *localName,
 			       const xmlch_t *prefix,
 			       const xmlch_t *uri)
 {
+	// After endElem(), _current has become _previous
+	NsNode *node = _current;
+
+	// common node work (must not delete node)
+	endElem();
+
+	// This callback is done last because structural stats counts the size of the
+	// node which is not accurate until endElem() has been called.
 	if (_ewriter) {
+		DBXML_ASSERT(node);
 		XMLChToUTF8Null uri8(uri);
 		XMLChToUTF8Null prefix8(prefix);
-		// After endElem(), _current has become _previous
-		NsNode *node = _current;
-		DBXML_ASSERT(node);
+		NsNodeIndexNodeInfo ninfo(node);
 		const xmlbyte_t *tname = (const xmlbyte_t *)node->getNameChars();
 		_ewriter->writeEndElementWithNode(
 			tname, (const unsigned char *)prefix8.str(),
-			(const unsigned char *)uri8.str(), node);
+			(const unsigned char *)uri8.str(), &ninfo);
 	}
 
-	// common node work
-	endElem();
 }
 
 void
 NsXercesTranscoder::doCharacters(const xmlch_t *characters,
-				 uint32_t len, bool isCDATA,
+				 size_t len, bool isCDATA,
 				 bool ignorable)
 {
 	if (_needsStart)
@@ -194,7 +207,7 @@ NsXercesTranscoder::doCharacters(const xmlch_t *characters,
 	if (!len)
 		len = NsUtil::nsStringLen(characters);
 	enum checkType ttype = (isCDATA || ignorable) ? ignore : isCharacters;
-	NsDonator chars(_memManager, characters, len, ttype);
+	NsDonator chars(characters, len, ttype);
 	uint32_t textType;
 	if (isCDATA)
 		textType = NS_CDATA;
@@ -222,13 +235,12 @@ NsXercesTranscoder::doCharacters(const xmlch_t *characters,
 					     chars.getLen(), needsEscape);
 	}
 	
-	addText(chars.getStr(), chars.getLen(), textType,
-		true, true);
+	addText(chars.getStr(), chars.getLen(), textType, true);
 }
 
 void
 NsXercesTranscoder::characters(const xmlch_t *characters,
-			       uint32_t len, bool isCDATA,
+			       size_t len, bool isCDATA,
 			       bool needsEscape)
 {
 	doCharacters(characters, len, isCDATA, false);
@@ -236,21 +248,21 @@ NsXercesTranscoder::characters(const xmlch_t *characters,
 
 void
 NsXercesTranscoder::ignorableWhitespace(const xmlch_t *characters,
-					uint32_t len, bool isCDATA)
+					size_t len, bool isCDATA)
 {
 	doCharacters(characters, len, isCDATA, true);
 }
 
 void
-NsXercesTranscoder::comment(const xmlch_t *comment, uint32_t len)
+NsXercesTranscoder::comment(const xmlch_t *comment, size_t len)
 {
 	if (_needsStart)
 		doStart();
 
 	if (!len)
 		len = NsUtil::nsStringLen(comment);
-	NsDonator chars(_memManager, comment, len);
-	addText(chars.getStr(), chars.getLen(), NS_COMMENT, true, true);
+	NsDonator chars(comment, len);
+	addText(chars.getStr(), chars.getLen(), NS_COMMENT, true);
 
 	// event writer (comment text will never be deleted by addText)
 	if (_ewriter)
@@ -265,8 +277,8 @@ NsXercesTranscoder::processingInstruction(const xmlch_t *target,
 	if (_needsStart)
 		doStart();
 
-	NsDonator chars(_memManager, target, data);
-	addPI(chars.getStr(), chars.getStr2(), true, true, chars.getLen());
+	NsDonator chars(target, data);
+	addPI(chars.getStr(), chars.getStr2(), true, chars.getLen());
 
 	// event writer (PI text will never be deleted by addText)
 	if (_ewriter)
@@ -283,8 +295,8 @@ NsXercesTranscoder::setTypeInfo(const xmlch_t *name,
 	// to NsHandlerBase() for this.
 	// event handler (PI text will never be deleted by addText)
 #if 0	
-	NsDonator lname(_memManager, name, NsUtil::nsStringLen(name));
-	NsDonator luri(_memManager, uri, (uri ? NsUtil::nsStringLen(uri) : 0));
+	NsDonator lname(name, NsUtil::nsStringLen(name));
+	NsDonator luri(uri, (uri ? NsUtil::nsStringLen(uri) : 0));
 
 	// TBD...
 	
@@ -295,33 +307,33 @@ NsXercesTranscoder::setTypeInfo(const xmlch_t *name,
 #endif
 
 void
-NsXercesTranscoder::docTypeDecl(const xmlch_t *data, uint32_t len)
+NsXercesTranscoder::docTypeDecl(const xmlch_t *data, size_t len)
 {
 	if (_needsStart)
 		doStart();
 
-	NsDonator chars(_memManager, data, len);
-	addText(chars.getStr(), chars.getLen(), NS_SUBSET, true, true);
+	NsDonator chars(data, len);
+	addText(chars.getStr(), chars.getLen(), NS_SUBSET, true);
 	if (_ewriter)
 		_ewriter->writeDTD(chars.getStr(), chars.getLen());
 }
 
 void
-NsXercesTranscoder::startEntity(const xmlch_t *name, uint32_t len)
+NsXercesTranscoder::startEntity(const xmlch_t *name, size_t len)
 {
 	if (_needsStart)
 		doStart();
-	NsDonator chars(_memManager, name, len);
-	addText(chars.getStr(), chars.getLen(), NS_ENTSTART, true, true);
+	NsDonator chars(name, len);
+	addText(chars.getStr(), chars.getLen(), NS_ENTSTART, true);
 	if (_ewriter)
 		_ewriter->writeStartEntity(chars.getStr(), true);
 }
 
 void
-NsXercesTranscoder::endEntity(const xmlch_t *name, uint32_t len)
+NsXercesTranscoder::endEntity(const xmlch_t *name, size_t len)
 {
-	NsDonator chars(_memManager, name, len);
-	addText(chars.getStr(), chars.getLen(), NS_ENTEND, true, true);
+	NsDonator chars(name, len);
+	addText(chars.getStr(), chars.getLen(), NS_ENTEND, true);
 	if (_ewriter)
 		_ewriter->writeEndEntity(chars.getStr());
 }
