@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: dbxml_dump.cpp,v 1.22 2006/10/30 17:45:59 bostic Exp $
 //
 #ifdef _MSC_VER
 #pragma warning( disable : 4996 ) // warning: deprecation of strdup() in VS8
@@ -15,6 +14,8 @@
 #include <iostream>
 #include <cerrno>
 #include "common_utils.h"
+#include <string.h>
+#include <cstdlib>
 
 #include <db.h>
 
@@ -124,8 +125,7 @@ main(int argc, char *const* argv)
 {
 	const char *progname = "dbxml_dump";
 	DB_ENV *dbenv;
-	DbEnv *cxx_env;
-  XmlManager *xmlDb;
+	XmlManager *xmlDb;
 	u_int32_t cache;
 	int ch, exitval, is_private, keyflag, nflag, ret, Rflag, rflag;
 	char *home, *passwd;
@@ -134,8 +134,7 @@ main(int argc, char *const* argv)
 		return (ret);
 
 	dbenv = NULL;
-	cxx_env = NULL;
-  xmlDb = NULL;
+	xmlDb = NULL;
 	exitval = nflag = rflag = Rflag = 0;
 	keyflag = 0;
 	cache = MEGABYTE;
@@ -145,7 +144,7 @@ main(int argc, char *const* argv)
 	while ((ch = getopt(argc, argv, "f:h:NP:rRV")) != EOF)
 		switch (ch) {
 		case 'f':
-			if (freopen(optarg, "w", stdout) == NULL) {
+			if (freopen(optarg, "wb", stdout) == NULL) {
 				fprintf(stderr, "%s: %s: reopen: %s\n",
 					progname, optarg, strerror(errno));
 				return (EXIT_FAILURE);
@@ -175,7 +174,7 @@ main(int argc, char *const* argv)
 			break;
 		case 'V':
 			printf("%s\n", DbXml::dbxml_version(NULL, NULL, NULL));
-			printf("%s\n", DbEnv::version(NULL, NULL, NULL));
+			printf("%s\n", db_version(NULL, NULL, NULL));
 			return (EXIT_SUCCESS);
 		case '?':
 		default:
@@ -224,8 +223,7 @@ main(int argc, char *const* argv)
 		goto err;
 	}
 
-	cxx_env = DbEnv::wrap_DB_ENV(dbenv);
-	xmlDb = new XmlManager(cxx_env);
+	xmlDb = new XmlManager(dbenv);
 	
 	if (rflag) {
 		try {
@@ -249,16 +247,13 @@ main(int argc, char *const* argv)
 		exitval = 1;
 	}
 done:
-	if (cxx_env != NULL) {
+	if (xmlDb)
 		delete xmlDb;
 		
-		if ((ret = cxx_env->close(0)) != 0) {
-			exitval = 1;
-			fprintf(stderr,
-				"%s: dbenv->close: %s\n", progname, db_strerror(ret));
-		}
-		
-		delete cxx_env;
+	if ((ret = dbenv->close(dbenv, 0)) != 0) {
+		exitval = 1;
+		fprintf(stderr,
+			"%s: dbenv->close: %s\n", progname, db_strerror(ret));
 	}
 	
 	return (exitval == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
