@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: dbxml_load.cpp,v 1.21 2006/10/30 17:46:00 bostic Exp $
 //
 #ifdef _MSC_VER
 #pragma warning( disable : 4996 ) // warning: deprecation of strdup() in VS8
@@ -18,6 +17,8 @@
 #include <db.h>
 #include "db_utils.h"
 #include "common_utils.h" // for signal handling
+#include <string.h>
+#include <cstdlib>
 
 #ifdef HAVE_GETOPT
 #include <unistd.h>
@@ -74,8 +75,7 @@ db_init(DB_ENV *dbenv, char *home, int *is_privatep)
 
 	*is_privatep = 0;
 	/* We may be loading into a live environment.  Try and join. */
-	flags = DB_USE_ENVIRON |
-		DB_INIT_LOCK | DB_INIT_LOG | DB_INIT_MPOOL | DB_INIT_TXN;
+	flags = DB_USE_ENVIRON;
 	if (dbenv->open(dbenv, home, flags, 0) == 0)
 		return (0);
 
@@ -91,8 +91,7 @@ db_init(DB_ENV *dbenv, char *home, int *is_privatep)
 	 * an mpool region exists).  Create one, but make it private so that
 	 * no files are actually created.
 	 */
-	LF_CLR(DB_INIT_LOCK | DB_INIT_LOG | DB_INIT_TXN);
-	LF_SET(DB_CREATE | DB_PRIVATE);
+	LF_SET(DB_CREATE | DB_PRIVATE | DB_INIT_MPOOL);
 	*is_privatep = 1;
 	if ((ret = dbenv->open(dbenv, home, flags, 0)) == 0)
 		return (0);
@@ -106,8 +105,7 @@ int main(int argc, char *argv[])
 {
 	const char *progname = "dbxml_load";
 	DB_ENV *dbenv;
-	DbEnv *cxx_env;
-  XmlManager *xmlDb;
+	XmlManager *xmlDb;
 	char *home, *passwd;
 	unsigned long lineno = 0;
 	int ch, exitval, is_private, ret;
@@ -116,8 +114,7 @@ int main(int argc, char *argv[])
 		return (ret);
 
 	dbenv = NULL;
-	cxx_env = NULL;
-  xmlDb = NULL;
+	xmlDb = NULL;
 	exitval = is_private = 0;
 	passwd = home = NULL;
 
@@ -144,7 +141,7 @@ int main(int argc, char *argv[])
 			break;
 		case 'V':
 			printf("%s\n", DbXml::dbxml_version(NULL, NULL, NULL));
-			printf("%s\n", DbEnv::version(NULL, NULL, NULL));
+			printf("%s\n", db_version(NULL, NULL, NULL));
 			return (EXIT_SUCCESS);
 		case '?':
 		default:
@@ -180,8 +177,7 @@ int main(int argc, char *argv[])
 		goto err;
 	}
 
-	cxx_env = DbEnv::wrap_DB_ENV(dbenv);
-	xmlDb = new XmlManager(cxx_env);
+	xmlDb = new XmlManager(dbenv);
 
 	try {
 		XmlUpdateContext uc = xmlDb->createUpdateContext();
@@ -195,16 +191,13 @@ int main(int argc, char *argv[])
 err:
 		exitval = 1;
 	}
-	if (cxx_env != NULL) {
+	if (xmlDb)
 		delete xmlDb;
-		if ((ret = cxx_env->close(0)) != 0) {
-			exitval = 1;
-			fprintf(stderr,
-				"%s: dbenv->close: %s\n", progname, db_strerror(ret));
-		}
-		delete cxx_env;
+	if ((ret = dbenv->close(dbenv, 0)) != 0) {
+		exitval = 1;
+		fprintf(stderr,
+			"%s: dbenv->close: %s\n", progname, db_strerror(ret));
 	}
-
 	return (exitval == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
