@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: XmlHelper.java,v 1.5 2006/10/30 17:46:01 bostic Exp $
 //
 
 // This class allows dipping into package-protected state
@@ -14,10 +13,23 @@ package com.sleepycat.db;
 import com.sleepycat.db.internal.DbEnv;
 import com.sleepycat.db.internal.DbTxn;
 import com.sleepycat.db.LockMode;
+import com.sleepycat.dbxml.XmlException;
 
 public class XmlHelper {
 
-    public static DbEnv getDbEnv(Environment env) {
+    public static DbEnv getDbEnv(Environment env) 
+    throws XmlException {
+	try{
+	    env.getHome();
+	}catch(DatabaseException e){
+	    throw (new XmlException(XmlException.DATABASE_ERROR, 
+		   "Invalid Environment object, may be closed.", 
+		    e, e.getErrno()));
+	}catch(Exception e){
+	    throw (new XmlException(XmlException.DATABASE_ERROR, 
+		   "Invalid Environment object, may be closed.", 
+		   new DatabaseException("Invalid environment."), 0));
+	}
 	return env.unwrap();
     }
     public static DbTxn getDbTxn(Transaction txn) {
