@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: XmlException.java,v 1.26.2.1 2006/12/08 16:32:28 gmf Exp $
 //
 // IMPORTANT: the enumerations here must match those in 
 // include/dbxml/XmlException.hpp
@@ -52,12 +51,6 @@ public class XmlException extends com.sleepycat.db.DatabaseException {
      *
      */
     public final static int DOCUMENT_NOT_FOUND = 11;
-    /**
-     *
-     *</ul>
-     *
-     */
-    public final static int DOM_PARSER_ERROR = 7;
     /**
      *
      *</ul>
@@ -94,12 +87,6 @@ public class XmlException extends com.sleepycat.db.DatabaseException {
      *
      */
     public final static int LAZY_EVALUATION = 10;
-    /**
-     *
-     *</ul>
-     *
-     */
-    public final static int NO_VARIABLE_BINDING = 9;
     /**
      *
      *</ul>
@@ -111,13 +98,13 @@ public class XmlException extends com.sleepycat.db.DatabaseException {
      *</ul>
      *
      */
-    public final static int XPATH_EVALUATION_ERROR = 8;
+    public final static int QUERY_EVALUATION_ERROR = 8;
     /**
      *
      *</ul>
      *
      */
-    public final static int XPATH_PARSER_ERROR = 6;
+    public final static int QUERY_PARSER_ERROR = 6;
     /**
      *
      *</ul>
@@ -168,10 +155,10 @@ public class XmlException extends com.sleepycat.db.DatabaseException {
             "NULL_POINTER",
             "INDEXER_PARSER_ERROR",
             "DATABASE_ERROR",
-            "XPATH_PARSER_ERROR",
-            "DOM_PARSER_ERROR",
-            "XPATH_EVALUATION_ERROR",
-            "NO_VARIABLE_BINDING",
+            "QUERY_PARSER_ERROR",
+            "UNUSED1",
+            "QUERY_EVALUATION_ERROR",
+            "UNUSED2",
             "LAZY_EVALUATION",
             "DOCUMENT_NOT_FOUND",
             "CONTAINER_EXISTS",
@@ -194,7 +181,7 @@ public class XmlException extends com.sleepycat.db.DatabaseException {
 
 
     public XmlException(int errcode, String description) {
-        super(null, 0);
+        super(description, 0);
         this.errcode = errcode;
 	this.dbexc = dbexc;
 	this.qLine = 0;
@@ -258,8 +245,8 @@ public class XmlException extends com.sleepycat.db.DatabaseException {
 
     /**
      * @return    The getQueryLine() method returns the line
-     *  of the XQuery expression that contains the error causing
-     *  the exception, in the case of XPATH_PARSER_ERROR.</p>
+     *  of the query expression that contains the error causing
+     *  the exception, in the case of QUERY_PARSER_ERROR.</p>
      */
     public int getQueryLine() {
         return qLine;
@@ -267,8 +254,8 @@ public class XmlException extends com.sleepycat.db.DatabaseException {
 
     /**
      * @return    The getQueryColumn() method returns the column
-     *  within the line of the XQuery expression that contains the error causing
-     *  the exception, in the case of XPATH_PARSER_ERROR.</p>
+     *  within the line of the query expression that contains the error causing
+     *  the exception, in the case of QUERY_PARSER_ERROR.</p>
      */
     public int getQueryColumn() {
         return qColumn;
