@@ -4,7 +4,7 @@ use 5.006001;
 use strict;
 use warnings;
 
-our $VERSION = '2.003';
+our $VERSION = '4.008024';
 
 our $strict_txn = 1 ;
 
@@ -217,7 +217,6 @@ __DATA__
 
 	DB_AFTER
 	DB_AGGRESSIVE
-	DB_ALREADY_ABORTED
 
 	DB_APPEND
 	DB_ARCH_ABS
@@ -225,9 +224,6 @@ __DATA__
 	DB_ARCH_LOG
 	DB_AUTO_COMMIT
 	DB_BEFORE
-	DB_BTREEMAGIC
-	DB_BTREEOLDVER
-	DB_BTREEVERSION
 #	DB_CACHED_COUNTS
 	DB_CDB_ALLDB
 	DB_CHKSUM		DEFINE
@@ -240,10 +236,8 @@ __DATA__
 
 	DB_DBT_PARTIAL
 
-	DB_DELETED
 	DB_DIRECT
 	DB_DIRECT_DB
-	DB_DIRECT_LOG
 	DB_DIRTY_READ
 	DB_DONOTINDEX
 	DB_DUP
@@ -252,45 +246,15 @@ __DATA__
 	DB_EID_INVALID
 	DB_ENCRYPT
 	DB_ENCRYPT_AES
-	DB_ENV_AUTO_COMMIT
-	DB_ENV_CDB
-	DB_ENV_CDB_ALLDB
-	DB_ENV_CREATE
-	DB_ENV_DBLOCAL
-	DB_ENV_DIRECT_DB
-	DB_ENV_DIRECT_LOG
-	DB_ENV_FATAL
-	DB_ENV_LOCKDOWN
-	DB_ENV_NOLOCKING
-	DB_ENV_NOMMAP
-	DB_ENV_NOPANIC
-	DB_ENV_OPEN_CALLED
-	DB_ENV_OVERWRITE
-	DB_ENV_PRIVATE
-	DB_ENV_REGION_INIT
-	DB_ENV_RPCCLIENT
-	DB_ENV_RPCCLIENT_GIVEN
-	DB_ENV_SYSTEM_MEM
-	DB_ENV_THREAD
-	DB_ENV_TXN_NOSYNC
-	DB_ENV_TXN_WRITE_NOSYNC
-	DB_ENV_YIELDCPU
 	DB_EXCL
-	DB_EXTENT
 	DB_FAST_STAT
-	DB_FCNTL_LOCKING
 	DB_FILE_ID_LEN
 	DB_FIRST
 	DB_FLUSH
 	DB_FORCE
 	DB_GET_BOTH
-	DB_GET_BOTHC
 	DB_GET_BOTH_RANGE
 	DB_GET_RECNO
-	DB_HANDLE_LOCK
-	DB_HASHMAGIC
-	DB_HASHOLDVER
-	DB_HASHVERSION
 	DB_INIT_CDB
 	DB_INIT_LOCK
 	DB_INIT_LOG
@@ -305,44 +269,25 @@ __DATA__
 	DB_KEYLAST
 	DB_LAST
 	DB_LOCKDOWN
-	DB_LOCKVERSION
 	DB_LOCK_DEADLOCK
 	DB_LOCK_DEFAULT
 	DB_LOCK_EXPIRE
 	DB_LOCK_MAXLOCKS
 	DB_LOCK_MINLOCKS
 	DB_LOCK_MINWRITE
-	DB_LOCK_NORUN
-	DB_LOCK_NOTEXIST        DEFINE
 	DB_LOCK_NOTGRANTED
 	DB_LOCK_NOWAIT
 	DB_LOCK_OLDEST
 	DB_LOCK_RANDOM
-	DB_LOCK_RECORD
-	DB_LOCK_REMOVE      DEFINE
-	DB_LOCK_SET_TIMEOUT
-	DB_LOCK_SWITCH
-	DB_LOCK_UPGRADE
 	DB_LOCK_YOUNGEST
 #	DB_LOGC_BUF_SIZE
-	DB_LOGFILEID_INVALID
-	DB_LOGMAGIC
-	DB_LOGOLDVER
-	DB_LOGVERSION
-	DB_LOG_DISK
-	DB_LOG_LOCKED
-	DB_LOG_SILENT_ERR
-	DB_MAX_PAGES
-	DB_MAX_RECORDS
 #	DB_MPOOL_CLEAN
 	DB_MPOOL_CREATE
 	DB_MPOOL_DIRTY
-	DB_MPOOL_DISCARD
 	DB_MPOOL_LAST
 	DB_MPOOL_NEW
 	DB_MULTIPLE
 	DB_MULTIPLE_KEY
-	DB_NEEDSPLIT
 	DB_NEXT
 	DB_NEXT_DUP
 	DB_NEXT_NODUP
@@ -358,15 +303,9 @@ __DATA__
 	DB_NOSYNC
 	DB_NOTFOUND
 	DB_ODDFILESIZE
-	DB_OK_BTREE
-	DB_OK_HASH
-	DB_OK_QUEUE
-	DB_OK_RECNO
 	DB_OLD_VERSION
-	DB_OPFLAGS_MASK
 	DB_ORDERCHKONLY
 	DB_OVERWRITE
-	DB_PAGE_LOCK
 	DB_PAGE_NOTFOUND
 	DB_PANIC_ENVIRONMENT
 	DB_POSITION
@@ -374,21 +313,12 @@ __DATA__
 	DB_PREV_NODUP
 	DB_PRINTABLE
 	DB_PRIVATE
-	DB_PR_PAGE
-	DB_PR_RECOVERYTEST
-	DB_QAMMAGIC
-	DB_QAMOLDVER
-	DB_QAMVERSION
 	DB_RDONLY
-	DB_RDWRMASTER
 	DB_RECNUM
 #	DB_RECORDCOUNT
-	DB_RECORD_LOCK
 	DB_RECOVER
 	DB_RECOVER_FATAL
 	DB_REGION_INIT
-	DB_REGION_MAGIC
-	DB_RENAMEMAGIC
 	DB_RENUMBER
 	DB_REP_CLIENT
 	DB_REP_DUPMASTER
@@ -411,32 +341,16 @@ __DATA__
 	DB_SET_LOCK_TIMEOUT
 	DB_SET_RANGE
 	DB_SET_RECNO
-	DB_SET_TXN_NOW
 	DB_SET_TXN_TIMEOUT
 	DB_SNAPSHOT
 	DB_STAT_CLEAR
-	DB_SURPRISE_KID
-	DB_SWAPBYTES
 	DB_SYSTEM_MEM
-	DB_TEST_ELECTINIT
-	DB_TEST_POSTDESTROY
-	DB_TEST_POSTLOG
-	DB_TEST_POSTLOGMETA
-	DB_TEST_POSTOPEN
-	DB_TEST_POSTSYNC
-	DB_TEST_PREDESTROY
-	DB_TEST_PREOPEN
-	DB_TEST_SUBDB_LOCKS
 	DB_THREAD
-	DB_TIMEOUT
 	DB_TRUNCATE
-	DB_TXNVERSION
-	DB_TXN_CKP
 	DB_TXN_NOSYNC
 	DB_TXN_NOWAIT
 	DB_TXN_SYNC
 	DB_TXN_WRITE_NOSYNC
-	DB_UPDATE_SECONDARY
 	DB_UPGRADE
 	DB_USE_ENVIRON
 	DB_USE_ENVIRON_ROOT
@@ -447,56 +361,35 @@ __DATA__
 	DB_VERB_WAITSFOR
 	DB_VERIFY
 	DB_VERIFY_BAD
-	DB_VERIFY_FATAL
 	DB_VERSION_MAJOR
 	DB_VERSION_MINOR
 	DB_VERSION_PATCH
 	DB_VERSION_STRING          STRING
 	DB_WRITECURSOR
-	DB_WRITELOCK
-	DB_WRITEOPEN
-	DB_XA_CREATE
-	DB_XIDDATASIZE
+	#DB_XA_CREATE removed in 4.8
+	#DB_XIDDATASIZE removed in 4.8
 	DB_YIELDCPU
-	MP_FLUSH
-	MP_OPEN_CALLED
-	MP_READONLY
-	TXN_CHILDCOMMIT
-	TXN_COMPENSATE
-	TXN_DIRTY_READ      DEFINE
-	TXN_LOCKTIMEOUT
-	TXN_MALLOC
 	TXN_NOSYNC
 	TXN_NOWAIT
 	TXN_SYNC
 
         # new in 4.4
-        DB_ASSOC_IMMUTABLE_KEY     DEFINE
-        DB_COMPACT_FLAGS           DEFINE
         DB_FREELIST_ONLY           DEFINE
         DB_FREE_SPACE              DEFINE
         DB_IMMUTABLE_KEY           DEFINE
-        DB_MUTEX_ALLOCATED         DEFINE
-        DB_MUTEX_LOGICAL_LOCK      DEFINE
         DB_MUTEX_SELF_BLOCK        DEFINE
-        DB_MUTEX_THREAD            DEFINE
         DB_READ_COMMITTED          DEFINE
         DB_READ_UNCOMMITTED        DEFINE
         DB_REGISTER                DEFINE
         DB_REP_ANYWHERE            DEFINE
-        DB_REP_BULKOVF             DEFINE
         DB_REP_IGNORE              DEFINE
         DB_REP_LOCKOUT             DEFINE
         DB_REP_REREQUEST           DEFINE
-        DB_SEQ_WRAPPED             DEFINE
         DB_VERB_REGISTER           DEFINE
-        REP_CONF_AUTOINIT          DEFINE
+        REP_CONF_NOAUTOINIT        DEFINE
         REP_CONF_BULK              DEFINE
         REP_CONF_DELAYCLIENT       DEFINE
         REP_CONF_NOWAIT            DEFINE
-        TXN_READ_COMMITTED         DEFINE
-        TXN_READ_UNCOMMITTED       DEFINE
-        TXN_SYNC_FLAGS             DEFINE
         TXN_WRITE_NOSYNC           DEFINE
     
 
@@ -525,16 +418,6 @@ __DATA__
 	DB_LOCK_TRADE
 	DB_LOCK_UPGRADE_WRITE
 
-	# enum db_status_t
-	DB_LSTAT_ABORTED
-    #DB_LSTAT_ERR
-	DB_LSTAT_EXPIRED
-	DB_LSTAT_FREE
-	DB_LSTAT_HELD
-    #DB_LSTAT_NOTEXIST       
-	DB_LSTAT_PENDING
-	DB_LSTAT_WAITING
-
 	# enum DB_CACHE_PRIORITY
 	DB_PRIORITY_VERY_LOW
 	DB_PRIORITY_LOW
@@ -545,12 +428,9 @@ __DATA__
 	# enum db_recops
 	DB_TXN_ABORT
 	DB_TXN_APPLY
-	DB_TXN_BACKWARD_ALLOC
 	DB_TXN_BACKWARD_ROLL
 	DB_TXN_FORWARD_ROLL
     #DB_TXN_GETPGNOS
-	DB_TXN_OPENFILES
-	DB_TXN_POPENFILES
 	DB_TXN_PRINT
 
 	# enum DBTYPE
