@@ -3,7 +3,7 @@ package Sleepycat::DbXml;
 use strict;
 use warnings;
 
-our $VERSION = '2.003';
+our $VERSION = '2.0050013';
 
 our $strict_txn = 1 ;
 
@@ -76,8 +76,8 @@ XSLoader::load('Sleepycat::DbXml', $VERSION);
 *XmlException::AUTOLOAD = \&DbXml::AUTOLOAD;
 *XmlIndexSpecification::AUTOLOAD = \&DbXml::AUTOLOAD;
 *XmlQueryContext::AUTOLOAD = \&DbXml::AUTOLOAD;
+*XmlContainerConfig::AUTOLOAD = \&DbXml::AUTOLOAD;
 *XmlValue::AUTOLOAD = \&DbXml::AUTOLOAD;
-*XmlModify::AUTOLOAD = \&DbXml::AUTOLOAD;
 *XmlIndexLookup::AUTOLOAD = \&DbXml::AUTOLOAD;
 *XmlEventType::AUTOLOAD = \&DbXml::AUTOLOAD;
 
@@ -92,6 +92,7 @@ sub DbXml::DBXML_CHKSUM ;
 sub DbXml::DBXML_ENCRYPT ; 
 sub DbXml::DBXML_GEN_NAME ; 
 sub DbXml::DBXML_LAZY_DOCS ; 
+sub DbXml::DBXML_DOCUMENT_PROJECTION ; 
 sub DbXml::DBXML_WELL_FORMED_ONLY ; 
 sub DbXml::DBXML_INDEX_NODES ; 
 sub DbXml::DBXML_NO_INDEX_NODES ; 
@@ -113,30 +114,20 @@ sub DbXml::CATEGORY_ALL ;
 sub DbXml::DBXML_VERSION_MAJOR ; 
 sub DbXml::DBXML_VERSION_MINOR ; 
 sub DbXml::DBXML_VERSION_PATCH ; 
-#sub DbXml::StartElement;
-#sub DbXml::EndElement;
-#sub DbXml::Characters;
-#sub DbXml::CDATA;
-#sub DbXml::Comment;
-#sub DbXml::Whitespace;
-#sub DbXml::StartDocument;
-#sub DbXml::EndDocument;
-#sub DbXml::StartEntityReference;
-#sub DbXml::EndEntityReference;
-#sub DbXml::ProcessingInstruction;
-#sub DbXml::DTD;
 sub XmlContainer::WholedocContainer ; 
 sub XmlContainer::NodeContainer ; 
+sub XmlContainerConfig::On ; 
+sub XmlContainerConfig::Off ; 
+sub XmlContainerConfig::UseDefault ;
 sub XmlException::INTERNAL_ERROR ; 
 sub XmlException::CONTAINER_OPEN ; 
 sub XmlException::CONTAINER_CLOSED ; 
 sub XmlException::NULL_POINTER ; 
 sub XmlException::INDEXER_PARSER_ERROR ; 
 sub XmlException::DATABASE_ERROR ; 
-sub XmlException::XPATH_PARSER_ERROR ; 
+sub XmlException::QUERY_PARSER_ERROR ; 
 sub XmlException::DOM_PARSER_ERROR ; 
-sub XmlException::XPATH_EVALUATION_ERROR ; 
-sub XmlException::NO_VARIABLE_BINDING ; 
+sub XmlException::QUERY_EVALUATION_ERROR ; 
 sub XmlException::LAZY_EVALUATION ; 
 sub XmlException::DOCUMENT_NOT_FOUND ; 
 sub XmlException::CONTAINER_EXISTS ; 
@@ -163,7 +154,6 @@ sub XmlIndexSpecification::KEY_NONE ;
 sub XmlIndexSpecification::KEY_PRESENCE ; 
 sub XmlIndexSpecification::KEY_EQUALITY ; 
 sub XmlIndexSpecification::KEY_SUBSTRING ; 
-sub XmlQueryContext::DeadValues ; 
 sub XmlQueryContext::LiveValues ; 
 sub XmlQueryContext::Eager ; 
 sub XmlQueryContext::Lazy ; 
@@ -205,11 +195,6 @@ sub XmlValue::DOCUMENT_NODE ;
 sub XmlValue::DOCUMENT_TYPE_NODE ; 
 sub XmlValue::DOCUMENT_FRAGMENT_NODE ; 
 sub XmlValue::NOTATION_NODE ; 
-sub XmlModify::Element ; 
-sub XmlModify::Attribute ; 
-sub XmlModify::Text ; 
-sub XmlModify::ProcessingInstruction ; 
-sub XmlModify::Comment ; 
 sub XmlIndexLookup::NONE ; 
 sub XmlIndexLookup::EQ ; 
 sub XmlIndexLookup::GT ; 
@@ -274,7 +259,7 @@ package DbXml;
 	    [ 'XmlManager', 'openContainer',	1, 3],
 	    [ 'XmlManager', 'removeContainer',	1, 2],
 	    [ 'XmlManager', 'renameContainer',	2, 3],
-	    [ 'XmlManager', 'reindexContainer',	2, 4],
+	    [ 'XmlManager', 'reindexContainer',	1, 4],
 	    [ 'XmlManager', 'prepare',	1, 3],
 	    [ 'XmlManager', 'query',	1, 4],
 
@@ -579,34 +564,6 @@ sub dump
 	
 }
 
-package XmlModify;
-
-use Carp;
-
-sub execute
-{
-    my $THIS = shift ;
-
-    unless ($Sleepycat::DbXml::strict_txn ||
-        (! defined $_[0] || ref $_[0] && UNIVERSAL::isa($_[0], "XmlTransaction")))
-        {
-           unshift @_, undef ;
-        
-        }
-
-    croak "execute needs beween 3 and 4 parameters\n" unless @_ >= 3 && @_ <= 4 ;
-
-    if ( ref $_[1] && UNIVERSAL::isa($_[1], "XmlResults") &&
-        ref $_[2] && UNIVERSAL::isa($_[2], "XmlQueryContext")  &&
-        ref $_[3] && UNIVERSAL::isa($_[3], "XmlUpdateContext") ) 
-    {
-        $THIS->_execute2(@_) ;	    
-    }	   
-    else 
-    {
-        $THIS->_execute1(@_) ;	    
-    }
-}
 package DbtPtr;
 
 use Carp;
