@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2004,2006 Oracle.  All rights reserved.
+ * Copyright (c) 2004,2009 Oracle.  All rights reserved.
  *
  * http://www.apache.org/licenses/LICENSE-2.0.txt
  * 
@@ -11,6 +11,7 @@
 
 PHP_DBXML_DECLARE_CLASS(XmlManager);
 PHP_DBXML_DECLARE_CLASS(XmlContainer);
+PHP_DBXML_DECLARE_CLASS(XmlContainerConfig);
 PHP_DBXML_DECLARE_CLASS(XmlData);
 PHP_DBXML_DECLARE_CLASS(XmlDocument);
 PHP_DBXML_DECLARE_CLASS(XmlEventReader);
@@ -18,7 +19,6 @@ PHP_DBXML_DECLARE_CLASS(XmlEventReaderToWriter);
 PHP_DBXML_DECLARE_CLASS(XmlEventWriter);
 PHP_DBXML_DECLARE_CLASS(XmlIndexLookup);
 PHP_DBXML_DECLARE_CLASS(XmlIndexSpecification);
-PHP_DBXML_DECLARE_CLASS(XmlModify);
 PHP_DBXML_DECLARE_CLASS(XmlQueryContext);
 PHP_DBXML_DECLARE_CLASS(XmlQueryExpression);
 PHP_DBXML_DECLARE_CLASS(XmlResults);
@@ -50,7 +50,7 @@ static PHP_FUNCTION(dbxml_version)
     return;
   }
 
-  RETVAL_STRING((char*)dbxml_version(&M, &m, &p), 1);
+  RETVAL_STRING((char*)dbxml_version(&M, &m, &p), (int)1);
 
   if (ZEND_NUM_ARGS()) {
     convert_to_long(maj);
@@ -70,7 +70,7 @@ static PHP_FUNCTION(dbxml_set_log_level)
     return;
   }
 
-  DbXml::setLogLevel((DbXml::LogLevel)level, on);
+  DbXml::setLogLevel((DbXml::LogLevel)level, (on != 0));
 }
 
 static PHP_FUNCTION(dbxml_set_log_category)
@@ -81,7 +81,7 @@ static PHP_FUNCTION(dbxml_set_log_category)
     return;
   }
 
-  DbXml::setLogCategory((DbXml::LogCategory)level, on);
+  DbXml::setLogCategory((DbXml::LogCategory)level, (on != 0));
 }
 
 function_entry dbxml_functions[] = {
@@ -96,11 +96,11 @@ PHP_MINIT_FUNCTION(dbxml)
 {
   zend_class_entry ce;
 #ifdef PHP_DBXML_EXCEPTIONS
-  php_dbxml_init_exceptions();
+  php_dbxml_init_exceptions(TSRMLS_C);
 #endif
   PHP_DBXML_REGISTER_CLASS(XmlData, php_dbxml_XmlData_dtor);
   PHP_DBXML_REGISTER_CLASS(XmlValue, php_dbxml_XmlValue_dtor);
-  PHP_DBXML_REGISTER_CLASS(XmlContainer, php_dbxml_XmlContainer_dtor);
+  PHP_DBXML_REGISTER_CLASS(XmlContainer, php_dbxml_XmlContainer_dtor); 
 #if 0
   PHP_DBXML_REGISTER_CLASS(XmlResolver, NULL);
 #endif
@@ -108,8 +108,8 @@ PHP_MINIT_FUNCTION(dbxml)
   PHP_DBXML_REGISTER_CLASS(XmlEventReader, php_dbxml_XmlEventReader_dtor);
   PHP_DBXML_REGISTER_CLASS(XmlEventReaderToWriter, php_dbxml_XmlEventReaderToWriter_dtor);
   PHP_DBXML_REGISTER_CLASS(XmlEventWriter, php_dbxml_XmlEventWriter_dtor);
+  PHP_DBXML_REGISTER_CLASS(XmlIndexLookup, php_dbxml_XmlIndexLookup_dtor);
   PHP_DBXML_REGISTER_CLASS(XmlIndexSpecification, php_dbxml_XmlIndexSpecification_dtor);
-  PHP_DBXML_REGISTER_CLASS(XmlModify, php_dbxml_XmlModify_dtor);
   PHP_DBXML_REGISTER_CLASS(XmlQueryContext, php_dbxml_XmlQueryContext_dtor);
   PHP_DBXML_REGISTER_CLASS(XmlQueryExpression, php_dbxml_XmlQueryExpression_dtor);
   PHP_DBXML_REGISTER_CLASS(XmlResults, php_dbxml_XmlResults_dtor);
@@ -118,6 +118,7 @@ PHP_MINIT_FUNCTION(dbxml)
   PHP_DBXML_REGISTER_CLASS(XmlInputStream, php_dbxml_XmlInputStream_dtor);
   PHP_DBXML_REGISTER_CLASS(XmlStatistics, php_dbxml_XmlStatistics_dtor);
   PHP_DBXML_REGISTER_CLASS(XmlManager, php_dbxml_XmlManager_dtor);
+  PHP_DBXML_REGISTER_CLASS(XmlContainerConfig, php_dbxml_XmlContainerConfig_dtor);
   REGISTER_LONG_CONSTANT( "XmlEventReader_StartElement", XmlEventReader::StartElement, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "XmlEventReader_EndElement", XmlEventReader::EndElement, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "XmlEventReader_Characters", XmlEventReader::Characters, CONST_CS | CONST_PERSISTENT);
@@ -141,9 +142,14 @@ PHP_MINIT_FUNCTION(dbxml)
   REGISTER_LONG_CONSTANT( "DBXML_ENCRYPT", DBXML_ENCRYPT, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "DBXML_GEN_NAME", DBXML_GEN_NAME, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "DBXML_LAZY_DOCS", DBXML_LAZY_DOCS, CONST_CS | CONST_PERSISTENT);
+  REGISTER_LONG_CONSTANT( "DBXML_DOCUMENT_PROJECTION", DBXML_DOCUMENT_PROJECTION, CONST_CS | CONST_PERSISTENT);
+  REGISTER_LONG_CONSTANT( "DBXML_NO_AUTO_COMMIT", DBXML_NO_AUTO_COMMIT, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "DBXML_CACHE_DOCUMENTS", DBXML_CACHE_DOCUMENTS, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "DBXML_NO_INDEX_NODES", DBXML_NO_INDEX_NODES, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "DBXML_INDEX_NODES", DBXML_INDEX_NODES, CONST_CS | CONST_PERSISTENT);
+  REGISTER_LONG_CONSTANT( "DBXML_NO_STATISTICS", DBXML_NO_STATISTICS, CONST_CS | CONST_PERSISTENT);
+  REGISTER_LONG_CONSTANT( "DBXML_STATISTICS", DBXML_STATISTICS, CONST_CS | CONST_PERSISTENT);
+  REGISTER_LONG_CONSTANT( "DBXML_WELL_FORMED_ONLY", DBXML_WELL_FORMED_ONLY, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "LEVEL_NONE", LEVEL_NONE, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "LEVEL_DEBUG", LEVEL_DEBUG, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "LEVEL_INFO", LEVEL_INFO, CONST_CS | CONST_PERSISTENT);
@@ -165,7 +171,9 @@ PHP_MINIT_FUNCTION(dbxml)
   REGISTER_STRING_CONSTANT("metaDataName_root", "root", CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "XmlContainer_WholedocContainer", XmlContainer::WholedocContainer, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "XmlContainer_NodeContainer", XmlContainer::NodeContainer, CONST_CS | CONST_PERSISTENT);
-  REGISTER_LONG_CONSTANT( "XmlQueryContext_DeadValues", XmlQueryContext::DeadValues, CONST_CS | CONST_PERSISTENT);
+  REGISTER_LONG_CONSTANT( "XmlContainerConfig_On", XmlContainerConfig::On, CONST_CS | CONST_PERSISTENT);
+  REGISTER_LONG_CONSTANT( "XmlContainerConfig_Off", XmlContainerConfig::Off, CONST_CS | CONST_PERSISTENT);
+  REGISTER_LONG_CONSTANT( "XmlContainerConfig_UseDefault", XmlContainerConfig::UseDefault, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "XmlQueryContext_LiveValues", XmlQueryContext::LiveValues, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "XmlQueryContext_Eager", XmlQueryContext::Eager, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "XmlQueryContext_Lazy", XmlQueryContext::Lazy, CONST_CS | CONST_PERSISTENT);
@@ -219,11 +227,6 @@ PHP_MINIT_FUNCTION(dbxml)
   REGISTER_LONG_CONSTANT( "XmlIndexSpecification_KEY_PRESENCE", XmlIndexSpecification::KEY_PRESENCE, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "XmlIndexSpecification_KEY_EQUALITY", XmlIndexSpecification::KEY_EQUALITY, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT( "XmlIndexSpecification_KEY_SUBSTRING", XmlIndexSpecification::KEY_SUBSTRING, CONST_CS | CONST_PERSISTENT);
-  REGISTER_LONG_CONSTANT( "XmlModify_Element", XmlModify::Element, CONST_CS | CONST_PERSISTENT);
-  REGISTER_LONG_CONSTANT( "XmlModify_Attribute", XmlModify::Attribute, CONST_CS | CONST_PERSISTENT);
-  REGISTER_LONG_CONSTANT( "XmlModify_Text", XmlModify::Text, CONST_CS | CONST_PERSISTENT);
-  REGISTER_LONG_CONSTANT( "XmlModify_ProcessingInstruction", XmlModify::ProcessingInstruction, CONST_CS | CONST_PERSISTENT);
-  REGISTER_LONG_CONSTANT( "XmlModify_Comment", XmlModify::Comment, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT("XmlIndexLookup_NONE", XmlIndexLookup::NONE, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT("XmlIndexLookup_EQ", XmlIndexLookup::EQ, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT("XmlIndexLookup_GT", XmlIndexLookup::GT, CONST_CS | CONST_PERSISTENT);
@@ -237,10 +240,8 @@ PHP_MINIT_FUNCTION(dbxml)
   REGISTER_LONG_CONSTANT("XmlException_NULL_POINTER", XmlException::NULL_POINTER, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT("XmlException_INDEXER_PARSER_ERROR", XmlException::INDEXER_PARSER_ERROR, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT("XmlException_DATABASE_ERROR", XmlException::DATABASE_ERROR, CONST_CS | CONST_PERSISTENT);
-  REGISTER_LONG_CONSTANT("XmlException_XPATH_PARSER_ERROR", XmlException::XPATH_PARSER_ERROR, CONST_CS | CONST_PERSISTENT);
-  REGISTER_LONG_CONSTANT("XmlException_DOM_PARSER_ERROR", XmlException::DOM_PARSER_ERROR, CONST_CS | CONST_PERSISTENT);
-  REGISTER_LONG_CONSTANT("XmlException_XPATH_EVALUATION_ERROR", XmlException::XPATH_EVALUATION_ERROR, CONST_CS | CONST_PERSISTENT);
-  REGISTER_LONG_CONSTANT("XmlException_NO_VARIABLE_BINDING", XmlException::NO_VARIABLE_BINDING, CONST_CS | CONST_PERSISTENT);
+  REGISTER_LONG_CONSTANT("XmlException_QUERY_PARSER_ERROR", XmlException::QUERY_PARSER_ERROR, CONST_CS | CONST_PERSISTENT);
+  REGISTER_LONG_CONSTANT("XmlException_QUERY_EVALUATION_ERROR", XmlException::QUERY_EVALUATION_ERROR, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT("XmlException_LAZY_EVALUATION", XmlException::LAZY_EVALUATION, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT("XmlException_DOCUMENT_NOT_FOUND", XmlException::DOCUMENT_NOT_FOUND, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT("XmlException_CONTAINER_EXISTS", XmlException::CONTAINER_EXISTS, CONST_CS | CONST_PERSISTENT);
@@ -302,6 +303,9 @@ zend_module_entry dbxml_module_entry = {
 };
 
 #if COMPILE_DL_DBXML
+#ifdef PHP_WIN32
+#include "zend_arg_defs.c"
+#endif
 BEGIN_EXTERN_C()
 ZEND_GET_MODULE(dbxml)
 END_EXTERN_C()
