@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2004,2006 Oracle.  All rights reserved.
+ * Copyright (c) 2004,2009 Oracle.  All rights reserved.
  *
  * http://www.apache.org/licenses/LICENSE-2.0.txt
  * 
@@ -31,7 +31,7 @@ PHP_DBXML_METHOD_BEGIN(XmlQueryExpression, getQuery)
     WRONG_PARAM_COUNT;
   }
   std::string str = This.getQuery();
-  RETURN_STRINGL((char*)str.data(), str.length(), 1);
+  DBXML_RETURN_STRINGL((char*)str.data(), str.length(), 1);
 } PHP_DBXML_METHOD_END()
 /* }}} */
 
@@ -44,10 +44,22 @@ PHP_DBXML_METHOD_BEGIN(XmlQueryExpression, getQueryPlan)
     WRONG_PARAM_COUNT;
   }
   std::string str = This.getQueryPlan();
-  RETURN_STRINGL((char*)str.data(), str.length(), 1);
+  DBXML_RETURN_STRINGL((char*)str.data(), str.length(), 1);
 } PHP_DBXML_METHOD_END()
 /* }}} */
 
+/* {{{ proto bool XmlQueryExpression::isUpdateExpression()
+   returns true if query is update */
+PHP_DBXML_METHOD_BEGIN(XmlQueryExpression, isUpdateExpression) 
+{
+  PHP_DBXML_STUFFED_THIS(XmlQueryExpression);
+  if (ZEND_NUM_ARGS()) {
+    WRONG_PARAM_COUNT;
+  }
+  RETURN_BOOL(This.isUpdateExpression());
+} PHP_DBXML_METHOD_END()
+ /* }}} */
+
 /* {{{ proto XmlResults XmlQueryExpression::execute()
    executes the query */
 PHP_DBXML_METHOD_BEGIN(XmlQueryExpression, execute)
@@ -98,6 +110,7 @@ function_entry php_dbxml_XmlQueryExpression_methods[] = {
   PHP_ME(XmlQueryExpression, XmlQueryExpression, NULL, 0)
   PHP_ME(XmlQueryExpression, getQuery, NULL, 0)
   PHP_ME(XmlQueryExpression, getQueryPlan, NULL, 0)
+  PHP_ME(XmlQueryExpression, isUpdateExpression, NULL, 0)
   PHP_ME(XmlQueryExpression, execute, NULL, 0)
   {NULL,NULL,NULL}
 };
