@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: LoadContainer.hpp,v 1.12 2006/10/30 17:46:05 bostic Exp $
 //
 
 #ifndef LOADCONTAINER_HPP
@@ -29,15 +28,14 @@ public:
 	~LoadContainer() {}
 	
 	int addFiles(const std::vector<std::string > &fileList, 
-		     const std::string &pathPrepend) const;
+		const std::string &pathPrepend, bool failOnError) const;
 	void addFile(const std::string &fileName) const;
 	int addFileList(const std::string &pathPrepend,
-			const std::string &fileName) const;
+		const std::string &fileName, bool failOnError) const;
 
 	int count() const { return _count; }
 private:
  
-	std::string readFile(const std::string &theFile) const;
 	std::string getNameFromPath(const std::string &theFile) const;
 
 	DbXml::XmlManager &_mgr;
