@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: GetDocumentsCommand.cpp,v 1.7 2006/10/30 17:46:07 bostic Exp $
 //
 
 #include "GetDocumentsCommand.hpp"
@@ -65,10 +64,12 @@ void GetDocumentsCommand::execute(Args &args, Environment &env)
 	}
 
 	if(env.context().getEvaluationType() == XmlQueryContext::Eager) {
-		if(env.verbose()) cout << env.results()->size() << " documents found"
-				       << endl << endl;
+		if(env.verbose())
+			cout << (unsigned int)env.results()->size()
+			     << " documents found" << endl << endl;
 	} else {
-		if(env.verbose()) cout << "Lazy getDocuments completed" << endl << endl;
+		if(env.verbose())
+			cout << "Lazy getDocuments completed" << endl << endl;
 	}
 }
 
