@@ -1,20 +1,18 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: HelpCommand.hpp,v 1.6 2006/10/30 17:46:08 bostic Exp $
 //
 
 #ifndef __HELPCOMMAND_HPP
 #define	__HELPCOMMAND_HPP
 
-#include "dbxml/DbXml.hpp"
-#include <vector>
-#include <string>
-
 #include "ShellCommand.hpp"
 
+namespace DbXml
+{
+
 class HelpCommand : public ShellCommand
 {
 public:
@@ -32,4 +30,6 @@ private:
 	std::vector<ShellCommand*> &commands_;
 };
 
+}
+
 #endif
