@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: IndexLookupCommand.cpp,v 1.13 2006/10/30 17:46:08 bostic Exp $
 //
 
 #include "IndexLookupCommand.hpp"
@@ -97,10 +96,14 @@ void IndexLookupCommand::execute(Args &args, Environment &env)
 	env.results() = newResults;
 
 	if(env.context().getEvaluationType() == XmlQueryContext::Eager) {
-		if(env.verbose()) cout << env.results()->size() << " objects returned for eager index lookup '"
-				       << args[1] << "'" << endl << endl;
+		if(env.verbose())
+			cout << (unsigned int) env.results()->size()
+			     << " objects returned for eager index lookup '"
+			     << args[1] << "'" << endl << endl;
 	} else {
-		if(env.verbose()) cout << "Lazy index lookup '" << args[1] << "' completed" << endl << endl;
+		if(env.verbose())
+			cout << "Lazy index lookup '"
+			     << args[1] << "' completed" << endl << endl;
 	}
 }
 
