@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: QueryPlanTest.cpp,v 1.13 2006/10/30 17:46:13 bostic Exp $
 //
 
 // Generates query plans for a specified container query. The container
@@ -20,6 +19,7 @@
 #include <string>
 #include <fstream>
 #include <sstream>
+#include <iostream>
 
 // Xerces includes
 
@@ -78,7 +78,7 @@ QueryPlanTest::QueryPlanTest(TestEnvironment &env, const Iterator<std::string> &
           string s(args.next());
           // ensure we have a trailing slash
           // FIXME x-platform compatability?
-          unsigned int pos = s.rfind("/");
+          string::size_type pos = s.rfind("/");
           if(pos != s.length()-1) { //maybe on windows?
             pos = s.rfind("\\");
           }
