@@ -1,9 +1,8 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002,2006 Oracle.  All rights reserved.
+// Copyright (c) 2002,2009 Oracle.  All rights reserved.
 //
-// $Id: XQueryTest.cpp,v 1.33 2006/10/30 17:46:14 bostic Exp $
 //
 
 // tests the execution of XQuery 1.0 expressions
@@ -39,6 +38,10 @@ using namespace DbXmlTest;
 using namespace std;
 XERCES_CPP_NAMESPACE_USE
 
+#define CHECK_QUERY_PLANS 0
+#define UPDATE_QUERY_RESULTS 0
+#define UPDATE_QUERY_PLANS 0
+
 XQueryTest::XQueryTest(TestEnvironment &env, const Iterator<std::string> &args)
 	: UnitTest(env, "XQueryTest")
 {
@@ -58,7 +61,7 @@ void XQueryTest::execute()
 	     it != result.end(); it++) {
 		if((*it)->isNode()) {
 			// get query (name of file, in the data directory)
-			DOMNode *domnode = (DOMNode*)(*it)->getInterface(Node::gXerces);
+			DOMNode *domnode = (DOMNode*)(*it)->getInterface(XercesConfiguration::gXerces);
 			assert(domnode != 0);
 
 			Sequence result2 =
@@ -67,7 +70,7 @@ void XQueryTest::execute()
 			     it2 != result2.end(); it2++) {
 
 				if((*it2)->isNode()) {
-					DOMNode *dn = (DOMNode*)(*it2)->getInterface(Node::gXerces);
+					DOMNode *dn = (DOMNode*)(*it2)->getInterface(XercesConfiguration::gXerces);
 					assert(dn != 0);
 
 					string file, xquery;
@@ -189,12 +192,12 @@ static size_t parseQP(const std::string &qp, QPItem &result, size_t pos = 0) {
 	return pos;
 }
 
-static std::string makeQueryPlanComparable(const std::string &qp)
+std::string makeQueryPlanComparable(const std::string &qp)
 {
 	string result;
-	int qp_end = 0;
-	int qp_start = qp.find("QPlan>");
-	while(qp_start != -1) {
+	size_t qp_end = 0;
+	size_t qp_start = qp.find("QPlan>");
+	while(qp_start != (size_t)-1) {
 		result += qp.substr(qp_end, qp_start + 6 - qp_end);
 		QPItem qpi;
 		qp_end = parseQP(qp, qpi, qp_start + 6);
@@ -206,9 +209,6 @@ static std::string makeQueryPlanComparable(const std::string &qp)
 	return result;
 }
 
-#define UPDATE_QUERY_RESULTS 0
-#define UPDATE_QUERY_PLANS 0
-
 // Prepare and execute the specified XQuery - use both doc() and collection()
 // expressions as the input source (although only doc() results are verified).
 // Queries are run for for both document and value return types, using eager
@@ -230,8 +230,8 @@ void XQueryTest::doTest(const std::string &file, const std::string &xquery)
 	string doc_prefix = "dbxml:/" + container_name + "/";
 
 	string tmp(xquery);
-	u_int32_t pos(0);
-	size_t idx = tmp.find(doc_function, pos);
+	size_t pos(0);
+	string::size_type idx = tmp.find(doc_function, pos);
 	while( idx != string::npos) {
 		size_t pos_function = idx + doc_function.length();
 		tmp.replace(pos_function, 0, doc_prefix);
@@ -243,7 +243,6 @@ void XQueryTest::doTest(const std::string &file, const std::string &xquery)
 	// build map of return types
 	map<XmlQueryContext::ReturnType, string> returnTypes;
 	typedef map<XmlQueryContext::ReturnType, string>::const_iterator CI2;
-	returnTypes[XmlQueryContext::DeadValues] = "Dead Values";
 	returnTypes[XmlQueryContext::LiveValues] = "Live Values";
 
 	XmlQueryContext *xqc = env_.context();
@@ -277,11 +276,12 @@ void XQueryTest::doTest(const std::string &file, const std::string &xquery)
 
 			XmlResults *results = 0;
 			if(env_.transacted()) {
-				results = new XmlResults(expr->execute(*txn, *xqc));
+				results = new XmlResults(expr->execute(*txn, *xqc, DBXML_LAZY_DOCS | DBXML_DOCUMENT_PROJECTION));
 			} else {
-				results = new XmlResults(expr->execute(*xqc));
+				results = new XmlResults(expr->execute(*xqc, DBXML_LAZY_DOCS | DBXML_DOCUMENT_PROJECTION));
 			}
-			if(log) TEST_MSG(*log, "Executed expression; %1 hits found") << results->size();
+			if(log) TEST_MSG(*log, "Executed expression; %1 hits found")
+				<< (unsigned int)results->size();
 
 			// string together individual values to get actual result
 			static const string expected_extension(".out");
@@ -312,13 +312,12 @@ void XQueryTest::doTest(const std::string &file, const std::string &xquery)
 				if(log) ERROR_MSG(*log, "No expected result found");
 			}
 
+#if CHECK_QUERY_PLANS
 			string qp_extension;
+			if(env_.storageModel() == XmlContainer::WholedocContainer) {
+				qp_extension += ".dls";
+			}
 			if(env_.flags() & DBXML_INDEX_NODES) {
-				// The query plan is different if have enabled
-				// the indexing of nodes
-				if(env_.storageModel() == XmlContainer::WholedocContainer) {
-					qp_extension += ".dls";
-				}
 				qp_extension += ".index_nodes";
 			}
 			qp_extension += ".qplan";
@@ -346,8 +345,9 @@ void XQueryTest::doTest(const std::string &file, const std::string &xquery)
 				}
 			} else {
 				setResult(file, qp_extension, actual_qp);
-				if(log) ERROR_MSG(*log, "No query plan result found");
+				if(log) TEST_MSG(*log, "No query plan result found");
 			}
+#endif
 
 			if(env_.transacted()) {
 				env_.commitTransaction(*txn);
@@ -368,7 +368,7 @@ bool XQueryTest::getResult(const std::string &file, const std::string &new_exten
 	// ".xquery"  -> ".out" for file name
 	static const string old_extension(".xquery");
 	string tmp(file);
-	size_t idx = tmp.rfind(old_extension);
+	string::size_type idx = tmp.rfind(old_extension);
 	if(idx != string::npos) {
 		tmp.replace(idx, old_extension.length(), new_extension);
 	}
@@ -380,7 +380,7 @@ bool XQueryTest::getResult(const std::string &file, const std::string &new_exten
 		unsigned char buffer[buffer_size];
 		do {
 			size_t len = fread(buffer, sizeof(unsigned char), buffer_size, fd);
-			oss.write((char*)buffer, len);
+			oss.write((char*)buffer, (unsigned int)len);
 		} while(!feof(fd));
 		fclose(fd);
 		expected.assign(oss.str());
@@ -396,7 +396,7 @@ bool XQueryTest::setResult(const std::string &file, const std::string &new_exten
 	// ".xquery"  -> ".out" for file name
 	static const string old_extension(".xquery");
 	string tmp(file);
-	size_t idx = tmp.rfind(old_extension);
+	string::size_type idx = tmp.rfind(old_extension);
 	if(idx != string::npos) {
 		tmp.replace(idx, old_extension.length(), new_extension);
 	}
