@@ -1,28 +1,31 @@
 /*
- * Copyright (c) 2001-2006
+ * Copyright (c) 2001-2008
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2006
- *     Progress Software Corporation. All rights reserved.
- * Copyright (c) 2004-2006
+ * Copyright (c) 2004-2008
  *     Oracle. All rights reserved.
  *
- * See the file LICENSE for redistribution information.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * $Id: XQQuery.cpp,v 1.31.2.2 2007/01/03 19:19:51 jpcs Exp $
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ *
+ * $Id: XQQuery.cpp 723 2009-01-14 15:01:05Z lauren_foutz $
  */
 
-//////////////////////////////////////////////////////////////////////
-// XQQuery.cpp: implementation of the XQQuery class.
-//////////////////////////////////////////////////////////////////////
-
 #include <xqilla/simple-api/XQQuery.hpp>
 #include <xqilla/framework/XQillaExport.hpp>
 #include <xqilla/simple-api/XQilla.hpp>
 #include <xqilla/context/DynamicContext.hpp>
-#include <xqilla/context/impl/XQContextImpl.hpp>
-#include <xqilla/context/XQDebugCallback.hpp>
 #include <xqilla/context/VariableStore.hpp>
 #include <xqilla/context/VariableTypeStore.hpp>
+#include <xqilla/context/ContextHelpers.hpp>
 #include <xqilla/functions/FunctionLookup.hpp>
 #include <xqilla/functions/XQUserFunction.hpp>
 #include <xqilla/exceptions/XQException.hpp>
@@ -31,13 +34,15 @@
 #include <xqilla/items/Node.hpp>
 #include <xqilla/ast/XQGlobalVariable.hpp>
 #include <xqilla/ast/XQSequence.hpp>
-#include <xqilla/ast/StaticResolutionContext.hpp>
+#include <xqilla/ast/StaticAnalysis.hpp>
 #include <xqilla/runtime/Result.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
 #include <xqilla/utils/XPath2NSUtils.hpp>
 #include <xqilla/utils/PrintAST.hpp>
+#include <xqilla/events/EventHandler.hpp>
+#include <xqilla/optimizer/StaticTyper.hpp>
+#include <xqilla/functions/XQillaFunction.hpp>
 
-#include <xercesc/dom/DOM.hpp>
 #include <xercesc/util/XMLUni.hpp>
 #include <xercesc/util/XMLURL.hpp>
 #include <xercesc/util/XMLResourceIdentifier.hpp>
@@ -55,10 +60,6 @@ using namespace std;
 XERCES_CPP_NAMESPACE_USE
 #endif
 
-//////////////////////////////////////////////////////////////////////
-// Construction/Destruction
-//////////////////////////////////////////////////////////////////////
-
 XQQuery::XQQuery(const XMLCh* queryText, DynamicContext *context, bool contextOwned, MemoryManager *memMgr)
   : m_memMgr(memMgr),
     m_context(context),
@@ -80,8 +81,8 @@ XQQuery::~XQQuery()
       it != m_importedModules.end(); ++it) {
     delete *it;
   }
-	if(m_contextOwned)
-		delete m_context;
+  if(m_contextOwned)
+    delete m_context;
 }
 
 DynamicContext *XQQuery::createDynamicContext(MemoryManager *memMgr) const
@@ -91,14 +92,88 @@ DynamicContext *XQQuery::createDynamicContext(MemoryManager *memMgr) const
 
 Result XQQuery::execute(DynamicContext* context) const
 {
-  if(context->getDebugCallback()) {
-    return new DebugResult(this, context);
+  return new QueryResult(this);
+}
+
+Result XQQuery::execute(const Item::Ptr &contextItem, DynamicContext *context) const
+{
+  context->setContextItem(contextItem);
+  return execute(context);
+}
+
+Result XQQuery::execute(const XMLCh *templateQName, DynamicContext *context) const
+{
+  Item::Ptr value = context->getItemFactory()->createUntypedAtomic(templateQName, context);
+  context->setExternalVariable(XQillaFunction::XMLChFunctionURI, X("name"), value);
+  return execute(context);
+}
+
+void XQQuery::executeProlog(DynamicContext *context) const
+{
+  try {
+    // Execute the imported modules
+    for(ImportedModules::const_iterator modIt = m_importedModules.begin();
+        modIt != m_importedModules.end(); ++modIt) {
+
+      // Derive the module's execution context from it's static context
+      AutoDelete<DynamicContext> moduleCtx(context->createModuleDynamicContext((*modIt)->getStaticContext(),
+                                                                               context->getMemoryManager()));
+      (*modIt)->executeProlog(moduleCtx);
+
+      // Copy the module's imported variables into our context
+      for(GlobalVariables::const_iterator varIt = (*modIt)->m_userDefVars.begin();
+          varIt != (*modIt)->m_userDefVars.end(); ++varIt) {
+        Result value = moduleCtx->getGlobalVariableStore()->
+          getVar((*varIt)->getVariableURI(), (*varIt)->getVariableLocalName());
+        context->setExternalVariable((*varIt)->getVariableURI(), (*varIt)->getVariableLocalName(),
+                                     value->toSequence(context));
+      }
+    }
+
+    // define global variables
+    for(GlobalVariables::const_iterator it3 = m_userDefVars.begin(); it3 != m_userDefVars.end(); ++it3)
+      (*it3)->execute(context);
   }
-  else {
-    return new QueryResult(this);
+  catch(XQException& e) {
+    if(e.getXQueryLine() == 0) {
+      e.setXQueryPosition(m_szCurrentFile, 1, 1);
+    }
+    throw e;
   }
 }
 
+void XQQuery::execute(EventHandler *events, DynamicContext* context) const
+{
+  executeProlog(context);
+
+  if(m_query != NULL) {
+    try {
+      // execute the query body
+      m_query->generateAndTailCall(events, context, true, true);
+      events->endEvent();
+    }
+    catch(XQException& e) {
+      if(e.getXQueryLine() == 0) {
+        e.setXQueryPosition(m_szCurrentFile, 1, 1);
+      }
+      throw e;
+    }
+  }
+}
+
+void XQQuery::execute(EventHandler *events, const Item::Ptr &contextItem, DynamicContext *context) const
+{
+  context->setContextItem(contextItem);
+  execute(events, context);
+}
+
+void XQQuery::execute(EventHandler *events, const XMLCh *templateQName, DynamicContext *context) const
+{
+  Item::Ptr value = context->getItemFactory()->createUntypedAtomic(templateQName, context);
+  context->setExternalVariable(XQillaFunction::XMLChFunctionURI, X("name"), value);
+  execute(events, context);
+}
+
 void XQQuery::staticResolution(StaticContext *context)
 {
   if(context == 0) context = m_context;
@@ -107,9 +182,17 @@ void XQQuery::staticResolution(StaticContext *context)
   // which gives them the static type they were defined with
   UserFunctions::iterator i;
   for(i = m_userDefFns.begin(); i != m_userDefFns.end(); ++i) {
-    if(getIsLibraryModule() && !XERCES_CPP_NAMESPACE::XMLString::equals((*i)->getURI(), getModuleTargetNamespace()))
-      XQThrow3(StaticErrorException,X("XQQuery::staticResolution"), X("Every function in a module must be in the module namespace [err:XQST0048]."), *i);
     (*i)->staticResolutionStage1(context);
+
+    if(getIsLibraryModule() && !(*i)->isTemplate() && !XERCES_CPP_NAMESPACE::XMLString::equals((*i)->getURI(), getModuleTargetNamespace()))
+      XQThrow3(StaticErrorException,X("XQQuery::staticResolution"), X("Every function in a module must be in the module namespace [err:XQST0048]."), *i);
+
+    if((*i)->isTemplate()) {
+      context->addTemplate(*i);
+    }
+    else if((*i)->getName()) {
+      context->addCustomFunction(*i);
+    }
   }
 
   // Define types for the imported variables
@@ -119,21 +202,20 @@ void XQQuery::staticResolution(StaticContext *context)
         varIt != (*modIt)->m_userDefVars.end(); ++varIt) {
       context->getVariableTypeStore()->
         declareGlobalVar((*varIt)->getVariableURI(), (*varIt)->getVariableLocalName(),
-                         (*varIt)->getStaticResolutionContext());
+                         (*varIt)->getStaticAnalysis());
     }
   }
 
   // Run staticResolution on the global variables
-  if(!m_userDefVars.empty())
-  {
+  if(!m_userDefVars.empty()) {
     GlobalVariables::iterator itVar, itVar2;
-    // declare all the global variables with a special StaticResolutionContext, in order to recognize 'variable is defined later' errors 
+    // declare all the global variables with a special StaticAnalysis, in order to recognize 'variable is defined later' errors 
     // instead of more generic 'variable not found'
     // In order to catch references to not yet defined variables (but when no recursion happens) we also create a scope where we undefine
     // the rest of the variables (once we enter in a function call, the scope will disappear and the forward references to the global variables 
     // will happear)
-    StaticResolutionContext forwardRef(context->getMemoryManager());
-    forwardRef.setProperties(StaticResolutionContext::FORWARDREF);
+    StaticAnalysis forwardRef(context->getMemoryManager());
+    forwardRef.setProperties(StaticAnalysis::FORWARDREF);
     for(itVar = m_userDefVars.begin(); itVar != m_userDefVars.end(); ++itVar) {
       const XMLCh* varName=(*itVar)->getVariableName();
       const XMLCh* prefix=XPath2NSUtils::getPrefix(varName, context->getMemoryManager());
@@ -143,8 +225,8 @@ void XQQuery::staticResolution(StaticContext *context)
       const XMLCh* name= XPath2NSUtils::getLocalName(varName);
       context->getVariableTypeStore()->declareGlobalVar(uri, name, forwardRef);
     }
-    StaticResolutionContext forwardRef2(context->getMemoryManager());
-    forwardRef2.setProperties(StaticResolutionContext::UNDEFINEDVAR);
+    StaticAnalysis forwardRef2(context->getMemoryManager());
+    forwardRef2.setProperties(StaticAnalysis::UNDEFINEDVAR);
     for(itVar = m_userDefVars.begin(); itVar != m_userDefVars.end(); ++itVar) {
       context->getVariableTypeStore()->addLogicalBlockScope();
       for(itVar2 = itVar; itVar2 != m_userDefVars.end(); ++itVar2) {
@@ -202,26 +284,17 @@ void XQQuery::staticResolution(StaticContext *context)
   // Run staticResolutionStage2 on the user defined functions,
   // which statically resolves their function bodies
   for(i = m_userDefFns.begin(); i != m_userDefFns.end(); ++i) {
-      try {
-        (*i)->staticResolutionStage2(context);
-      } catch(XQException& e) {
-        XMLBuffer errMsg(1023, context->getMemoryManager());
-        errMsg.set(X("Error while running static resolution on user-defined function {"));
-        errMsg.append((*i)->getURI());
-        errMsg.append(X("}"));
-        errMsg.append((*i)->getName());
-        errMsg.append(X(": "));
-        errMsg.append(e.getError());
-        XQThrow3(StaticErrorException,X("XQQuery::staticResolution"), errMsg.getRawBuffer(), *i);
-      }
+    (*i)->staticResolutionStage2(context);
   }
-  if(m_query) m_query = m_query->staticResolution(context);
 
-  staticTyping(context);
+  // Run static resolution on the query body
+  if(m_query) m_query = m_query->staticResolution(context);
 }
 
-void XQQuery::staticTyping(StaticContext *context)
+void XQQuery::staticTyping(StaticContext *context, StaticTyper *styper)
 {
+  StaticTyper defaultTyper;
+  if(styper == 0) styper = &defaultTyper;
   if(context == 0) context = m_context;
 
   VariableTypeStore* varStore = context->getVariableTypeStore();
@@ -232,23 +305,22 @@ void XQQuery::staticTyping(StaticContext *context)
     for(GlobalVariables::const_iterator varIt = (*modIt)->m_userDefVars.begin();
         varIt != (*modIt)->m_userDefVars.end(); ++varIt) {
       varStore->declareGlobalVar((*varIt)->getVariableURI(), (*varIt)->getVariableLocalName(),
-                                 (*varIt)->getStaticResolutionContext());
+                                 (*varIt)->getStaticAnalysis());
     }
   }
 
   // Run staticTyping on the global variables
-  if(!m_userDefVars.empty())
-  {
-    // declare all the global variables with a special StaticResolutionContext, in order to recognize 'variable is defined
+  if(!m_userDefVars.empty()) {
+    // declare all the global variables with a special StaticAnalysis, in order to recognize 'variable is defined
     // later' errors instead of more generic 'variable not found'. In order to catch references to not yet defined
     // variables (but when no recursion happens) we also create a scope where we undefine the rest of the variables (once
     // we enter in a function call, the scope will disappear and the forward references to the global variables will
     // appear).
-    StaticResolutionContext forwardRef(context->getMemoryManager());
-    forwardRef.setProperties(StaticResolutionContext::FORWARDREF);
+    StaticAnalysis forwardRef(context->getMemoryManager());
+    forwardRef.setProperties(StaticAnalysis::FORWARDREF);
 
-    StaticResolutionContext undefinedVar(context->getMemoryManager());
-    undefinedVar.setProperties(StaticResolutionContext::UNDEFINEDVAR);
+    StaticAnalysis undefinedVar(context->getMemoryManager());
+    undefinedVar.setProperties(StaticAnalysis::UNDEFINEDVAR);
 
     GlobalVariables::iterator itVar, itVar2;
     for(itVar = m_userDefVars.begin(); itVar != m_userDefVars.end(); ++itVar) {
@@ -262,29 +334,24 @@ void XQQuery::staticTyping(StaticContext *context)
         varStore->declareVar((*itVar2)->getVariableURI(), (*itVar2)->getVariableLocalName(),
                              undefinedVar);
       }
-      (*itVar)->staticTyping(context);
+      (*itVar)->staticTyping(context, styper);
       varStore->removeScope();
     }
   }
 
   // Run staticTyping on the user defined functions,
   // which calculates a better type for them
-  UserFunctions::iterator i;
+  UserFunctions::iterator i, j;
   for(i = m_userDefFns.begin(); i != m_userDefFns.end(); ++i) {
-      try {
-        (*i)->staticTyping(context);
-      } catch(XQException& e) {
-        XMLBuffer errMsg(1023, context->getMemoryManager());
-        errMsg.set(X("Error while calculating static type of user-defined function {"));
-        errMsg.append((*i)->getURI());
-        errMsg.append(X("}"));
-        errMsg.append((*i)->getName());
-        errMsg.append(X(": "));
-        errMsg.append(e.getError());
-        XQThrow3(StaticErrorException,X("XQQuery::staticTyping"), errMsg.getRawBuffer(), *i);
-      }
+    for(j = m_userDefFns.begin(); j != m_userDefFns.end(); ++j) {
+      (*j)->resetStaticTypingOnce();
+    }
+
+    (*i)->staticTypingOnce(context, styper);
   }
-  if(m_query) m_query = m_query->staticTyping(context);
+
+  // Run staticTyping on the query body
+  if(m_query) m_query = m_query->staticTyping(context, styper);
 }
 
 std::string XQQuery::getQueryPlan() const
@@ -335,61 +402,58 @@ const XMLCh* XQQuery::getModuleTargetNamespace() const
 class LoopDetector : public XMLEntityResolver
 {
 public:
-    LoopDetector(XMLEntityResolver* pParent, const XMLCh* myModuleURI) 
-    {
-        m_pParentResolver=pParent;
-        m_PreviousModuleNamespace=myModuleURI;
-    }
+  LoopDetector(XMLEntityResolver* pParent, const XMLCh* myModuleURI, const LocationInfo *location) 
+  {
+    m_pParentResolver = pParent;
+    m_PreviousModuleNamespace = myModuleURI;
+    m_location = location;
+  }
 
-    virtual InputSource* resolveEntity(XMLResourceIdentifier* resourceIdentifier)
-    {
-        if(resourceIdentifier->getResourceIdentifierType()==XMLResourceIdentifier::UnKnown &&
-            XPath2Utils::equals(resourceIdentifier->getNameSpace(), m_PreviousModuleNamespace))
-        {
-            XMLBuffer buf;
-            buf.set(X("The graph of module imports contains a cycle for namespace '"));
-            buf.append(resourceIdentifier->getNameSpace());
-            buf.append(X("' [err:XQST0073]"));
-            XQThrow2(StaticErrorException, X("XQQuery::ImportModule"), buf.getRawBuffer());
-        }
-        if(m_pParentResolver)
-            return m_pParentResolver->resolveEntity(resourceIdentifier);
-        return NULL;
+  virtual InputSource *resolveEntity(XMLResourceIdentifier* resourceIdentifier)
+  {
+    if(resourceIdentifier->getResourceIdentifierType() == XMLResourceIdentifier::UnKnown &&
+       XPath2Utils::equals(resourceIdentifier->getNameSpace(), m_PreviousModuleNamespace)) {
+      XMLBuffer buf;
+      buf.set(X("The graph of module imports contains a cycle for namespace '"));
+      buf.append(resourceIdentifier->getNameSpace());
+      buf.append(X("' [err:XQST0073]"));
+      XQThrow3(StaticErrorException, X("LoopDetector::resolveEntity"), buf.getRawBuffer(), m_location);
     }
+    if(m_pParentResolver)
+      return m_pParentResolver->resolveEntity(resourceIdentifier);
+    return NULL;
+  }
 
 protected:
-    XMLEntityResolver* m_pParentResolver;
-    const XMLCh* m_PreviousModuleNamespace;
+  XMLEntityResolver* m_pParentResolver;
+  const XMLCh* m_PreviousModuleNamespace;
+  const LocationInfo *m_location;
 };
 
-void XQQuery::importModule(XQilla::Language language, const XMLCh* szUri,
-                           VectorOfStrings* locations, StaticContext* context)
+void XQQuery::importModule(const XMLCh* szUri, VectorOfStrings* locations, StaticContext* context, const LocationInfo *location)
 {
   for(ImportedModules::iterator modIt = m_importedModules.begin();
       modIt != m_importedModules.end(); ++modIt) {
-      if(XPath2Utils::equals((*modIt)->getModuleTargetNamespace(),szUri))
-      {
-          XMLBuffer buf(1023,context->getMemoryManager());
-          buf.set(X("Module for namespace '"));
-          buf.append(szUri);
-          buf.append(X("' has already been imported [err:XQST0047]"));
-          XQThrow2(StaticErrorException, X("XQQuery::ImportModule"), buf.getRawBuffer());
-      }
+    if(XPath2Utils::equals((*modIt)->getModuleTargetNamespace(),szUri)) {
+      XMLBuffer buf(1023,context->getMemoryManager());
+      buf.set(X("Module for namespace '"));
+      buf.append(szUri);
+      buf.append(X("' has already been imported [err:XQST0047]"));
+      XQThrow3(StaticErrorException, X("XQQuery::ImportModule"), buf.getRawBuffer(), location);
+    }
   }
   if(locations==NULL)
     locations=context->resolveModuleURI(szUri);
-  if(locations==NULL || locations->empty())
-  {
+  if(locations==NULL || locations->empty()) {
     XMLBuffer buf(1023,context->getMemoryManager());
     buf.set(X("Cannot locate module for namespace "));
     buf.append(szUri);
     buf.append(X(" without the 'at <location>' keyword [err:XQST0059]"));
-    XQThrow2(StaticErrorException,X("XQQuery::ImportModule"), buf.getRawBuffer());
+    XQThrow3(StaticErrorException,X("XQQuery::ImportModule"), buf.getRawBuffer(), location);
   }
 
   bool bFound=false;
-  for(VectorOfStrings::iterator it=locations->begin();it!=locations->end();it++)
-  {
+  for(VectorOfStrings::iterator it=locations->begin();it!=locations->end();++it) {
     InputSource* srcToUse = 0;
     if (context->getDocumentCache()->getXMLEntityResolver()){
       XMLResourceIdentifier resourceIdentifier(XMLResourceIdentifier::UnKnown,
@@ -398,8 +462,7 @@ void XQQuery::importModule(XQilla::Language language, const XMLCh* szUri,
       srcToUse = context->getDocumentCache()->getXMLEntityResolver()->resolveEntity(&resourceIdentifier);
     }
 
-    if(srcToUse==0)
-    {
+    if(srcToUse==0) {
       try {
         XMLURL urlTmp(context->getBaseURI(), *it);
         if (urlTmp.isRelative()) {
@@ -422,32 +485,38 @@ void XQQuery::importModule(XQilla::Language language, const XMLCh* szUri,
     }
     Janitor<InputSource> janIS(srcToUse);
 
-    DynamicContext* moduleCtx = context->createModuleContext();
-    moduleCtx->setBaseURI(srcToUse->getSystemId());
-    LoopDetector loopDetector(context->getXMLEntityResolver(), szUri);
-    moduleCtx->setXMLEntityResolver(&loopDetector);
-    moduleCtx->setModuleResolver(context->getModuleResolver());
+    AutoDelete<DynamicContext> ctxGuard(context->createModuleContext());
+    DynamicContext *moduleCtx = ctxGuard.get();
 
-    XQQuery* pParsedQuery = XQilla::parse(*srcToUse, language, moduleCtx);
+    moduleCtx->setBaseURI(srcToUse->getSystemId());
+    LoopDetector loopDetector(context->getXMLEntityResolver(), szUri, location);
+    moduleCtx->setXMLEntityResolver(&loopDetector);
+
+    AutoDelete<XQQuery> pParsedQuery(XQilla::parse(*srcToUse, ctxGuard.adopt(), XQilla::NO_OPTIMIZATION));
 
     if(!pParsedQuery->getIsLibraryModule()) {
       XMLBuffer buf(1023,context->getMemoryManager());
       buf.set(X("The module at "));
       buf.append(srcToUse->getSystemId());
       buf.append(X(" is not a module"));
-      XQThrow2(StaticErrorException, X("XQQuery::ImportModule"), buf.getRawBuffer());
+      XQThrow3(StaticErrorException, X("XQQuery::ImportModule"), buf.getRawBuffer(), location);
     }
     if(!XERCES_CPP_NAMESPACE::XMLString::equals(szUri,pParsedQuery->getModuleTargetNamespace())) {
       XMLBuffer buf(1023,context->getMemoryManager());
       buf.set(X("The module at "));
       buf.append(srcToUse->getSystemId());
       buf.append(X(" specifies a different namespace [err:XQST0059]"));
-      XQThrow2(StaticErrorException, X("XQQuery::ImportModule"), buf.getRawBuffer());
+      XQThrow3(StaticErrorException, X("XQQuery::ImportModule"), buf.getRawBuffer(), location);
     }
     // now move the variable declarations and the function definitions into my context
     for(UserFunctions::iterator itFn = pParsedQuery->m_userDefFns.begin(); itFn != pParsedQuery->m_userDefFns.end(); ++itFn) {
       (*itFn)->setModuleDocumentCache(const_cast<DocumentCache*>(moduleCtx->getDocumentCache()));
-      context->addCustomFunction(*itFn);
+      if((*itFn)->isTemplate()) {
+        context->addTemplate(*itFn);
+      }
+      else if((*itFn)->getName()) {
+        context->addCustomFunction(*itFn);
+      }
     }
     for(GlobalVariables::iterator itVar = pParsedQuery->m_userDefVars.begin(); itVar != pParsedQuery->m_userDefVars.end(); ++itVar) {
       for(ImportedModules::const_iterator modIt = m_importedModules.begin();
@@ -468,9 +537,8 @@ void XQQuery::importModule(XQilla::Language language, const XMLCh* szUri,
       }
     }
 
-    moduleCtx->setXMLEntityResolver(NULL);
-    moduleCtx->setModuleResolver(NULL);
-    m_importedModules.push_back(pParsedQuery);
+    moduleCtx->setXMLEntityResolver(context->getXMLEntityResolver());
+    m_importedModules.push_back(pParsedQuery.adopt());
 
     bFound=true;
   }
@@ -480,7 +548,7 @@ void XQQuery::importModule(XQilla::Language language, const XMLCh* szUri,
     buf.set(X("Cannot locate the module for namespace \""));
     buf.append(szUri);
     buf.append(X("\" [err:XQST0059]"));
-    XQThrow2(StaticErrorException,X("XQQuery::ImportModule"), buf.getRawBuffer());
+    XQThrow3(StaticErrorException,X("XQQuery::ImportModule"), buf.getRawBuffer(), location);
   }
 }
 
@@ -501,155 +569,21 @@ const XMLCh* XQQuery::getQueryText() const
 
 XQQuery::QueryResult::QueryResult(const XQQuery *query)
   : ResultImpl(query->getQueryBody()),
-    _query(query),
-    _parent(0),
-    _toDo(true)
-{
-}
-
-Item::Ptr XQQuery::QueryResult::next(DynamicContext *context)
-{
-  if(_toDo) {
-    _toDo = false;
-
-    // Execute the imported modules
-    for(ImportedModules::const_iterator modIt = _query->m_importedModules.begin();
-        modIt != _query->m_importedModules.end(); ++modIt) {
-
-      // Derive the module's execution context from it's static context
-      AutoDelete<DynamicContext> moduleCtx((*modIt)->createDynamicContext(context->getMemoryManager()));
-      moduleCtx->setMemoryManager(context->getMemoryManager());
-
-      (*modIt)->execute(moduleCtx)->next(moduleCtx);
-
-      // Copy the module's imported variables into our context
-      for(GlobalVariables::const_iterator varIt = (*modIt)->m_userDefVars.begin();
-          varIt != (*modIt)->m_userDefVars.end(); ++varIt) {
-        std::pair<bool, Sequence> value = moduleCtx->getVariableStore()->
-          getGlobalVar((*varIt)->getVariableURI(), (*varIt)->getVariableLocalName(), moduleCtx);
-        assert(value.first);
-        Sequence newSeq(value.second.getLength());
-        for(Sequence::iterator it=value.second.begin(); it!=value.second.end(); it++)
-          if((*it)->isNode())
-            newSeq.addItem(context->getItemFactory()->cloneNode((const Node::Ptr)(*it), context));
-          else
-            newSeq.addItem(*it);
-        context->getVariableStore()->
-          setGlobalVar((*varIt)->getVariableURI(), (*varIt)->getVariableLocalName(), newSeq, context);
-      }
-    }
-
-    // define global variables
-    for(GlobalVariables::const_iterator it3 = _query->m_userDefVars.begin(); it3 != _query->m_userDefVars.end(); ++it3)
-      (*it3)->execute(context);
-
-    // execute the query body
-    if(_query->getQueryBody() != NULL) {
-      _parent = _query->getQueryBody()->collapseTree(context);
-    }
-  }
-
-  Item::Ptr item = _parent->next(context);
-
-  if(item == NULLRCP) {
-    _parent = 0;
-  }
-
-  return item;
-}
-
-std::string XQQuery::QueryResult::asString(DynamicContext *context, int indent) const
-{
-  std::ostringstream oss;
-  std::string in(getIndent(indent));
-
-  oss << in << "<queryresult>" << std::endl;
-  oss << _parent->asString(context, indent + 1);
-  oss << in << "</queryresult>" << std::endl;
-
-  return oss.str();
-}
-
-XQQuery::DebugResult::DebugResult(const XQQuery *query, DynamicContext *context)
-  : LazySequenceResult(query->getQueryBody(), context),
     _query(query)
 {
 }
 
-void XQQuery::DebugResult::getResult(Sequence &toFill, DynamicContext *context) const
+Item::Ptr XQQuery::QueryResult::nextOrTail(Result &tail, DynamicContext *context)
 {
-  static XMLCh szMain[]= { chLatin_M, chLatin_a, chLatin_i, chLatin_n, chNull };
+  _query->executeProlog(context);
 
-  if(context->getDebugCallback()) {
-    context->getDebugCallback()->NotifyQueryBegin(context, _query->getQueryText());
-    context->getDebugCallback()->EnterFunction(context, _query->getFile(), szMain, 0, 0);
+  if(_query->getQueryBody() != NULL) {
+    // No closure needed here
+    tail = _query->getQueryBody()->createResult(context);
   }
-
-  try
-  {
-    // Execute the imported modules
-    for(ImportedModules::const_iterator modIt = _query->m_importedModules.begin();
-        modIt != _query->m_importedModules.end(); ++modIt) {
-
-      // Derive the module's execution context from it's static context
-      AutoDelete<DynamicContext> moduleCtx((*modIt)->createDynamicContext(context->getMemoryManager()));
-      moduleCtx->setMemoryManager(context->getMemoryManager());
-      // propagate debug settings
-      moduleCtx->setDebugCallback(context->getDebugCallback());
-
-      (*modIt)->execute(moduleCtx)->next(moduleCtx);
-
-      // Copy the module's imported variables into our context
-      for(GlobalVariables::const_iterator varIt = (*modIt)->m_userDefVars.begin();
-          varIt != (*modIt)->m_userDefVars.end(); ++varIt) {
-        std::pair<bool, Sequence> value = moduleCtx->getVariableStore()->
-          getGlobalVar((*varIt)->getVariableURI(), (*varIt)->getVariableLocalName(), moduleCtx);
-        assert(value.first);
-        Sequence newSeq(value.second.getLength());
-        for(Sequence::iterator it=value.second.begin(); it!=value.second.end(); it++)
-          if((*it)->isNode())
-            newSeq.addItem(context->getItemFactory()->cloneNode((const Node::Ptr)(*it), context));
-          else
-            newSeq.addItem(*it);
-        context->getVariableStore()->
-          setGlobalVar((*varIt)->getVariableURI(), (*varIt)->getVariableLocalName(), newSeq, context);
-      }
-    }
-
-    // define global variables
-    for(GlobalVariables::const_iterator it3 = _query->m_userDefVars.begin(); it3 != _query->m_userDefVars.end(); ++it3)
-      (*it3)->execute(context);
-
-    // execute the query body
-    if(_query->getQueryBody() != NULL) {
-      toFill = _query->getQueryBody()->collapseTree(context)->toSequence(context);
-    }
-  }
-  catch(XQException& e)
-  {
-    if(!e.isErrorReported()) {
-      if(context->getDebugCallback() && context->isDebuggingEnabled()) 
-        context->getDebugCallback()->ReportFirstError(context, e.getError(), _query->getFile(), 0);
-      e.setErrorReported();
-      if(e.getXQueryLine() == 0) {
-        e.setXQueryPosition(_query->getFile(), 1, 1);
-      }
-    }
-    throw e;
-  }
-
-  if(context->getDebugCallback()) {
-    context->getDebugCallback()->ExitFunction(context, szMain);
-    context->getDebugCallback()->NotifyQueryEnd(context, toFill);
+  else {
+    tail = 0;
   }
+  return 0;
 }
 
-std::string XQQuery::DebugResult::asString(DynamicContext *context, int indent) const
-{
-  std::ostringstream oss;
-  std::string in(getIndent(indent));
-
-  oss << in << "<debugresult/>" << std::endl;
-
-  return oss.str();
-}
